//
// Project:	Lhogho
// File:	asm.h
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-06-19	- File created
//	 2011-10-14 - Support for RUNMACRO
//   2012-01-02 - Command-line options are translatable
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_ASM_H
#define LHOGHO_ASM_H



#ifdef ADVANCED

// terminal coloring and disassembly are
// available only in ADVANCED mode

//#define TERM_INSTR  outter(TEXT("\033[0m\033[1m"),-1)		///< instructions
//#define TERM_INFO   outter(TEXT("\033[0m\033[2;32m"),-1)	///< infos
//#define TERM_REMARK outter(TEXT("\033[2;30m"),-1)		///< remarks
#define TERM_INSTR  	///< instructions
#define TERM_INFO   	///< infos
#define TERM_REMARK 	///< remarks

#ifdef UNICODE_CHARS
  #define INSTR(format,...)		disasm(ctx,1,L##format,##__VA_ARGS__)
  #define INFO(format,...)		disasm(ctx,2,L##format,##__VA_ARGS__)
  #define REM(format,...)		disasm(ctx,3,L##format,##__VA_ARGS__)
  #define LABEL(format,...)		disasm(ctx,4,L##format,##__VA_ARGS__)
#else
  #define INSTR(format,...)		disasm(ctx,1,format,##__VA_ARGS__)
  #define INFO(format,...)		disasm(ctx,2,format,##__VA_ARGS__)
  #define REM(format,...)		disasm(ctx,3,format,##__VA_ARGS__)
  #define LABEL(format,...)		disasm(ctx,4,format,##__VA_ARGS__)
#endif

extern void disasm( context_t* ctx, int mode, chars_t format, ... );
#endif //ADVANCED


#ifndef ADVANCED
  #define INSTR(format,...)	
  #define INFO(format,...)
  #define REM(format,...)
  #define LABEL(format,...)
#endif //if ADVANCED is undefined

#define ASM		(OPTION_ASSEMBLER && ctx->generate)

extern void asm_label( context_t* ctx, chars_t name );
extern void asm_prologue( context_t* ctx, atom_t func, int check );
extern void asm_epilogue( context_t* ctx, atom_t func, int is_macro );
extern void asm_preepilogue( context_t* ctx );
extern void asm_external_function( context_t* ctx, atom_t func );
extern void asm_internal_function( context_t* ctx, int static_link, atom_t func );
extern void asm_push_atom( context_t* ctx, atom_t atom );
extern void asm_call_atom( context_t* ctx, atom_t atom, int params );
extern void asm_pop_atom( context_t* ctx );
extern void asm_pop_result( context_t* ctx );
extern void asm_result_func( context_t* ctx, atom_t source );
extern void asm_result_unknown( context_t* ctx, atom_t source );
extern void asm_result_proc( context_t* ctx, atom_t source );
extern void asm_push_result( context_t* ctx );
extern void asm_pop_dummy( context_t* ctx );
extern void asm_make_direct( context_t* ctx, atom_t var, atom_t source );
extern void asm_reference( context_t* ctx, atom_t var, atom_t source, int check );
extern void asm_push_value_addr( context_t* ctx, atom_t var );
extern void asm_empty_body( context_t* ctx );
extern void asm_output( context_t* ctx, atom_t source, int true_output );
extern void asm_stop( context_t* ctx, atom_t source );
extern void asm_boolean( context_t* ctx, atom_t source );
extern int  asm_if_prologue( context_t* ctx );
extern int  asm_if_epilogue( context_t* ctx );
extern void asm_fix( context_t* ctx, int addr );
extern void asm_dump_source( context_t* ctx, atom_t source );

extern int  asm_repeat_prologue_const( context_t* ctx, int cnt );
extern int  asm_repeat_prologue_expr( context_t* ctx, atom_t source, int* branch2 );
extern void asm_repeat_epilogue( context_t* ctx, int branch, int branch2 );

extern int  asm_for_prologue( context_t* ctx, atom_t step_var, atom_t source, int* branch2 );
extern void asm_for_epilogue( context_t* ctx, int branch, int branch2 );

extern int  asm_forever_prologue( context_t* ctx );
extern void asm_forever_epilogue( context_t* ctx, int branch );

extern int  asm_while_prologue( context_t* ctx, int is_while, int is_do );
extern int  asm_while_inlogue(  context_t* ctx, atom_t source, int is_while );
extern void asm_while_epilogue( context_t* ctx, int loop_branch, int skip_branch, int is_while );

extern int asm_catch_prologue( context_t* ctx );
extern void asm_catch_epilogue( context_t* ctx );

extern void asm_push_mode( context_t* ctx, int mode );

extern void asm_push_frame( context_t* ctx );
extern void asm_pop_frame( context_t* ctx );

extern void asm_runtime_reference( context_t* ctx );
extern void asm_adjust_result( context_t* ctx );

extern void asm_goto( context_t* ctx, atom_t tag );
extern void asm_goto_prologue( context_t* ctx, atom_t source );
extern void asm_goto_epilogue( context_t* ctx, atom_t source );
extern int asm_test_prologue( context_t* ctx, int criteria );

extern void asm_run_epilogue( context_t* ctx );
extern void asm_runresult_epilogue( context_t* ctx );

extern void asm_int_3( context_t* ctx );
extern void asm_set_output_status( context_t* ctx, int n );
extern void asm_nop( context_t* ctx );

//extern void asm_enter_if_output( context_t* ctx );
//extern void asm_exit_if_output( context_t* ctx );

#endif  //LHOGHO_ASM_H
