//
// Project:	Lhogho
// File:	compiler.c
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-06-11	- file created
//   2007-06-19 - compile() renamed to compile_from_options()
//   2007-07-29 - changed parameters of init_compiler()
//   2010-06-25 - added inner_eof
//   2011-10-13 - support for Logo macros
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_COMPILER_H
#define LHOGHO_COMPILER_H

#include "globals.h"
#include "atoms.h"

typedef struct ctxnode { 
  //atom tags;		///< list of collected tags
  //int  repdepth;	///< number of nested REPEATs so far
  int	 size;		///< pointer within GENERATE for the current offset
  atom_t generate;	///< memory atom where code will be generated
  atom_t parent;	///< parent var for the context
  int    exit_addr;	///< address of exit label
} context_t; ///< structure holding compilation context


#define COMPILE_AS_FUNC		0 ///< compile and check result to be non-unbound
#define COMPILE_AS_PROC		1 ///< compile and check result to be unbound
#define COMPILE_AS_UNKNOWN	2 ///< compile and accept any result

/// The COMPILE_AS_*_MACRO are independent on the other COMPILE_AS_* symbols.

#define COMPILE_AS_NON_MACRO 0 ///< compile a function as a function
#define COMPILE_AS_MACRO 	 1 ///< compile a function as a macro

// ===========================================================
// A magic number used to identify embedded Lhogho sources
// ===========================================================

#define MAGIC_NUMBER		1980069381 ///< code for standard standalone executables
#define MAGIC_COMPILER_NUMBER	1761700097 ///< code for custom Lhogho compilers

extern int running_compiled_code;
extern int compiling_code;

extern void init_compiler( outter_t outter, inner_t inner, inner_eof_t inner_eof );
extern void finit_compiler( );
extern int compile_from_options( );
extern atom_t compile_to_file( );
extern atom_t compile_function( atom_t func, int mode, int is_macro );
extern atom_t compile_external_function( atom_t func );
extern atom_t compile_internal_function( atom_t func, int static_link );
extern int run_source( chars_t source );
extern int run_function( atom_t function );

#endif //LHOGHO_COMPILER_H
