//
// Project:	Lhogho
// File:	errors.h
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-06-01 - file created
//   2007-06-03 - errors renamed
//		- ERROR_CROWDED_SOURCES, ERROR_OS_ERROR
//		- ERROR_INCOMPATIBLE_REDEFINITION
//   2007-06-05 - definitions spread to where they belong
//   2007-06-09 - added dump_error()
//   2007-06-26 - ERROR_NOT_A_NUMBER
//   2007-07-01 - ERROR_UNUSED_VALUE
//		- ERROR_MISSING_VALUE
//   2007-09-09 - ERROR_DO_NOT_KNOW
//		- ERROR_UNKNOWN_VAR
//
//   2007-09-17 - ERROR_BOOLEAN_EXPECTED
//   2007-09-21 - ERROR_NOT_AN_INTEGER, ERROR_NOT_A_LIST
//   2007-10-06 - ERROR_NOT_A_WORD
//   2007-10-13 - ERROR_INCOMPATIBLE_DATA
//   2007-10-17 - ERROR_TOO_BIG_NUMBER, ERROR_TOO_SMALL_NUMBER
//   2007-11-04 - ERROR_NOT_A_VAR
//   2007-11-12 - EXIT_BY_BYE, EXIT_BY_THROW_TOPLEVEL, EXIT_BY_THROW_SYSTEM
//		- THROW
//   2007-11-15 - EXIT_BY_THROW_TAG
//   2008-01-05 - ERROR_VAR_HAS_NO_VALUE
//   2008-05-08 - ERROR_NOT_A_FUNCTION
//   2008-09-11 - ERROR_NOT_A_TYPE_NAME
//   2008-09-13 - ERROR_BAD_PROTOTYPE
//   2008-11-22 - ERROR_NOT_A_MEM
//   2011-12-26 - Error messages taken from string, stored to array of atoms
//   2012-01-20 - Added new_os_error_atom()
//   2012-02-02 - ERROR_FILE_NOT_OPENED
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_ERRORS_H
#define LHOGHO_ERRORS_H


#include "atoms.h"



//===================================================
/// \name Exit codes
/// See \ref errors for details
//===================================================
#define ERROR_INCOMPLETE_PAIR       1
#define ERROR_EMPTY_EXPRESSION      2
#define ERROR_CROWDED_EXPRESSION    3
#define ERROR_MISSING_LEFTS         4
#define ERROR_MISSING_RIGHTS        5
#define ERROR_EMPTY_TO_END          6
#define ERROR_MISSING_NAME          7
#define ERROR_DUPLICATE_INPUT       8
#define ERROR_UNKNOWN_OPTION        9
#define ERROR_CROWDED_SOURCES       10
#define ERROR_OS_ERROR              11
#define ERROR_INCOMPATIBLE_REDEFINITION	12
#define ERROR_NOT_A_NUMBER          13
#define ERROR_UNUSED_VALUE          14
#define ERROR_MISSING_VALUE         15
#define ERROR_DO_NOT_KNOW           16
#define ERROR_UNKNOWN_VAR           17
#define ERROR_BOOLEAN_EXPECTED      18
#define ERROR_NOT_AN_INTEGER        19
#define ERROR_NOT_A_LIST            20
#define ERROR_NOT_A_WORD            21
#define ERROR_INCOMPATIBLE_DATA     22
#define ERROR_TOO_BIG_NUMBER        23
#define ERROR_TOO_SMALL_NUMBER      24
#define ERROR_NOT_A_VAR             25
#define EXIT_BY_BYE		    26 // BYE
#define EXIT_BY_THROW_TOPLEVEL      27 // THROW "TOPLEVEL
#define EXIT_BY_THROW_SYSTEM        28 // THROW "SYSTEM
#define EXIT_BY_THROW_ERROR         29 // THROW "ERROR
#define EXIT_BY_THROW_USER_ERROR    30 // THROW "ERROR <value>
#define EXIT_BY_THROW_TAG           31 // THROW <tag>
#define EXIT_BY_THROW_TAG_VALUE     32 // THROW <tag> <value>
#define ERROR_NOT_A_TAG		    33
#define ERROR_NOT_A_USER_FUNCTION   34
#define ERROR_VAR_HAS_NO_VALUE      35
#define ERROR_NOT_A_FUNCTION	    36
#define ERROR_NOT_A_LIST_CONST      37
#define ERROR_NOT_A_WORD_CONST      38
#define ERROR_MISSING_FOR_LIMITS    39
#define ERROR_NOT_A_TYPE_NAME       40
#define ERROR_BAD_PROTOTYPE         41
#define ERROR_NOT_A_MEM		    42
#define ERROR_OUT_OF_MEM      43
#define ERROR_FILE_NOT_OPENED 44
#define ERROR_NOT_BLOCK_OR_DEF         45
//#define WARNING_UNSUPPORTED_COMPILER 43


#define FIRST_EXIT_CODE 26 ///< first EXIT_ code
#define LAST_EXIT_CODE	32 ///< last EXIT_ code
#define LAST_ERROR_CODE 45 ///< last (max) error or exit code

//===================================================
/// \name Error fields
/// These macros are used to access errors. Error code
/// is in \c ERRCODE(), error source is in \c ERRSRC(),
/// and the error position within the source is in \c
/// ERRPOS().
//===================================================
/// @{
#define ERRCODE(x)	((x)->unode.s[3])
#define ERRPOS(x)	((x)->unode.a[2])
#define ERRDATA(x)	((x)->unode.a[3])

#define IS_ERROR(x)	(ID(x)==ERROR_ID)
/// @}


#define FORMAT_ERR_CODE		TEXT("{ERR#%d")
#define FORMAT_ERR_POS		TEXT("@%d}")


extern atom_t error_texts[];
extern int last_os_error;

extern void init_errors( );
extern void finit_errors( );
extern atom_t new_error( uint_t code, atom_t data );
extern atom_t new_os_error( chars_t filename );
extern atom_t new_os_error_atom( atom_t filename );
extern atom_t new_parse_error( uint_t code, int position, atom_t source );

extern void add_error_source( atom_t error, atom_t source );
extern int  get_error_position( atom_t data, chars_t* source );

extern void delete_error( atom_t a );
extern void dump_error( atom_t a, int level );
extern void clear_all_errors(  );

#endif //LHOGHO_ERRORS_H
