//
// Project:	Lhogho
// File:	external.c
//
// Copyright (C) 2008 P.Boytchev
//
// Revision history:
//   2008-09-14	- file created
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_EXTERNAL_H
#define LHOGHO_EXTERNAL_H

#include "globals.h"
#include "atoms.h"
#include "lists.h"
#include "numbers.h"
#include "errors.h"
#include "external.h"

#define C_TYPE_UNKNOWN    0       ///< unknown C-type
#define C_TYPE_STRUCT     1       ///< struct C-type
#define C_TYPE_SIGNED     2       ///< signed integer C-type
#define C_TYPE_UNSIGNED   3       ///< unsigned integer C-type
#define C_TYPE_FLOAT      4       ///< float or double C-type
#define C_TYPE_VOID       5       ///< void C-type
#define C_TYPE_POINTER    6       ///< pointer C-type
#define C_TYPE_STRING     7       ///< string C-type
#define C_TYPE_ATOM       8       ///< Lhogho atom


typedef struct typeid_rec {
  chars_t name;           ///< type identifier
  int     size;           ///< size in bits
  int     class;          ///< corresponding c-type class
  fn      c_to_atom;      ///< function converting C->Atom
  fn      atom_to_c;      ///< function converting Atom->C
  chars_t c_to_atom_name; ///< name of C->Atom function
  chars_t atom_to_c_name; ///< name ot Atom->C function
} typeid_t;               ///< description structure of a C-type identifiers
extern typeid_t c_types[];

extern int get_c_type( int static_link, atom_t parent, atom_t type );
extern atom_t traverse_pack( int static_link, atom_t parent, atom_t prototype, atom_t protodata, char* ptr, int mode );

#endif //LHOGHO_EXTERNAL_H
