//
// Project:	Lhogho
// File:	lists.h
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-06-09	- file created from atoms.h
//   2007-08-13 - support for PRINTDEPTHLIMIT, PRINTWIDTHLIMIT
//   2007-09-21 - list_copy_n, list_copy_but_last, list_length, get_at_list
//   2008-01-17 - extended list atoms
//   2008-08-14 - added FLAG_WAS_LIST_CONST
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#ifndef LHOGHO_LISTS_H
#define LHOGHO_LISTS_H

#include "globals.h"


//===================================================
/// \name Flags for lists
/// These bitmask flags determine whether a token is
/// the first on a line (\c FLAG_NEWLINE), 
/// token is after or before a whitespace, and whether
/// the node start a subexpression <tt>(..)</tt>.
//===================================================
// @{
#define FLAG_NEWLINE		0x0001
#define FLAG_AFTER_SPACE	0x0002
#define FLAG_BEFORE_SPACE	0x0004
#define FLAG_EXPRESSION		0x0008
#define FLAG_TOKENIZED_DATA	0x0010
#define FLAG_TOKENIZED_COMMANDS 0x0020
#define FLAG_EXTENDED_NODE	0x0040
#define FLAG_WAS_LIST_CONST     0x0080

#define IS_EXPRESSION(a)    GET_FLAGS(a,FLAG_EXPRESSION)
#define IS_EXTENDED(a)      GET_FLAGS(a,FLAG_EXTENDED_NODE)
// @}


#define CAR(x)		((x)->unode.a[2])
#define CDR(x)		((x)->unode.a[3])
#define POS(x)		(((atom_ex_t)(x))->unode_ex.a[4])
#define IS_LIST(x)	(ID(x)==LIST_ID)
#define IS_EMPTY(a)	((a)==empty_list)
#define IS_NOT_EMPTY(a)	((a)!=empty_list)


#define CAR_OFFSET	8



extern atom_t   empty_list;
extern int      print_width_limit;
extern int      print_depth_limit;
extern atom_t   new_list( atom_t car, atom_t cdr );
extern atom_t   new_list_ex( atom_t car, atom_t cdr );
extern void     delete_list( atom_t a );
extern void     dump_list( atom_t a, int level );
extern void     append( atom_t element, atom_t *first, atom_t *last );
extern void     append_ex( atom_t element, atom_t *first, atom_t *last );
extern atom_t   copy_append( atom_t list, atom_t element);
extern atom_t   behead( atom_t a );
extern atom_t   list_copy_n( atom_t a, unsigned int n );
extern atom_t   list_copy_but_last( atom_t a );
extern int      list_length( atom_t a );
extern atom_t   get_at_list( atom_t a, int n );
#endif //LHOGHO_LISTS_H
