//
// Project:	Lhogho
// File:	parser.c
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2006-10-26	- file created (from atom.h)
//   2007-02-15 - command tokenization
//   2007-02-26 - changed tokenization to microprogramming
//		- ERROR_WINDOW_CHRACTER
//		- trim_shell_comment()
//		- purify()
//   2007-03-22 - ERROR_HUNGRY_LEFT
//   2007-05-17 - license info
//   2007-05-22	- doxygen-friendly documentation
//   2007-05-27 - preparse()
//		- ERROR_VACUUM_TO_END, ERROR_WHERES_NAME
//   2007-05-29 - deexternalize parse...()
//		- ERROR_ONE_NAME_ONLY
//   2007-06-01 - ERROR_BEG_ME_PARDON
//		- error codes moved to errors.h
//   2007-06-05 - definitions spread to where they belong
//   2007-11-26 - barrization, ENBAR, DEBAR
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_PARSER_H
#define LHOGHO_PARSER_H

#include "stdint.h"
#include "globals.h"




//===================================================
/// \name Tokenization methods
//===================================================
#define TOKENIZE_DATA		0	///< tokenize as data
#define TOKENIZE_COMMANDS	1	///< tokenize as commands
#define TOKENIZE_READWORD	2	///< tokenize as for READWORD function
#define TOKENIZE_READLIST	3	///< tokenize as for READLIST function





#ifdef UNICODE
  #define ENBAR(ch) ((ch)>127 ? (ch) : enbar[ch]) ///< enbars a char: a->|a|
  #define DEBAR(ch) ((ch)>31  ? (ch) : debar[ch]) ///< debars a char: |a|->a
#else
  #define ENBAR(ch) (((unsigned char)ch)>127 ? (ch) : enbar[(unsigned char)ch])
  #define DEBAR(ch) ((ch)>31  ? (ch) : debar[(unsigned char)ch])
#endif

extern char_t enbar[128]; ///< table for a->|a| conversions
extern char_t debar[32];  ///< table for |a|->a conversions
extern void init_parser( );


extern atom_t tokenize( atom_t input, int method );
extern atom_t trim_shell_comment( atom_t word );
extern atom_t purify( atom_t word );
extern atom_t build_syntax_tree( atom_t function );




#endif //LHOGHO_PARSER_H
