//
// Project:	Lhogho
// File:	pools.h
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2006-09-29	- Pools module created
//   2006-09-30	- Separated free atoms from blocks
//   2006-10-10	- More detailed statistics
//   2007-02-27	- Module renamed from \b Pool to \b Pools
//   2007-05-17	- Added license info
//   2007-05-21	- Added doxygen-friendly documentation
//   2007-06-05 - Simplified interface
//   2007-12-11 - dump_pool()
//   2008-01-18 - data_pool_ex
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//



//===================================================
/// \struct	pool_t
/// \brief	internal structure of a pool
//===================================================
typedef struct {
  int	id;		   ///< (const) pool id
  int	atom_size;	   ///< (const) size of an atom
  int	block_size;	   ///< (const) max number of atoms in the block
  int	free_block_atoms;  ///< current number of free atoms in the block
  void*	free_atom;	   ///< first free atom in the list
  void*	free_block;	   ///< first free atom in the block
  void* first_atom;	   ///< first atom in the pool
} pool_t;


extern pool_t data_pool;
extern pool_t data_pool_ex;
extern void  init_pool      ( pool_t* pool, int atom_size );
extern void* take_from_pool ( pool_t* pool );
extern void  return_to_pool ( pool_t* pool, void* a );

#ifdef DEBUG_MEMORY_LEAKS
extern void  dump_pool( );
#endif
