//
// Project:	Lhogho
// File:	runtime.c
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-06-19	- file created
//   2007-06-25 - added rt_plus()
//   2007-06-26 - rt_minus(), rt_mul(), rt_div()
//   2007-07-04 - rt_cmdchk(), rt_funchk()
//   2007-07-31 - rt_make()
//   2007-08-01 - rt_local()
//   2007-08-05 - rt_makechk()
//   2007-08-08 - Added arithmetical and trigonometric primitives
//   2007-09-03 - Fixed bug #1787023 ARCTAN and RADARCTAN
//   2007-09-14 - Added comparison primitives
//   2007-09-17 - Added AND, OR, NOT
//   2007-09-18 - rt_output(), rt_stop(), rt_dump()
//   2007-09-19 - rt_ignore()
//   2007-09-19 - rt_first(), rt_butfirst()
//   2007-09-19 - rt_last(), rt_butlast(), rt_item()
//   2007-09-23 - rt_parse(), rt_runparse()
//   2007-09-26 - rt_wordp(), rt_listp(), rt_numberp(), rt_empty()
//   2007-09-27 - rt_memberp()
//   2007-09-29 - rt_repeat()
//   2007-10-06 - rt_word(), rt_list(), rt_sent(), rt_fput(), rt_lput()
//   2007-10-13 - rt_count(), rt_char(), rt_ascii(), rt_upper(), rt_lower(), rt_member()
//   2007-10-28 - rt_random(), rt_rerandom(), rt_iseq(), rt_rseq()
//   2007-10-28 - rt_type(), rt_show(), rt_form(), rt_format()
//   2007-11-04 - rt_while(), rt_until()
//   2007-11-05 - support for PRINTDEPTHLIMIT, PRINTWIDTHLIMIT and
//		  FULLPRINTP
//   2007-11-06 - rt_dowhile(), rt_dountil()
//		- rt_definedp(), rt_primitivep(), namep(), procedurep()
//		- rt_thing()
//   2007-11-09 - rt_reference
//   2007-11-11 - rt_bye(), rt_wait()
//		- rt_name()
//   2007-11-12 - rt_throw()
//   2007-11-16 - rt_catch()
//   2007-11-21 - rt_test(), rt_iftrue(), rt_ifelse()
//   2007-12-02 - rt_backslashedp()
//   2007-12-02 - rt_ashift(), rt_lshift()
//		- rt_bitand(), rt_bitor(), rt_bitxor(), rt_bitnot()
//   2007-12-03 - rt_text()
//   2007-12-14 - rt_fulltext()
//   2007-12-16 - rt_pick(), rt_remdup(), rt_remove(), rt_reverse(),
//      - rt_rawascii(), rt_gensym(), rt_substrp()
//   2007-12-22 - rt_run()
//   2007-12-30 - rt_combine()
//   2007-01-05 - rt_runresult()
//   2008-03-22 - rt_quoted()
//   2008-05-16 - rt_define()
//   2008-08-14 - rt_for()
//   2008-08-17 - rt_firsts(), rt_butfirsts()
//   2008-09-05 - rt_dllload(), rt_libfree()
//              - rt_packsize(), rt_pack(), rt_unpack()
//   2008-09-06 - rt_packaddr(), rt_packto()
//   2008-09-09 - rt_external()
//   2008-09-18 - rt_internal()
//   2008-10-27 - rt_load()
//   2008-11-21 - rt_openfile(), rt_closefile(), rt_readpack(), rt_writepack()
//   2009-06-03 - rt_readchar(), rt_readchars(), rt_getenv(), rt_getenvs()
//   2009-08-03 - rt_readrawline()
//   2009-08-04 - rt_readword(), rt_readlist()
//   2010-06-25 - rt_eofp()
//   2011-10-14 - rt_runmacro()
//   2011-12-26 - rt_substring()
//   2012-01-19 - rt_currentfolder(), rt_makefolder()
//   2012-01-20 - rt_erasefolder(), rt_folderp(), rt_renamefolder()
//   2012-01-21 - rt_folders(), rt_changefolder()
//   2012-01-23 - rt_files(), rt_erasefile(), rt_file(), rt_renamefile()
//                rt_filesize(), rt_filetimes(), rt_formattime()
//   2012-01-24 - rt_openread(), rt_openwrite(), rt_openappend(),
//                rt_openupdate(), rt_setread(), rt_setwrite()
//   2012-01-25 - init_runtime(), finit_runtime, rt_reader(), rt_writer()
//                rt_allopen()
//   2012-01-27 - rt_closeall(), rt_readpos(), rt_setreadpos(), rt_writepos(),
//                rt_setwritepos()
//   2012-01-31 - rt_timezone()
//   2012-02-04 - adding mode to file open functions
//   2012-02-05 - rt_readinblock()
//   2012-02-09 - rt_dribble(), rt_nodribble()
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_RUNTIME_H
#define LHOGHO_RUNTIME_H

#include "globals.h"
#include "atoms.h"


#define RETURN(a)			\
  { atom_t _res_ = (a);			\
    __asm__( " " );			\
    return (_res_);			\
  } ///< used to return a value without GCC's tail-function optimizaiton

#define GET_FLOAT(a,b)				\
{									\
  if( IS_ERROR(a) )					\
    RETURN( use(a) );				\
  if( !atom_to_float( a, &b ) )		\
    RETURN( new_error( ERROR_NOT_A_NUMBER, unbound ) ) \
} ///< convert input to floating point value and check for errors



#define GET_INT(a,b)		                                \
{							                                \
    if (IS_ERROR(a))		                                \
        RETURN(use(a));						\
    if (IS_INTEGER(a))		                                \
        b = INTEGER(a);		                                \
    else  \
    if (!atom_to_int(a, &b))                                \
        RETURN(new_error(ERROR_NOT_AN_INTEGER, unbound))     \
}   ///< convert input to integer number and check for errors


#define GET_BOOLEAN(a, b)                                   \
{                                                           \
    if (IS_ERROR(a)) RETURN(use(a));			    \
    if (a == false_true[0]) b = 0;                          \
    else if (a == false_true[0]) b = 1;                     \
    if (!atom_to_boolean(a, &b))                            \
        RETURN(new_error(ERROR_BOOLEAN_EXPECTED, unbound))   \
}   ///< convert input to boolean number and check for errors



#define FASTRUNTIME(X) extern atom_t X __attribute__((used,noinline,regparm(0),stdcall)) ///< default attributes of special runtime functions

FASTRUNTIME( rt_cmdchk      ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_funchk      ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_exprchk     ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_repchk      ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_whlchk      ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_makechk     ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_boolchk     ( atom_t source, atom_t data ) );
FASTRUNTIME( rt_dump        ( atom_t source ) );
FASTRUNTIME( rt_predump     ( atom_t source ) );
FASTRUNTIME( rt_catchchk    ( int status, atom_t tag, atom_t data ) );
//FASTRUNTIME( rt_catchchk    ( atom_t tag, atom_t data ) );
FASTRUNTIME( rt_forchk      ( atom_t source, atom_t* step_var, atom_t step, atom_t to, atom_t from ) );
FASTRUNTIME( rt_check_inputs( int stack_frame ) );



#define RUNTIME(X) extern atom_t X __attribute__((used,noinline,regparm(0),cdecl)) ///< default attributes of runtime functions

RUNTIME( rt_print   (atom_t pdl, atom_t pwl, atom_t fpp, int data));
RUNTIME( rt_plus    (int count, atom_t data2, atom_t data1));
RUNTIME( rt_minus   (int count, atom_t data2, atom_t data1));
RUNTIME( rt_div     (atom_t data2, atom_t data1));
RUNTIME( rt_mul     (atom_t data2, atom_t data1));
RUNTIME( rt_sum     (int data));
RUNTIME( rt_difference (atom_t data2, atom_t data1));
RUNTIME( rt_unminus (atom_t data));
RUNTIME( rt_product (int data));
RUNTIME( rt_remainder  (atom_t data2, atom_t data1));
RUNTIME( rt_int     (atom_t data));
RUNTIME( rt_round   (atom_t data));
RUNTIME( rt_sqrt    (atom_t data));
RUNTIME( rt_power   (atom_t power, atom_t base));
RUNTIME( rt_exp     (atom_t power));
RUNTIME( rt_log10   (atom_t data));
RUNTIME( rt_ln      (atom_t data));
RUNTIME( rt_abs     (atom_t data));
RUNTIME( rt_pi      (void));
RUNTIME( rt_sin     (atom_t data));
RUNTIME( rt_radsin  (atom_t data));
RUNTIME( rt_cos     (atom_t data));
RUNTIME( rt_radcos  (atom_t data));
RUNTIME( rt_arctan  (int count, atom_t data2, atom_t data1));
RUNTIME( rt_radarctan (int count, atom_t data2, atom_t data1));
RUNTIME( rt_equal   (atom_t cip, atom_t data2, atom_t data1));
RUNTIME( rt_nequal  (atom_t cip, atom_t data2, atom_t data1));
RUNTIME( rt_less    (atom_t data2, atom_t data1));
RUNTIME( rt_more    (atom_t data2, atom_t data1));
RUNTIME( rt_lesseq  (atom_t data2, atom_t data1));
RUNTIME( rt_moreeq  (atom_t data2, atom_t data1));
RUNTIME( rt_before  (atom_t cip, atom_t data2, atom_t data1));
RUNTIME( rt_and     (int count));
RUNTIME( rt_or      (int count));
RUNTIME( rt_not     (atom_t data));
RUNTIME( rt_first   (atom_t data));
RUNTIME( rt_butfirst (atom_t data));
RUNTIME( rt_firsts  (atom_t data));
RUNTIME( rt_butfirsts (atom_t data));
RUNTIME( rt_last    (atom_t data));
RUNTIME( rt_butlast (atom_t data));
RUNTIME( rt_item    (atom_t data, atom_t index));
RUNTIME( rt_wordp   (atom_t data));
RUNTIME( rt_listp   (atom_t data));
RUNTIME( rt_numberp (atom_t data));
RUNTIME( rt_empty   (atom_t data));
RUNTIME( rt_memberp (atom_t cip, atom_t data, atom_t elem));
RUNTIME( rt_word    (int data));
RUNTIME( rt_list    (int data));
RUNTIME( rt_sent    (int data));
RUNTIME( rt_fput    (atom_t data2, atom_t data1));
RUNTIME( rt_lput    (atom_t data2, atom_t data1));
RUNTIME( rt_count   (atom_t data));
RUNTIME( rt_char    (atom_t data));
RUNTIME( rt_ascii   (atom_t data));
RUNTIME( rt_lower   (atom_t data));
RUNTIME( rt_upper   (atom_t data));
RUNTIME( rt_member  (atom_t cip, atom_t elem, atom_t data));
RUNTIME( rt_iseq    (atom_t to, atom_t from));
RUNTIME( rt_rseq    (atom_t count, atom_t to, atom_t from));
RUNTIME( rt_random  (int count, atom_t data2, atom_t data1));
RUNTIME( rt_rerandom(int count, atom_t seed));
RUNTIME( rt_show    (atom_t pdl, atom_t pwl, atom_t fpp, int data));
RUNTIME( rt_type    (atom_t pdl, atom_t pwl, atom_t fpp, int data));
RUNTIME( rt_form    (atom_t precision, atom_t width, atom_t num));
RUNTIME( rt_format  (atom_t format, atom_t data));
RUNTIME( rt_formattime (atom_t format, atom_t data));
RUNTIME( rt_timezone ( ));
RUNTIME( rt_bye     (void));
RUNTIME( rt_wait    (atom_t time));
RUNTIME( rt_ashift  (atom_t bits, atom_t num));
RUNTIME( rt_lshift  (atom_t bits, atom_t num));
RUNTIME( rt_bitand  (int data));
RUNTIME( rt_bitor   (int data));
RUNTIME( rt_bitxor  (int data));
RUNTIME( rt_bitnot  (atom_t data));



RUNTIME( rt_if		( int count ) );
RUNTIME( rt_repeat	( atom_t repcount, atom_t commands ) ); 
RUNTIME( rt_while	( atom_t condition, atom_t commands ) ); 
RUNTIME( rt_until	( atom_t condition, atom_t commands ) ); 
RUNTIME( rt_dowhile	( atom_t commands, atom_t condition ) ); 
RUNTIME( rt_dountil	( atom_t commands, atom_t condition ) ); 
RUNTIME( rt_repcount	( int repcount ) ); 
RUNTIME( rt_forever	( atom_t commands ) ); 
RUNTIME( rt_make	( int static_link, atom_t parent, atom_t value, atom_t name ) );
RUNTIME( rt_name	( int static_link, atom_t parent, atom_t value, atom_t name ) );
RUNTIME( rt_local	( int static_link, atom_t parent, int data ) );
RUNTIME( rt_output	( atom_t data ) );
RUNTIME( rt_maybeoutput	( atom_t data ) );
RUNTIME( rt_stop	( ) );
RUNTIME( rt_ignore	( atom_t data ) );
RUNTIME( rt_for         ( atom_t body, atom_t limits, atom_t var ) );
        
RUNTIME( rt_parse	(atom_t data) );
RUNTIME( rt_runparse	(atom_t data) );

RUNTIME( rt_definedp	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_primitivep	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_namep	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_procedurep	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_thing	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_reference	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_text	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_fulltext	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_run		( int static_link, atom_t parent, atom_t data, int mode ) );
RUNTIME( rt_runresult	( int static_link, atom_t parent, atom_t data ) );
RUNTIME( rt_runmacro	( int static_link, atom_t parent, atom_t data, int mode ) );
RUNTIME( rt_define	( int static_link, atom_t parent, atom_t value, atom_t name ) );
RUNTIME( rt_load	( atom_t data ) );
RUNTIME( rt_commandline	( ) );

RUNTIME( rt_throw	( int count, atom_t value, atom_t tag ) );
RUNTIME( rt_catch	( atom_t commands, atom_t tag ) );
RUNTIME( rt_error	( ) );

RUNTIME( rt_tag		( ) );
RUNTIME( rt_goto	( int static_link, atom_t parent, atom_t data, atom_t source ) );

RUNTIME( rt_test	( atom_t condition, int frame ) ); 
RUNTIME( rt_iftrue	( atom_t commands ) );
RUNTIME( rt_iffalse	( atom_t commands ) );

RUNTIME( rt_backslashedp ( atom_t data ) );

RUNTIME( rt_pick    (atom_t list) );
RUNTIME( rt_remdup  (atom_t cip, atom_t list) );
RUNTIME( rt_remove  (atom_t cip, atom_t data, atom_t elem) );
RUNTIME( rt_reverse (atom_t list) );
RUNTIME( rt_rawascii    (atom_t data) );
RUNTIME( rt_gensym() );
RUNTIME( rt_combine (atom_t data2, atom_t data1));
RUNTIME( rt_quoted  (atom_t data));
RUNTIME( rt_substringp (atom_t cip, atom_t data2, atom_t data1));
RUNTIME( rt_substring (atom_t cip, atom_t data2, atom_t data1));

RUNTIME( rt_libload (atom_t data));
RUNTIME( rt_libfree (atom_t data));
RUNTIME( rt_blocksize (int static_link, atom_t parent, atom_t prototype));
RUNTIME( rt_listtoblock (int static_link, atom_t parent, atom_t prototype, atom_t data));
RUNTIME( rt_blocktolist (int static_link, atom_t parent, atom_t prototype, atom_t data));
RUNTIME( rt_dataaddr (atom_t data));
RUNTIME( rt_listintoblock (int static_link, atom_t parent, atom_t prototype, atom_t dest, atom_t data));
RUNTIME( rt_funcaddr( int static_link, atom_t parent, atom_t data ));

RUNTIME( rt_openfile   (atom_t mode, atom_t filename, int call_mode));
RUNTIME( rt_closefile  (atom_t handle));
RUNTIME( rt_readblock  (int static_link, atom_t parent, atom_t size));
RUNTIME( rt_readinblock (atom_t block, int call_mode));
RUNTIME( rt_writeblock (atom_t data));

RUNTIME( rt_external (int static_link, atom_t parent, atom_t handle, atom_t prototype, atom_t name));
RUNTIME( rt_internal (int static_link, atom_t parent, atom_t prototype, atom_t name));
RUNTIME( rt_int3 ());
RUNTIME( rt_stackframe (int static_link, atom_t parent, atom_t offset, atom_t frame));
RUNTIME( rt_stackframeatom (int static_link, atom_t parent, atom_t offset, atom_t frame));

RUNTIME( rt_readchar    (void));
RUNTIME( rt_readchars   (atom_t data));
RUNTIME( rt_readrawline (void));
RUNTIME( rt_readword    (void));
RUNTIME( rt_readlist    (void));

RUNTIME( rt_getenv (atom_t data));
RUNTIME( rt_getenvs ( ));
RUNTIME( rt_eofp ( ));

RUNTIME( rt_currentfolder ( ));
RUNTIME( rt_makefolder (atom_t name));
RUNTIME( rt_erasefolder (atom_t name));
RUNTIME( rt_folderp (atom_t name));
RUNTIME( rt_renamefolder (atom_t toname, atom_t fromname));
RUNTIME( rt_folders (atom_t name));
RUNTIME( rt_changefolder (atom_t name));
RUNTIME( rt_files (atom_t name));
RUNTIME( rt_erasefile (atom_t name));
RUNTIME( rt_filep (atom_t name));
RUNTIME( rt_renamefile (atom_t toname, atom_t fromname));
RUNTIME( rt_filesize (atom_t name));
RUNTIME( rt_filetimes (atom_t name));
RUNTIME( rt_openread (atom_t name, int call_mode));
RUNTIME( rt_openwrite (atom_t name, int call_mode));
RUNTIME( rt_openappend (atom_t name, int call_mode));
RUNTIME( rt_openupdate (atom_t name, int call_mode));
RUNTIME( rt_setread (atom_t file));
RUNTIME( rt_setwrite (atom_t file));
RUNTIME( rt_reader ( ));
RUNTIME( rt_writer ( ));
RUNTIME( rt_allopen ( ));
RUNTIME( rt_closeall ( ));
RUNTIME( rt_readpos ( ));
RUNTIME( rt_setreadpos (atom_t pos));
RUNTIME( rt_writepos ( ));
RUNTIME( rt_setwritepos (atom_t pos));
RUNTIME( rt_dribble (atom_t file));
RUNTIME( rt_nodribble ( ));


extern atom_t __attribute__((used,noinline,regparm(0),stdcall)) rt_use_var( atom_t source, atom_t value );
extern atom_t rt_var_value( int static_link, atom_t parent, atom_t var );

extern void   __attribute__ ((used,noinline,regparm(0),stdcall)) rt_repeat_enter( int frame, int count ); 
extern atom_t __attribute__ ((used,noinline,regparm(0),stdcall)) rt_repeat_exit( int frame );
extern void   __attribute__ ((used,noinline,regparm(0),stdcall)) rt_forever_enter( int frame );
extern atom_t __attribute__ ((used,noinline,regparm(0),stdcall)) rt_runresult_fix( atom_t data );

extern void init_runtime( );
extern void finit_runtime( );

#endif //LHOGHO_RUNTIME_H
