//
// Project:	Lhogho
// File:	translate.h
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-02-27	- file created
//   2007-03-20 - TR_PRIMITIVES
//   2007-05-17 - license info
//   2007-05-22	- doxygen-friendly documentation
//   2007-05-26 - translation of TO and END
//   2007-05-27 - texts for error messages
//		- TR_ERROR_VACUUM_TO_END
//		- TR_ERROR_WHERES_NAME
//   2007-05-29 - TR_ERROR_ONE_NAME_ONLY
//   2007-05-31 - TR_LHOGHO_CAPION, TR_OPTION_HELP
//   2007-06-01 - added -ci option
//		- TR_ERROR_BEG_ME_PARDON
//   2007-06-02	- OPTION_MEMORY, OPTION_MEMORY_LONG
//   2007-06-03 - errors renamed
//		- TR_ERROR_CROWDED_SOURCES, ERROR_OS_ERROR
//		- TR_ERROR_DUPLICATE_FUNC
//		- OPTION_VARS, OPTION_VARS_LONG
//   2007-06-08 - fixed translation of TR_ERROR_MISSING_RIGHTS
//   2007-06-11 - OPTION_USERVARS, OPTION_USERVARS_LONG
//		- OPTION_EXECUTABLE, OPTION_EXECUTABLE_LONG
//   2007-06-12 - fixed bug #1735806 (Unaligned options)
//   2007-06-21 - OPTION_ASM, OPTION_ASM_LONG
//   2007-06-26 - TR_ERROR_NOT_A_NUMBER
//   2007-07-01 - TR_ERROR_UNUSED_VALUE
//		- TR_ERROR_UNUSED_VALUE
//   2007-07-09 - OPTION_RUNTIME, OPTION_RUNTIME_LONG
//   2007-07-11 - OPTION_TRADITIONAL, OPTION_TRADITIONAL_LONG
//   2007-08-09 - Fixed bug #1771163 Warning building the dev docs
//   2007-09-03 - Fixed bug #1787023 ARCTAN and RADARCTAN
//   2007-09-09 - TR_ERROR_DO_NOT_KNOW
//		- TR_ERROR_UNKNOWN_VAR
//   2007-09-14 - Added caseignoredp
//   2007-09-17 - TR_ERROR_BOOLEAN_EXPECTED
//   2007-09-21 - TR_ERROR_NOT_AN_INTEGER, TR_ERROR_NOT_A_LIST
//   2007-09-23 - Support for RUNPARSE
//   2007-10-06 - TR_ERROR_NOT_A_WORD
//   2007-10-13 - TR_ERROR_INCOMPATIBLE_DATA
//   2007-10-17 - TR_ERROR_TOO_BIG_NUMBER, TR_ERROR_TOO_SMALL_NUMBER
//   2007-10-23 - Added translation of REPCOUNT
//   2007-11-04 - TR_ERROR_NOT_A_VAR
//   2007-11-06 - Added DO.WHILE and DO.UNTIL
//		- Added PRIMITIVE? PRIMITIVEP DEFINED? DEFINEDP
//   2007-11-11 - Added NAME
//   2007-11-12 - TR_EXIT_BY_BYE, TR_EXIT_BY_THROW_TOPLEVEL,
//		  TR_EXIT_BY_THROW_SYSTEM, TR_EXIT_BY_THROW_ERROR
//		- Added CATCH, THROW
//		- Added TOPLEVEL, SYSTEM, ERROR
//   2007-11-15 - TR_EXIT_BY_THROW_TAG, TR_EXIT_BY_THROW_TAG_VALUE
//   2007-11-21 - Added TEST, IFT, IFTRUE, IFF, IFFALSE
//   2007-12-02 - Added BACKSLASHEDP, BACKSLASHED?
//   2007-12-03 - Added LSHIFT, ASHIFT, BITAND, BITOR, BITXOR, BITNOT
//   2007-12-03 - Added TEXT, FULLTEXT, DEFINE
//   2007-12-04 - TR_ERROR_NOT_A_USER_FUNCTION
//   2007-12-15 - Added LSHIFT, ASHIFT, BITAND, BITOR, BITXOR, BITNOT
//   2008-01-05 - TR_ERROR_VAR_HAS_NO_VALUE
//		- Added RUNRESULT
//   2008-03-22 - Added QUOTED
//   2008-08-14 - Added TR_ERROR_NOT_A_LIST_CONST
//   2008-08-17 - Added FIRSTS, BUTFIRSTS, BFS
//   2008-09-05 - Added LIBLOAD, LIBFREE
//              - Added PACKSIZE, PACK, UNPACK
//   2008-09-06 - Added PACKADDR, PACKTO
//   2008-09-09 - Added EXTERNAL
//   2008-09-11 - TR_ERROR_NOT_A_TYPE_NAME
//   2008-09-13 - TR_ERROR_BAD_PROTOTYPE
//   2008-09-18 - Added INTERNAL
//   2008-11-21 - Added PACKOPEN, PACKCLOSE, PACKREAD, PACKWRITE
//   2009-05-16 - Added ABS
//   2009-05-18 - Added COMMANDLINE
//   2009-06-03 - Added READCHAR, READCHARS, GETENV, GETENVS
//   2009-08-03 - Added READRAWLINE
//   2009-08-04 - Added READWORD, READLIST
//   2009-08-19 - Support for MAYBEOUTPUT
//   2009-08-20 - Support for _STACKFRAME, _STACKFRAMEATOM
//   2010-06-25 - Added EOFP, EOF?
//   2011-02-11 - File split into separate languages
//   2012-01-02 - Command-line options are translatable
//   
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_TRANSLATE_H
#define LHOGHO_TRANSLATE_H


#include "options.h"

#ifdef LANGUAGE_EN
  #define LANG_STR "en"
  #include "translate_en.h"
#endif

#ifdef LANGUAGE_DE
  #define LANG_STR "de"
  #include "translate_de.h"
#endif

#ifdef LANGUAGE_BG
  #define LANG_STR "bg"
  #include "translate_bg.h"
#endif


#ifndef LANG_STR
  #define LANG_STR "en"
  #include "translate_en.h"
#endif


#endif //LHOGHO_TRANSLATE_H
