//
// Project:	Lhogho
// File:	translate_en.h
//
// Copyright (C) 2011-2012 P.Boytchev
//
// Revision history:
//   2011-02-11	- file created from old translate.h
//   2011-10-07 - Removed TR_WARNING_UNSUPPORTED_COMPILER, because GCC 4+ is now supported
//   2011-10-14 - Added RUNMACRO
//   2011-12-02 - Added TR_ERROR_OUT_OF_MEM
//   2011-12-26 - Error messages taken from string, stored to array of atoms
//   2012-01-02 - Command-line options are translatable
//   2012-01-05 - More synonyms for primitives
//   
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_TRANSLATE_EN_H
#define LHOGHO_TRANSLATE_EN_H


// ===========================================================
/// \def	TR_OPTION_HELP
/// \brief	message to appear for -h and --help option
// ===========================================================
#define TR_OPTIONS  TEXT( \
  " |-h|    |--help|                || |print this help message| "\
  " |-x|    |--executable|          || |create executable file| "\
  " |-xc|   |--executable-compiler| || |embed Lhogho in executable file| "\
  " |-ci|   |--case-insensitive|    || |switch to case-insentivie mode| "\
  " |-t|    |--traditional|         || |switch to traditional mode| "\
  " |-Za|   |--Zassembler|          || |dump assembler code| "\
  " |-Zm|   |--Zmemory|             || |dump memory statistics| "\
  " |-Zmad| |--Zmemory-all-details| || |dump detailed memory statistics| "\
  " |-Zrt|  |--Zrun-time|           || |dump runtime evaluations| "\
  " |-Zt|   |--Ztree|               || |dump abstract syntax trees| "\
  " |-Zv|   |--Zvariables|          || |dump variables| "\
  " |-Zuv|  |--Zuser-variables|     || |dump user variables| "\
  " " )

#define TR_HELP_TEXTS TEXT( \
  " |LHOGHO - The LOGO Compiler| "\
  " |Usage:\n  lhogho [option] [option] ... filename [arguments]\n\nOptions:\n| "\
  " |\nAdvanced options:| "\
  " |\nUnsupported options:| "\
  " " )

  
// ===========================================================
/// \def	TR_PRIMITIVES
/// \brief	translations of primitives
// ===========================================================
#define TR_PRIMITIVES	TEXT(				                 \
  " false true to learn end toplevel system error"                 \
  " * / + - = < > <= >= <> all? and any? or not "	                     \
  " first || butfirst bf firsts || butfirsts bfs"            \
  " last || butlast bl item || "                                \
  " word list sentence se firstput fput lastput lput "                        \
  " word? wordp list? listp number? numberp empty? emptyp "  \
  " equal? equalp notequal? notequalp before? beforep "      \
  " less? lessp greater? greaterp lessequal? lessequalp "    \
  " greaterequal? greaterequalp member? memberp || || count || "      \
  " char ascii lowercase || uppercase || member parse runparse "   \
  " sum difference minus product quotient remainder "        \
  " int round sqrt power exp log10 ln abs "                  \
  " pi sin radsin cos radcos arctan radarctan "              \
  " iseq rseq random || rerandom || pick "                         \
  " lshift ashift bitand bitor bitxor bitnot "               \
  " print pr ? show type || form format formattime timezone "                       \
  " if ifelse repeat || repcount while do.while until do.until "\
  " run forever ignore stop || make name output op maybeoutput local thing bye " \
  " wait tag goto defined? definedp primitive? primitivep  " \
  " name? namep procedure? procedurep "			             \
  " for case cond text fulltext define || load commandline "    \
  " printdepthlimit printwidthlimit fullprintp caseignoredp "\
  " logoplatform logoversion logodialect "			         \
  " catch throw error apply "					             \
  " test iftrue ift iffalse iff "			                 \
  " backslashed? backslashedp "					             \
  " remdup remove reverse rawascii"                          \
  " substring? substringp substring gensym combine quoted"             \
  " runresult runmacro " \
  " libload libfree " \
  " blocksize listtoblock blocktolist dataaddr listintoblock funcaddr " \
  " openfile closefile readblock readinblock writeblock " \
  " external internal _int3 _stackframe _stackframeatom " \
  " readchar rc readchars rcs readrawline readword rw readlist rl " \
  " getenv getenvs eof? eofp " \
  " currentfolder makefolder erasefolder folder? folderp " \
  " renamefolder folders changefolder files erasefile erf file? filep " \
  " renamefile filesize filetimes openread openwrite openappend openupdate " \
  " setread setwrite reader writer allopen closeall "  \
  " readpos setreadpos writepos setwritepos dribble nodribble " \
  " " )


// ===========================================================
/// \name Error messages
// ===========================================================
// @{
#define TR_ERRORS TEXT( \
  " |The other counterpart is missing| "          /*ERROR_INCOMPLETE_PAIR*/\
  " |Empty expression| "                          /*ERROR_EMPTY_EXPRESSION*/\
  " |Crowded expression| "                        /*ERROR_CROWDED_EXPRESSION*/\
  " |Some left inputs are missing| "              /*ERROR_MISSING_LEFTS*/\  
  " |Some right inputs are missing| "             /*ERROR_MISSING_RIGHTS*/\
  " |Empty TO..END definition| "                  /*ERROR_EMPTY_TO_END*/\
  " |Missing name of a TO..END definition| "      /*ERROR_MISSING_NAME*/\
  " |Duplicate name of input| "                   /*ERROR_DUPLICATE_INPUT*/\
  " |Unknown option| "                            /*ERROR_UNKNOWN_OPTION*/\
  " |Two or more input source files| "            /*ERROR_CROWDED_SOURCES*/\
  " |OS Error| "                                  /*ERROR_OS_ERROR*/\
  " |Incompatible TO..END redefinition| "         /*ERROR_INCOMPATIBLE_REDEFINITION*/\
  " |Not a number| "                              /*ERROR_NOT_A_NUMBER*/\
  " |Unused value| "                              /*ERROR_UNUSED_VALUE*/\
  " |Missing value| "                             /*ERROR_MISSING_VALUE*/\
  " |Do not know how to do this| "                /*ERROR_DO_NOT_KNOW*/\
  " |Reference to unknown variable| "             /*ERROR_UNKNOWN_VAR*/\
  " |Boolean expression expected| "               /*ERROR_BOOLEAN_EXPECTED*/\
  " |Not an integer number| "                     /*ERROR_NOT_AN_INTEGER*/\
  " |Not a list| "                                /*ERROR_NOT_A_LIST*/\
  " |Not a word| "                                /*ERROR_NOT_A_WORD*/\
  " |Value not compatible with function requirements| " /*ERROR_INCOMPATIBLE_DATA*/\
  " |Numeric value is too big| "                  /*ERROR_TOO_BIG_NUMBER*/\
  " |Numeric value is too small| "                /*ERROR_TOO_SMALL_NUMBER*/\
  " |Not a variable| "                            /*ERROR_NOT_A_VAR*/\
  " |Bye| "                                       /*EXIT_BY_BYE*/\
  " |Thrown to toplevel| "                        /*EXIT_BY_THROW_TOPLEVEL*/\
  " |Thrown to system| "                          /*EXIT_BY_THROW_SYSTEM*/\
  " |Thrown by error| "                           /*EXIT_BY_THROW_ERROR*/\
  " |Thrown by error| "                           /*EXIT_BY_THROW_USER_ERROR*/\
  " |Uncatched throw| "                           /*EXIT_BY_THROW_TAG*/\
  " |Uncatched throw| "                           /*EXIT_BY_THROW_TAG_VALUE*/\
  " |Not a tag| "                                 /*ERROR_NOT_A_TAG*/\
  " |Not a user-defined function| "               /*ERROR_NOT_A_USER_FUNCTION*/\
  " |Variable has no value| "                     /*ERROR_VAR_HAS_NO_VALUE*/\
  " |Not a command or a function| "               /*ERROR_NOT_A_FUNCTION*/\
  " |Not a list constant| "                       /*ERROR_NOT_A_LIST_CONST*/\
  " |Not a word constant| "                       /*ERROR_NOT_A_WORD_CONST*/\
  " |Missing initial or final limit| "            /*ERROR_MISSING_FOR_LIMITS*/\
  " |Not a name of a supported type| "            /*ERROR_NOT_A_TYPE_NAME*/\
  " |Not a valid function prototype| "            /*ERROR_BAD_PROTOTYPE*/\
  " |Not a memory block| "                        /*ERROR_NOT_A_MEM*/\
  " |Out of memory| "                             /*ERROR_OUT_OF_MEM*/\
  " |File is not opened| "                        /*ERROR_FILE_NOT_OPENED*/\
  " |Not a block or a block definition list| "    /*ERROR_NOT_BLOCK_OR_DEF*/\
  " " )
// @}

#endif //LHOGHO_TRANSLATE_EN_H
