//
// Project:	Lhogho
// File:	translate_gr.h
//
// Copyright (C) 2011 P.Boytchev
// Translation by ????
//
// Revision history:
//   2011-02-11	- file created from old translate.h
//   2011-10-07 - Removed TR_WARNING_UNSUPPORTED_COMPILER, because GCC 4+ is now supported
//   2011-10-14 - Added RUNMACRO
//   2011-12-02 - Added TR_ERROR_OUT_OF_MEM
//   
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_TRANSLATE_GR_H
#define LHOGHO_TRANSLATE_GR_H


// ===========================================================
/// \def	TR_LHOGHO_CAPTION
/// \brief	Lhogho welcome message
// ===========================================================
#define TR_LHOGHO_CAPTION	                \
  TEXT("LHOGHO - The LOGO Compiler ["	        \
	OS_NAME "-" PROCESSOR_NAME "(En), "	\
	__DATE__ "]\n"				\
      "\n")



// ===========================================================
/// \def	ADVANCED_OPTIONS
/// \brief	text describing the advanced options
// ===========================================================
#ifdef ADVANCED
#define ADVANCED_OPTIONS "Advanced options:\n"
#else
#define ADVANCED_OPTIONS "Unsupported options:\n"
#endif




// ===========================================================
/// \def	TR_OPTION_HELP
/// \brief	message to appear for -h and --help option
// ===========================================================
#define TR_OPTION_HELP TEXT(						\
    "Usage:\n"								\
    "  lhogho [option] [option] ... filename [arguments]\n"		\
    "\n"								\
    "Options:\n"							\
    "  -h     --help                print this help message\n"		\
    "  -x     --executable          create executable file\n"		\
    "  -xc    --executable-compiler embed Lhogho in executable file\n"	\
    "  -ci    --case-insensitive    switch to case-insentivie mode\n"	\
    "  -t     --traditional         switch to traditional mode\n"	\
    "\n"								\
    ADVANCED_OPTIONS							\
    "  -Za    --Zassembler          dump assembler code\n"		\
    "  -Zm    --Zmemory             dump memory statistics\n"		\
    "  -Zmad  --Zmemory-all-details dump detailed memory statistics\n"	\
    "  -Zrt   --Zrun-time           dump runtime evaluations\n"		\
    "  -Zt    --Ztree               dump abstract syntax trees\n"	\
    "  -Zv    --Zvariables          dump variables\n"			\
    "  -Zuv   --Zuser-variables     dump user variables\n"		\
    "" )




// ===========================================================
/// \def	TR_PRIMITIVES
/// \brief	translations of primitives
// ===========================================================
#define TR_PRIMITIVES	TEXT(				                 \
  " false true to end toplevel system error"                 \
  " * / + - = < > <= >= <> and or not "	                     \
  " first butfirst bf firsts butfirsts bfs"                  \
  " last butlast bl item "                                   \
  " word list sentence se fput lput "                        \
  " word? wordp list? listp number? numberp empty? emptyp "  \
  " equal? equalp notequal? notequalp before? beforep "      \
  " less? lessp greater? greaterp lessequal? lessequalp "    \
  " greaterequal? greaterequalp member? memberp count "      \
  " char ascii lowercase uppercase member parse runparse "   \
  " sum difference minus product quotient remainder "        \
  " int round sqrt power exp log10 ln abs "                  \
  " pi sin radsin cos radcos arctan radarctan "              \
  " iseq rseq random rerandom pick "                         \
  " lshift ashift bitand bitor bitxor bitnot "               \
  " print pr ? show type form format "                       \
  " if ifelse repeat repcount while do.while until do.until "\
  " run forever ignore stop make name output op maybeoutput local thing bye " \
  " wait tag goto defined? definedp primitive? primitivep  " \
  " name? namep procedure? procedurep "			             \
  " for case cond text fulltext define load commandline "    \
  " printdepthlimit printwidthlimit fullprintp caseignoredp "\
  " logoplatform logoversion logodialect "			         \
  " catch throw error apply "					             \
  " test iftrue ift iffalse iff "			                 \
  " backslashed? backslashedp "					             \
  " remdup remove reverse rawascii"                          \
  " substring? substringp substring gensym combine quoted"             \
  " runresult runmacro " \
  " libload libfree " \
  " packsize pack unpack packaddr packto funcaddr " \
  " openfile closefile readpack writepack " \
  " external internal _int3 _stackframe _stackframeatom " \
  " readchar rc readchars rcs readrawline readword rw readlist rl " \
  " getenv getenvs eof? eofp " \
  " " )


// ===========================================================
/// \name Error messages
// ===========================================================
// @{
#define TR_ERROR_INCOMPLETE_PAIR    TEXT("The other counterpart is missing")
#define TR_ERROR_EMPTY_EXPRESSION   TEXT("Empty expression")
#define TR_ERROR_CROWDED_EXPRESSION TEXT("Crowded expression")
#define TR_ERROR_MISSING_LEFTS      TEXT("Some left inputs are missing")
#define TR_ERROR_MISSING_RIGHTS     TEXT("Some right inputs are missing")
#define TR_ERROR_EMPTY_TO_END       TEXT("Empty TO..END definition")
#define TR_ERROR_MISSING_NAME       TEXT("Missing name of a TO..END definition")
#define TR_ERROR_DUPLICATE_INPUT    TEXT("Duplicate name of input")
#define TR_ERROR_UNKNOWN_OPTION     TEXT("Unknown option")
#define TR_ERROR_CROWDED_SOURCES    TEXT("Two or more input source files")
#define TR_ERROR_OS_ERROR           TEXT("OS Error")
#define TR_ERROR_INCOMPATIBLE_REDEFINITION  TEXT("Incompatible TO..END redefinition")
#define TR_ERROR_NOT_A_NUMBER       TEXT("Not a number")
#define TR_ERROR_UNUSED_VALUE       TEXT("Unused value")
#define TR_ERROR_MISSING_VALUE      TEXT("Missing value")
#define TR_ERROR_DO_NOT_KNOW        TEXT("Do not know how to do this")
#define TR_ERROR_UNKNOWN_VAR        TEXT("Reference to unknown variable")
#define TR_ERROR_BOOLEAN_EXPECTED   TEXT("Boolean expression expected")
#define TR_ERROR_NOT_AN_INTEGER     TEXT("Not an integer number")
#define TR_ERROR_NOT_A_LIST         TEXT("Not a list")
#define TR_ERROR_NOT_A_WORD         TEXT("Not a word")
#define TR_ERROR_INCOMPATIBLE_DATA  TEXT("Value not compatible with function requirements")
#define TR_ERROR_TOO_BIG_NUMBER     TEXT("Numeric value is too big")
#define TR_ERROR_TOO_SMALL_NUMBER   TEXT("Numeric value is too small")
#define TR_ERROR_NOT_A_VAR          TEXT("Not a variable")
#define TR_EXIT_BY_BYE		    TEXT("Bye")
#define TR_EXIT_BY_THROW_TOPLEVEL   TEXT("Thrown to toplevel")
#define TR_EXIT_BY_THROW_SYSTEM     TEXT("Thrown to system")
#define TR_EXIT_BY_THROW_ERROR      TEXT("Thrown by error")
#define TR_EXIT_BY_THROW_USER_ERROR	TEXT("Thrown by error")
#define TR_EXIT_BY_THROW_TAG		TEXT("Uncatched throw")
#define TR_EXIT_BY_THROW_TAG_VALUE	TEXT("Uncatched throw")
#define TR_ERROR_NOT_A_TAG		TEXT("Not a tag")
#define TR_ERROR_NOT_A_USER_FUNCTION	TEXT("Not a user-defined function")
#define TR_ERROR_VAR_HAS_NO_VALUE   TEXT("Variable has no value")
#define TR_ERROR_NOT_A_FUNCTION     TEXT("Not a command or a function")
#define TR_ERROR_NOT_A_LIST_CONST   TEXT("Not a list constant")
#define TR_ERROR_NOT_A_WORD_CONST   TEXT("Not a word constant")
#define TR_ERROR_MISSING_FOR_LIMITS TEXT("Missing initial or final limit")
#define TR_ERROR_NOT_A_TYPE_NAME    TEXT("Not a name of a supported type")
#define TR_ERROR_BAD_PROTOTYPE      TEXT("Not a valid function prototype")
#define TR_ERROR_NOT_A_MEM	    TEXT("Not a memory block or packed data")
#define TR_ERROR_OUT_OF_MEM	    TEXT("Out of memory")
//#define TR_WARNING_UNSUPPORTED_COMPILER     TEXT("This Lhogho is compiled with unsupported compiler")

// @}

#endif //LHOGHO_TRANSLATE_GR_H
