//
// Project:  Lhogho
// File:  vars.c
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-02-27  - file created
//   2007-05-17 - added license info
//   2007-05-22  - added doxygen-friendly documentation
//   2007-05-26 - calling primitives' finalization
//   2007-05-29 - root var defined as a function
//    - find_local_var()
//   2007-06-05 - definitions spread to where they belong
//   2007-06-09 - added dump_var()
//    - fixed bug #1734095 (Cannot dump vars)
//   2007-06-11 - process option_user_variables
//   2007-06-18 - new_local_var()
//   2007-06-24 - fixed bug #1742361 (Memory leak using LOCAL)
//   2007-08-08 - Added addresses of arithmetical and trigonometric primitives
//   2007-08-13 - globals
//    - rt_printdepthlimit, rt_printwidthlimit, rt_fullprintp
//   2007-09-03 - Fixed bug #1787023 ARCTAN and RADARCTAN
//   2007-09-15 - Added addresses of comparison operators and primitives
//   2007-09-15 - global rt_caseignoredp
//   2007-09-17 - Added addresses of AND, OR, NOT
//   2007-09-18 - Support for OUTPUT, STOP
//   2007-09-19 - Support for IGNORE
//   2007-09-19 - Support for FIRST BUTFIRST
//   2007-09-21 - Support for LAST BUTLAST ITEM
//   2007-09-23 - Support for IF, IFELSE
//    - Support for PARSE, RUNPARSE
//   2007-09-26 - Support for WORDP, LISTP, NUMBERP, EMPTYP
//   2007-09-27 - Support for MEMBERP
//    - Support for WORD, LIST, SENTENCE, SE, LPUT, FPUT
//   2007-09-29 - Support for REPEAT
//   2007-10-10 - Fixed bug #1806484 Wrobg variable's offset
//   2007-10-13 - Support for COUNT, CHAR, ASCII, LOWERCASE, UPPERCASE, MEMBER
//   2007-10-21 - Support for FOREVER
//   2007-10-23 - Support for REPCOUNT
//   2007-10-28 - Support for RANDOM, RERANDOM, ISEQ, RSEQ
//   2007-10-28 - Support for TYPE, SHOW, FORM, FORMAT
//   2007-11-04 - Support for WHILE, UNTIL
//   2007-11-06 - Support for DO.WHILE, DO.UNTIL
//    - Support for DEFINED?, DEFINEDP
//    - Support for PRIMITIVE?, PRIMITIVEP
//    - Support for PROCEDURE?, PROCEDUREP
//    - Support for LOGOPLATFORM, LOGOVERSION, LOGODIALECT
//    - Support for THING
//   2007-11-10 - Fixed bug #1828881 Test case make-05.lgo
//    - Fixed bug #1828897 Problem with indirect MAKE
//   2007-11-11 - Support for WAIT and BYE 
//    - Support for NAME
//   2007-11-12 - Support for CATCH, THROW
//   2007-11-21 - Support for TEST, IFTRUE, IFFALSE
//   2007-12-02 - Support for BACKSLASHED?
//   2007-12-03 - Support for TEXT, FULLTEXT
//   2007-12-15 - Support for LSHIFT, ASHIFT, BITAND, BITOR, BITXOR, BITNOT
//   2007-12-16 - Support for PICK, REMDUP, REMOVE, REVERSE, 
//                   RAWASCII, GENSYM, SUBSTRINGP
//   2007-12-22 - Support for RUN
//   2007-12-30 - Support for COMBINE
//   2008-01-07 - Fixed bug #1856864 Defined but unset variables
//    - Support for RUNRESULT
//   2008-03-22 - Support for QUOTED
//   2008-08-17 - Support for FIRSTS, BUTFIRSTS, BFS
//   2008-08-23 - Support for FOR
//   2008-09-05 - Support for LIBLOAD, LIBFREE
//              - Support for PACKSIZE, PACK, UNPACK
//   2008-09-09 - Support for EXTERNAL
//   2008-09-18 - Support for INTERNAL
//   2008-11-21 - Support for PACKOPEN, PACKCLOSE, PACKREAD, PACKWRITE
//   2009-05-16 - Support for ABS
//   2009-05-18 - Support for COMMANDLINE
//   2009-06-03 - Support for READCHAR, READCHARS, GETENV, GETENVS
//   2009-08-03 - Fixed bug #2191139 Errors/exits in internal functions
//    - Support for READRAWLINE
//   2009-08-04 - Support for READWORD, READLIST
//   2009-08-16 - Fixed bug #2838617 RUN suppresses STOP
//   2009-08-19 - Support for MAYBEOUTPUT
//   2009-08-20 - Support for _STACKFRAME, _STACKFRAMEATOM
//   2009-08-29 - Fixed bug #2845099 Memory leak in THROW
//    - Fixed bug #2845579 DEBUG_MEMORY_LEAKS finds two bad tests
//    - Fixed bug #2846281 Errors not (de)used correctly
//   2010-06-25 - Support for EOF?,EOFP
//   2011-02-05 - Renamed PACKOPEN, PACKCLOSE, PACKREAD, PACKWRITE
//   2011-02-28 - Fixed bug #3195253 product 2 7
//   2011-03-02 - Fixed bug #3197401 Warnings and errors compiling the developer's documentation
//   2011-04-16 - Fixed bug #3279870 sum 1 2
//   2011-10-14 - Support for RUNMACRO
//   2011-10-22 - Fixed bug #3427251 Indirect MAKE and LOCAL cause a crash
//   2011-10-29 - Fixed bug #3427254 Indirect MAKE and LOCAL in RUNMACRO cause a crash
//   2011-12-26 - Support for SUBSTRING
//   2012-01-01 - Fixed bug #3468189 Doxygen warnings
//   2012-01-01 - Adding synonyms for FIRST FIRSTS LAST LASTS
//   2012-01-02 - Command-line options are translatable
//   2012-01-05 - More synonyms for primitives
//   2012-01-19 - Support for CURRENTFOLDER, MAKEFOLDER
//   2012-01-20 - Support for ERASEFOLDER, FOLDER?, FOLDERP, RENAMEFOLDER
//   2012-01-21 - Support for FOLDERS, CHANGEFOLDER
//   2012-01-23 - Support for FILES, ERASEFILE, FILE?, FILEP, RENAMEFILE
//                FILESIZE, FILETIMES, FORMATTIME
//   2012-01-24 - Support for OPENREAD, OPENWRITE, OPENAPPEND, OPENUPDATE
//                SETREAD, SETWRITE
//   2012-01-25 - Support for READER, WRITER, ALLOPEN
//   2012-01-27 - Support for CLOSEALL, READPOS, SETREADPOS, WRiTEPOS, SETWRITEPOS
//   2012-01-31 - Support for TIMEZONE
//   2012-02-05 - Support for READINBLOCK
//   2012-02-09 - Support for DRIBBLE, NODRIBBLE
//   2012-02-26 - Fixed bug #3493871 Externals do not capture errors
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <assert.h>  // assert()
#include <stdio.h>
#include "globals.h"
#include "atoms.h"
#include "words.h"
#include "lists.h"
#include "numbers.h"
#include "unicode.h"
#include "parser.h"
#include "translate.h"
#include "vars.h"
#include "pools.h"
#include "options.h"
#include "errors.h"
#include "runtime.h"
#include "compiler.h"


atom_t root;    ///< parent of all variables
atom_t word_to;    ///< word containing "TO" token
atom_t word_to_syn;    ///< word containing a synonym of "TO" token
atom_t word_end;  ///< word containing "END" token
atom_t word_run;        ///< word containing "RUN" token
atom_t word_make;       ///< word containing "MAKE" token
atom_t word_plus;       ///< word containing "+" token
atom_t word_toplevel;  ///< word containing "TOPLEVEL" token
atom_t word_system;  ///< word containing "SYSTEM" token
atom_t word_error;  ///< word containing "ERROR" token

atom_t unbound;    ///< atom representing unbound values
atom_t stopped;    ///< atom representing stop values

atom_t repeat_chain;  ///< atom for the topmost repeat chain
atom_t globals;    ///< collection of global system-wide persistent variables

atom_t printdepthlimit; ///< variable PRINTDEPTHLIMIT
atom_t printwidthlimit; ///< variable PRINTWIDTHLIMIT
atom_t fullprintp;  ///< variable FULLPRINTP
atom_t caseignoredp;  ///< variable CASEIGNOREDP

atom_t last_error;  ///< last caught error (used by ERROR primitive)
atom_t delayed_free;    ///< atoms sheduled for delayed free

int root_frame;         ///< frame of root
int backup_frame;       ///< run-time backup of frame
atom_t error_flag;  ///< error flag (used by internal/external functions)
int catch_output_flag;  ///< 1 = there was output/stop in catch

//===================================================
/// \struct    prim_t vars.c
/// \brief    description of a primitive
///
/// This structure describes the properties of primitive
/// functions, commands, and operators. The actual
/// properties are defined in ::vars[] array.
//===================================================

typedef struct prim_t {
  fn  function;  ///< function address
  short  largs;    ///< number of left arguments
  short rargs;    ///< number of right arguments
  int   flags;    ///< additional VAR_ID flags
} primitive_t;    ///< description structure of a primitive



#define FLAG_EX_PRINTDEPTHLIMIT  0x00010000 ///< extended flag for PRINTDEPTHLIMIT
#define FLAG_EX_PRINTWIDTHLIMIT  0x00020000 ///< extended flag for PRINTWIDTHLIMIT
#define FLAG_EX_FULLPRINTP  0x00040000 ///< extended flag for FULLPRINTP
#define FLAG_EX_CASEIGNOREDP  0x00080000 ///< extended flag for CASEIGNOREDP
#define FLAG_EX_LOGOPLATFORM  0x00100000 ///< extended flag for LOGOPLATFORM
#define FLAG_EX_LOGOVERSION  0x00200000 ///< extended flag for LOGOVERSION
#define FLAG_EX_LOGODIALECT  0x00400000 ///< extended flag for LOGODIALECT

// list of all primitives
primitive_t vars[] =
  {
    // multiplicative operators
    {(fn)rt_mul,      1, 1, FLAG_FUNCTION|FLAG_PRIORITY_MUL}, // *
    {(fn)rt_div,      1, 1, FLAG_FUNCTION|FLAG_PRIORITY_MUL}, // /

    // additive operators
    {(fn)rt_plus,     1, 1, FLAG_FUNCTION|FLAG_CAN_BE_UNARY|FLAG_PRIORITY_ADD}, // +
    {(fn)rt_minus,    1, 1, FLAG_FUNCTION|FLAG_CAN_BE_UNARY|FLAG_PRIORITY_ADD}, // -

    // comare operators
    {(fn)rt_equal,    1, 1, FLAG_FUNCTION|FLAG_PRIORITY_CMP|FLAG_EQUAL_VARS}, // =
    {(fn)rt_less,     1, 1, FLAG_FUNCTION|FLAG_PRIORITY_CMP}, // <
    {(fn)rt_more,     1, 1, FLAG_FUNCTION|FLAG_PRIORITY_CMP}, // >
    {(fn)rt_lesseq,   1, 1, FLAG_FUNCTION|FLAG_PRIORITY_CMP}, // <=
    {(fn)rt_moreeq,   1, 1, FLAG_FUNCTION|FLAG_PRIORITY_CMP}, // >=
    {(fn)rt_nequal,   1, 1, FLAG_FUNCTION|FLAG_PRIORITY_CMP|FLAG_EQUAL_VARS}, // <>

     // logical operators and functions
    {(fn)rt_and,      0, 2, FLAG_FUNCTION|FLAG_PRIORITY_LOG|FLAG_INFINITE_ARGS}, // and
    {(fn)rt_and,      0, 2, FLAG_FUNCTION|FLAG_PRIORITY_LOG|FLAG_INFINITE_ARGS}, // and
    {(fn)rt_or,       0, 2, FLAG_FUNCTION|FLAG_PRIORITY_LOG|FLAG_INFINITE_ARGS}, // or
    {(fn)rt_or,       0, 2, FLAG_FUNCTION|FLAG_PRIORITY_LOG|FLAG_INFINITE_ARGS}, // or
    {(fn)rt_not,      0, 1, FLAG_FUNCTION}, // not

    // primitive functions SELECTORS
    {(fn)rt_first,    0, 1, FLAG_FUNCTION}, // first
    {(fn)rt_first,    0, 1, FLAG_FUNCTION}, // first (synonym)
    {(fn)rt_butfirst, 0, 1, FLAG_FUNCTION}, // butfirst
    {(fn)rt_butfirst, 0, 1, FLAG_FUNCTION}, // bf
    {(fn)rt_firsts,   0, 1, FLAG_FUNCTION}, // firsts
    {(fn)rt_firsts,   0, 1, FLAG_FUNCTION}, // firsts (synonym)
    {(fn)rt_butfirsts,0, 1, FLAG_FUNCTION}, // butfirsts
    {(fn)rt_butfirsts,0, 1, FLAG_FUNCTION}, // bfs
    {(fn)rt_last,     0, 1, FLAG_FUNCTION}, // last
    {(fn)rt_last,     0, 1, FLAG_FUNCTION}, // last (synonym)
    {(fn)rt_butlast,  0, 1, FLAG_FUNCTION}, // butlast
    {(fn)rt_butlast,  0, 1, FLAG_FUNCTION}, // bl
    {(fn)rt_item,     0, 2, FLAG_FUNCTION}, // item
    {(fn)rt_item,     0, 2, FLAG_FUNCTION}, // item

    // primitive functions CONSTRUCTORS
    {(fn)rt_word,     0, 2, FLAG_FUNCTION|FLAG_INFINITE_ARGS}, // word
    {(fn)rt_list,     0, 2, FLAG_FUNCTION|FLAG_INFINITE_ARGS}, // list
    {(fn)rt_sent,     0, 2, FLAG_FUNCTION|FLAG_INFINITE_ARGS}, // sentence
    {(fn)rt_sent,     0, 2, FLAG_FUNCTION|FLAG_INFINITE_ARGS}, // se
    {(fn)rt_fput,     0, 2, FLAG_FUNCTION}, // fput
    {(fn)rt_fput,     0, 2, FLAG_FUNCTION}, // fput
    {(fn)rt_lput,     0, 2, FLAG_FUNCTION}, // lput
    {(fn)rt_lput,     0, 2, FLAG_FUNCTION}, // lput

    // primitive functions PREDICATES
    {(fn)rt_wordp,    0, 1, FLAG_FUNCTION}, // word?
    {(fn)rt_wordp,    0, 1, FLAG_FUNCTION}, // wordp
    {(fn)rt_listp,    0, 1, FLAG_FUNCTION}, // list?
    {(fn)rt_listp,    0, 1, FLAG_FUNCTION}, // listp
    {(fn)rt_numberp,  0, 1, FLAG_FUNCTION}, // number?
    {(fn)rt_numberp,  0, 1, FLAG_FUNCTION}, // numberp
    {(fn)rt_empty,    0, 1, FLAG_FUNCTION}, // empty?
    {(fn)rt_empty,    0, 1, FLAG_FUNCTION}, // emptyp
    {(fn)rt_equal,    0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // equal?
    {(fn)rt_equal,    0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // equalp
    {(fn)rt_nequal,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // notequal?
    {(fn)rt_nequal,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // notequalp
    {(fn)rt_before,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // before?
    {(fn)rt_before,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // beforep
    {(fn)rt_less,     0, 2, FLAG_FUNCTION}, // less?
    {(fn)rt_less,     0, 2, FLAG_FUNCTION}, // lessp
    {(fn)rt_more,     0, 2, FLAG_FUNCTION}, // greater?
    {(fn)rt_more,     0, 2, FLAG_FUNCTION}, // greaterp
    {(fn)rt_lesseq,   0, 2, FLAG_FUNCTION}, // lessequal?
    {(fn)rt_lesseq,   0, 2, FLAG_FUNCTION}, // lessequalp
    {(fn)rt_moreeq,   0, 2, FLAG_FUNCTION}, // greaterequal?
    {(fn)rt_moreeq,   0, 2, FLAG_FUNCTION}, // greaterequalp
    {(fn)rt_memberp,  0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // member?
    {(fn)rt_memberp,  0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // member?
    {(fn)rt_memberp,  0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // memberp
    {(fn)rt_memberp,  0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // memberp

    // queries
    {(fn)rt_count,    0, 1, FLAG_FUNCTION}, // count
    {(fn)rt_count,    0, 1, FLAG_FUNCTION}, // count
    {(fn)rt_char,     0, 1, FLAG_FUNCTION}, // char
    {(fn)rt_ascii,    0, 1, FLAG_FUNCTION}, // ascii
    {(fn)rt_lower,    0, 1, FLAG_FUNCTION}, // lowercase
    {(fn)rt_lower,    0, 1, FLAG_FUNCTION}, // lowercase
    {(fn)rt_upper,    0, 1, FLAG_FUNCTION}, // uppercase
    {(fn)rt_upper,    0, 1, FLAG_FUNCTION}, // uppercase
    {(fn)rt_member,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // member
    {(fn)rt_parse,    0, 1, FLAG_FUNCTION}, // parse
    {(fn)rt_runparse, 0, 1, FLAG_FUNCTION}, // runparse

    // arithmetic
    {(fn)rt_sum,        0, 2, FLAG_FUNCTION | FLAG_INFINITE_ARGS}, // sum
    {(fn)rt_difference, 0, 2, FLAG_FUNCTION}, // difference
    {(fn)rt_unminus,    0, 1, FLAG_FUNCTION}, // minus
    {(fn)rt_product,    0, 2, FLAG_FUNCTION | FLAG_INFINITE_ARGS}, // product
    {(fn)rt_div,        0, 2, FLAG_FUNCTION}, // div
    {(fn)rt_remainder,  0, 2, FLAG_FUNCTION}, // remainder
    {(fn)rt_int,        0, 1, FLAG_FUNCTION}, // int
    {(fn)rt_round,      0, 1, FLAG_FUNCTION}, // round
    {(fn)rt_sqrt,       0, 1, FLAG_FUNCTION}, // sqrt
    {(fn)rt_power,      0, 2, FLAG_FUNCTION}, // power
    {(fn)rt_exp,        0, 1, FLAG_FUNCTION}, // exp
    {(fn)rt_log10,      0, 1, FLAG_FUNCTION}, // log10
    {(fn)rt_ln,         0, 1, FLAG_FUNCTION}, // ln
    {(fn)rt_abs,        0, 1, FLAG_FUNCTION}, // abs

    // trigonometric
    {(fn)rt_pi,         0, 0, FLAG_FUNCTION}, // pi
    {(fn)rt_sin,        0, 1, FLAG_FUNCTION}, // sin
    {(fn)rt_radsin,     0, 1, FLAG_FUNCTION}, // radsin
    {(fn)rt_cos,        0, 1, FLAG_FUNCTION}, // cos
    {(fn)rt_radcos,     0, 1, FLAG_FUNCTION}, // radcos
    {(fn)rt_arctan,     0, 1, FLAG_FUNCTION | FLAG_MAY_HAVE_EXTRA_ARG}, // arctan
    {(fn)rt_radarctan,  0, 1, FLAG_FUNCTION | FLAG_MAY_HAVE_EXTRA_ARG}, // radarctan
    //    {(fn)rt_arctanxy,  0, 2, FLAG_FUNCTION}, // arctanxy
    //    {(fn)rt_radarctanxy,  0, 2, FLAG_FUNCTION}, // radarctanxy

    // sequences and random numbers
    {(fn)rt_iseq,       0, 2, FLAG_FUNCTION}, // iseq
    {(fn)rt_rseq,       0, 3, FLAG_FUNCTION}, // rseq
    {(fn)rt_random,     0, 1, FLAG_FUNCTION | FLAG_MAY_HAVE_EXTRA_ARG}, // random
    {(fn)rt_random,     0, 1, FLAG_FUNCTION | FLAG_MAY_HAVE_EXTRA_ARG}, // random
    {(fn)rt_rerandom,   0, 0, FLAG_COMMAND | FLAG_MAY_HAVE_EXTRA_ARG},  // rerandom
    {(fn)rt_rerandom,   0, 0, FLAG_COMMAND | FLAG_MAY_HAVE_EXTRA_ARG},  // rerandom
    {(fn)rt_pick,       0, 1, FLAG_FUNCTION }, // pick

    // bit operations
    {(fn)rt_lshift,     0, 2, FLAG_FUNCTION}, // lshift
    {(fn)rt_ashift,     0, 2, FLAG_FUNCTION}, // ashift
    {(fn)rt_bitand,     0, 2, FLAG_FUNCTION | FLAG_INFINITE_ARGS}, // bitand
    {(fn)rt_bitor,      0, 2, FLAG_FUNCTION | FLAG_INFINITE_ARGS}, // bitor
    {(fn)rt_bitxor,     0, 2, FLAG_FUNCTION | FLAG_INFINITE_ARGS}, // bitxor
    {(fn)rt_bitnot,     0, 1, FLAG_FUNCTION }, // bitnot
    
    // transmitter commands
    {(fn)rt_print,      0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_PRINT_VARS}, // print
    {(fn)rt_print,      0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_PRINT_VARS}, // pr
    {(fn)rt_print,      0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_PRINT_VARS}, // ?
    {(fn)rt_show,       0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_PRINT_VARS}, // show
    {(fn)rt_type,       0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_PRINT_VARS}, // type
    {(fn)rt_type,       0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_PRINT_VARS}, // type
    {(fn)rt_form,       0, 3, FLAG_FUNCTION}, // form
    {(fn)rt_format,     0, 2, FLAG_FUNCTION}, // format
    {(fn)rt_formattime, 0, 2, FLAG_FUNCTION}, // formattime
    {(fn)rt_timezone,   0, 0, FLAG_FUNCTION}, // timezone

    // primitive commands
    {(fn)rt_if,         0, 3, FLAG_FUNCTION | FLAG_COMMAND | FLAG_MAY_SKIP_LAST_ARG | FLAG_PROCESS_ARGS}, // if
    {(fn)rt_if,         0, 3, FLAG_FUNCTION | FLAG_COMMAND | FLAG_PROCESS_ARGS}, // ifelse
    {(fn)rt_repeat,     0, 2, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // repeat
    {(fn)rt_repeat,     0, 2, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // repeat
    {(fn)rt_repcount,   0, 0, FLAG_FUNCTION | FLAG_PUSH_FRAME}, // repcount
    {(fn)rt_while,      0, 2, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // while
    {(fn)rt_dowhile,    0, 2, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // do.while
    {(fn)rt_until,      0, 2, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // until
    {(fn)rt_dountil,    0, 2, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // do.until
    {(fn)rt_run,        0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT | FLAG_PUSH_MODE}, // run
    {(fn)rt_forever,    0, 1, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // forever
    {(fn)rt_ignore,     0, 1, FLAG_COMMAND}, // ignore
    {(fn)rt_stop,       0, 0, FLAG_COMMAND}, // stop
    {(fn)rt_stop,       0, 0, FLAG_COMMAND}, // stop
    {(fn)rt_make,       0, 2, FLAG_COMMAND | FLAG_SET_ONE_VAR | FLAG_PUSH_PARENT}, // make
    {(fn)rt_name,       0, 2, FLAG_COMMAND | FLAG_PUSH_PARENT}, // name
    {(fn)rt_output,     0, 1, FLAG_COMMAND}, // output
    {(fn)rt_output,     0, 1, FLAG_COMMAND}, // op
    {(fn)rt_maybeoutput,0, 1, FLAG_COMMAND}, // maybeoutput
    {(fn)rt_local,      0, 1, FLAG_COMMAND | FLAG_INFINITE_ARGS | FLAG_SET_ALL_VARS | FLAG_PUSH_PARENT }, // local
    {(fn)rt_thing,      0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // thing
    {(fn)rt_bye,        0, 0, FLAG_COMMAND}, // bye
    {(fn)rt_wait,       0, 1, FLAG_COMMAND}, // wait
    {(fn)rt_tag,        0, 1, FLAG_COMMAND}, // tag
    {(fn)rt_goto,       0, 1, FLAG_COMMAND  | FLAG_PUSH_PARENT}, // goto
    {(fn)rt_definedp,   0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // defined?
    {(fn)rt_definedp,   0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // definedp
    {(fn)rt_primitivep, 0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // primitive?
    {(fn)rt_primitivep, 0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // primitivep
    {(fn)rt_namep,      0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // name?
    {(fn)rt_namep,      0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // namep
    {(fn)rt_procedurep, 0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // procedure?
    {(fn)rt_procedurep, 0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // procedurep
    {(fn)rt_for,        0, 3, FLAG_COMMAND  | FLAG_SET_ONE_VAR | FLAG_PROCESS_ARGS}, // for
    {NULL,              0, 2, FLAG_COMMAND}, // case
    {NULL,              0, 1, FLAG_COMMAND}, // cond
    {(fn)rt_text,       0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // text
    {(fn)rt_fulltext,   0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // fulltext
    {(fn)rt_define,     0, 2, FLAG_COMMAND  | FLAG_PUSH_PARENT}, // define
    {(fn)rt_define,     0, 2, FLAG_COMMAND  | FLAG_PUSH_PARENT}, // define
    {(fn)rt_load,       0, 1, FLAG_COMMAND | FLAG_PUSH_PARENT | FLAG_PUSH_MODE}, // load
    {(fn)rt_commandline,0, 0, FLAG_FUNCTION }, // commandline

    // misc
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_PRINTDEPTHLIMIT}, // printdepthlimit
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_PRINTWIDTHLIMIT}, // printwidthlimit
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_FULLPRINTP},      // fullprintp
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_CASEIGNOREDP},    // caseignoredp
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_LOGOPLATFORM},    // logoplatform
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_LOGOVERSION},     // logoversion
    {NULL,  0, 0, FLAG_VARIABLE | FLAG_EX_LOGODIALECT},     // logodialect

    // error handling
    {(fn)rt_catch,    0, 2, FLAG_FUNCTION | FLAG_PROCESS_ARGS},  // catch
    {(fn)rt_throw,    0, 1, FLAG_FUNCTION | FLAG_MAY_HAVE_EXTRA_ARG},  // throw
    {(fn)rt_error,    0, 0, FLAG_FUNCTION},        // error

    {NULL,  0, 2, FLAG_COMMAND}, // apply

    {(fn)rt_test,     0, 1, FLAG_COMMAND | FLAG_PUSH_FRAME }, // test
    {(fn)rt_iftrue,   0, 1, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // iftrue
    {(fn)rt_iftrue,   0, 1, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // ift
    {(fn)rt_iffalse,  0, 1, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // iffalse
    {(fn)rt_iffalse,  0, 1, FLAG_COMMAND | FLAG_PROCESS_ARGS}, // iff

    {(fn)rt_backslashedp, 0, 1, FLAG_FUNCTION}, // backslashed?
    {(fn)rt_backslashedp, 0, 1, FLAG_FUNCTION}, // backslashedp

    {(fn)rt_remdup,       0, 1, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // remdup
    {(fn)rt_remove,       0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // remove
    {(fn)rt_reverse,      0, 1, FLAG_FUNCTION}, // reverse
    {(fn)rt_rawascii,     0, 1, FLAG_FUNCTION}, // rawascii
    {(fn)rt_substringp,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // substring?
    {(fn)rt_substringp,   0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // substringp
    {(fn)rt_substring,    0, 2, FLAG_FUNCTION | FLAG_EQUAL_VARS}, // substring
    {(fn)rt_gensym,       0, 0, FLAG_FUNCTION}, // gensym
    {(fn)rt_combine,      0, 2, FLAG_FUNCTION}, // combine
    {(fn)rt_quoted,       0, 1, FLAG_FUNCTION}, // quoted

    {(fn)rt_runresult,    0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT}, // runresult
    {(fn)rt_runmacro,     0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT | FLAG_PUSH_MODE}, // runmacro

    // internal access
    {(fn)rt_libload,        0, 1, FLAG_FUNCTION}, // libload
    {(fn)rt_libfree,        0, 1, FLAG_COMMAND},  // libfree
    {(fn)rt_blocksize,      0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT},  // packsize
    {(fn)rt_listtoblock,    0, 2, FLAG_FUNCTION | FLAG_PUSH_PARENT},  // listtoblock
    {(fn)rt_blocktolist,    0, 2, FLAG_FUNCTION | FLAG_PUSH_PARENT},  // blocktolist
    {(fn)rt_dataaddr,       0, 1, FLAG_FUNCTION},      // dataaddr
    {(fn)rt_listintoblock,  0, 3, FLAG_COMMAND | FLAG_PUSH_PARENT},  // listintoblock
    {(fn)rt_funcaddr,       0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT},  // funcaddr
    {(fn)rt_openfile,       0, 2, FLAG_FUNCTION | FLAG_PUSH_MODE},      // openfile
    {(fn)rt_closefile,      0, 1, FLAG_COMMAND},      // closefile
    {(fn)rt_readblock,      0, 1, FLAG_FUNCTION | FLAG_PUSH_PARENT},      // readblock
    {(fn)rt_readinblock,    0, 1, FLAG_FUNCTION | FLAG_PUSH_MODE},      // readinblock
    {(fn)rt_writeblock,     0, 1, FLAG_COMMAND},      // writeblock
    {(fn)rt_external,       0, 3, FLAG_COMMAND | FLAG_PUSH_PARENT},  // external
    {(fn)rt_internal,       0, 2, FLAG_COMMAND | FLAG_PUSH_PARENT},  // internal
    {(fn)rt_int3,           0, 0, FLAG_COMMAND},      // _int3
    {(fn)rt_stackframe,     0, 2, FLAG_FUNCTION | FLAG_PUSH_PARENT},  // _stackframe
    {(fn)rt_stackframeatom, 0, 2, FLAG_FUNCTION | FLAG_PUSH_PARENT},  // _stackframeatom

    // text i/o
    {(fn)rt_readchar,     0, 0, FLAG_FUNCTION},  // readchar
    {(fn)rt_readchar,     0, 0, FLAG_FUNCTION},  // rc
    {(fn)rt_readchars,    0, 1, FLAG_FUNCTION},  // readchars
    {(fn)rt_readchars,    0, 1, FLAG_FUNCTION},  // rcs
    {(fn)rt_readrawline,  0, 0, FLAG_FUNCTION},  // readrawline
    {(fn)rt_readword,     0, 0, FLAG_FUNCTION},  // readword
    {(fn)rt_readword,     0, 0, FLAG_FUNCTION},  // rw
    {(fn)rt_readlist,     0, 0, FLAG_FUNCTION},  // readlist
    {(fn)rt_readlist,     0, 0, FLAG_FUNCTION},  // rl

    // environment
    {(fn)rt_getenv,    0, 1, FLAG_FUNCTION},  // getenv
    {(fn)rt_getenvs,   0, 0, FLAG_FUNCTION},  // getenvs
    {(fn)rt_eofp,      0, 0, FLAG_FUNCTION},  // eof?
    {(fn)rt_eofp,      0, 0, FLAG_FUNCTION},  // eofp

    // files and folders
    {(fn)rt_currentfolder, 0, 0, FLAG_FUNCTION},  // currentfolder
    {(fn)rt_makefolder,    0, 1, FLAG_COMMAND},   // makefolder
    {(fn)rt_erasefolder,   0, 1, FLAG_COMMAND},   // erasefolder
    {(fn)rt_folderp,       0, 1, FLAG_FUNCTION},  // folder?
    {(fn)rt_folderp,       0, 1, FLAG_FUNCTION},  // folderp
    {(fn)rt_renamefolder,  0, 2, FLAG_COMMAND},   // renamefolder
    {(fn)rt_folders,       0, 1, FLAG_FUNCTION},  // folders
    {(fn)rt_changefolder,  0, 1, FLAG_COMMAND},   // changefolder
    {(fn)rt_files,         0, 1, FLAG_FUNCTION},  // files
    {(fn)rt_erasefile,     0, 1, FLAG_COMMAND},   // erasefile
    {(fn)rt_erasefile,     0, 1, FLAG_COMMAND},   // erf
    {(fn)rt_filep,         0, 1, FLAG_FUNCTION},  // file?
    {(fn)rt_filep,         0, 1, FLAG_FUNCTION},  // filep
    {(fn)rt_renamefile,    0, 2, FLAG_COMMAND},   // renamefile
    {(fn)rt_filesize,      0, 1, FLAG_FUNCTION},  // filesize
    {(fn)rt_filetimes,     0, 1, FLAG_FUNCTION},  // filetimes
    {(fn)rt_openread,      0, 1, FLAG_FUNCTION | FLAG_PUSH_MODE},  // openread
    {(fn)rt_openwrite,     0, 1, FLAG_FUNCTION | FLAG_PUSH_MODE},  // openwrite
    {(fn)rt_openappend,    0, 1, FLAG_FUNCTION | FLAG_PUSH_MODE},  // openappend
    {(fn)rt_openupdate,    0, 1, FLAG_FUNCTION | FLAG_PUSH_MODE},  // openupdate
    {(fn)rt_setread,       0, 1, FLAG_COMMAND},   // setread
    {(fn)rt_setwrite,      0, 1, FLAG_COMMAND},   // setwrite
    {(fn)rt_reader,        0, 0, FLAG_FUNCTION},  // reader
    {(fn)rt_writer,        0, 0, FLAG_FUNCTION},  // writer
    {(fn)rt_allopen,       0, 0, FLAG_FUNCTION},  // allopen
    {(fn)rt_closeall,      0, 0, FLAG_COMMAND},   // closeall
    {(fn)rt_readpos,       0, 0, FLAG_FUNCTION},  // readpos
    {(fn)rt_setreadpos,    0, 1, FLAG_COMMAND},   // setreadpos
    {(fn)rt_writepos,      0, 0, FLAG_FUNCTION},  // writepos
    {(fn)rt_setwritepos,   0, 1, FLAG_COMMAND},   // setwritepos
    {(fn)rt_dribble,       0, 1, FLAG_COMMAND},   // dribble
    {(fn)rt_nodribble,     0, 0, FLAG_COMMAND},   // nodribble


    {NULL,  -1,-1, -1}
  }; ///< array with actual properties of primitives




//===================================================
/// \fn    void init_vars( )
/// \brief  initializes variables
///
/// Initializes the variables module by creating the
/// system root variable which is named as defined by
/// ROOT_VAR_NAME, and globals variable defined by 
//  GLOBALS_VAR_NAME. Then the primitives are created
/// as local variables to the root variable. The names
/// of primitives are taken from ::TR_PRIMITIVES, the
/// properties are taken from ::vars[]. Two words :to
/// and :end are created.
//===================================================
void init_vars( )
{
  #ifdef DEBUG_VAR
  printf("<VAR> Vars initialized\n");
  #endif //DEBUG_VAR

  // create the root variable
  atom_t name = new_word( ROOT_VAR_NAME, -1 );
  root = new_var( name, 0, 0 );
  need_descr2( root );
  SET_FLAGS( root, FLAG_FUNCTION );
  LARGS( root ) = 0;
  RARGS( root ) = 0;
  LEVEL( root ) = 0;
  PRIORITY( root ) = PRIORITY_CMD;
  DEUSE( name );

  // create the globals variable
  name = new_word( GLOBALS_VAR_NAME, -1 );
  globals = new_var( name, 0, 0 );
  SET_FLAGS( globals, FLAG_VARIABLE );
  LARGS( root ) = 0;
  RARGS( root ) = 0;
  LEVEL( root ) = 0;
  PRIORITY( root ) = PRIORITY_CMD;
  DEUSE( name );

  unbound = new_integer( 0 );
  stopped = new_integer( 1 );

  repeat_chain = new_list( new_integer(0), empty_list );
  REPCOUNT( CAR(repeat_chain) ) = -1;

  last_error = empty_list;
  delayed_free = empty_list;

  atom_t names = new_word( TR_PRIMITIVES, UNKNOWN );
  atom_t tokens = tokenize( names, TOKENIZE_DATA );

  // setting TO, END and other words
  atom_t t = tokens; 
  false_true[0] = USE(CAR(t)); t = CDR(t);
  false_true[1] = USE(CAR(t)); t = CDR(t);
  word_to       = USE(CAR(t)); t = CDR(t);
  word_to_syn   = USE(CAR(t)); t = CDR(t);
  word_end      = USE(CAR(t)); t = CDR(t);
  word_toplevel = USE(CAR(t)); t = CDR(t);
  word_system   = USE(CAR(t)); t = CDR(t);
  word_error    = USE(CAR(t)); t = CDR(t);
  //word_run -- set in the next FOR-cycle

  // create primitives
  int i;
  for( i = 0; vars[i].largs>=0; i++, t=CDR(t) )
    {
      #ifdef SAFEMODE
        assert( IS_NOT_EMPTY(t) ); // too few words in TR_PRIMITIVES
      #endif

      //printf("%d ",i); dumpln(CAR(t));
      atom_t name = CAR(t);
      if( LENGTH(name) )
      {
        atom_t var = new_var( name, globals, 1 );
        ADDRESS( var ) = (int_t)vars[i].function;
        LEVEL( var ) = 1;
        LARGS( var ) = vars[i].largs;
        RARGS( var ) = vars[i].rargs;

        if( vars[i].flags & FLAG_PRIORITY_MUL )
          PRIORITY( var ) = PRIORITY_MUL;
        else if( vars[i].flags & FLAG_PRIORITY_ADD )
          PRIORITY( var ) = PRIORITY_ADD;
        else if( vars[i].flags & FLAG_PRIORITY_LOG )
          PRIORITY( var ) = PRIORITY_LOG;
        else if( vars[i].flags & FLAG_PRIORITY_CMP )
          PRIORITY( var ) = PRIORITY_CMP;
        else if( vars[i].flags & FLAG_COMMAND )
          PRIORITY( var ) = PRIORITY_CMD;
        else
          PRIORITY( var ) = PRIORITY_FUN;

        SET_FLAGS( var, (vars[i].flags&ALL_VAR_FLAGS) | FLAG_PRIMITIVE );

        // initialize system variables to (unbound)
        if( IS_VARIABLE(var) )
          {
            if( vars[i].flags & FLAG_EX_PRINTDEPTHLIMIT ) printdepthlimit = var;
            if( vars[i].flags & FLAG_EX_PRINTWIDTHLIMIT ) printwidthlimit = var;
            if( vars[i].flags & FLAG_EX_FULLPRINTP      ) fullprintp = var;
            if( vars[i].flags & FLAG_EX_CASEIGNOREDP    ) caseignoredp = var;

            if( vars[i].flags & FLAG_EX_LOGOPLATFORM )
              VALUE(var) = new_word(LOGO_PLATFORM,-1);
            else if( vars[i].flags & FLAG_EX_LOGOVERSION )
              VALUE(var) = new_word(LOGO_VERSION,-1);
            else if( vars[i].flags & FLAG_EX_LOGODIALECT )
              VALUE(var) = new_word(LOGO_DIALECT,-1);
            else
              VALUE(var) = USE( unbound );

            VARTYPE(var) = VAR_TYPE_RUNTIME; // value is in var's atom
          }
        
        //printf("{%d",REF(word_run));      
        if( ADDRESS(var) == (int_t)rt_run  ) word_run  = USE(name);
        if( ADDRESS(var) == (int_t)rt_make ) word_make = USE(name);
        if( ADDRESS(var) == (int_t)rt_plus ) word_plus = USE(name);
        //printf("%d}\n",REF(word_run));      
      }
    }

  #ifdef SAFEMODE
    assert( IS_EMPTY(t) ); // too many words in TR_PRIMITIVES
  #endif

  DEUSE( names );
  DEUSE( tokens );
}




//===================================================
/// \fn    void finit_vars( ) 
/// \brief  finalizes the Varss module
///
/// Finilizes the Vars module by freeing :to, :end and
/// :root atoms. Deleting :root will recursively delete
/// all other variables including the primitives.
///
/// Some system primitive variables like fullprintp,
/// printwidthlimit, and printdepthlimit does not need
/// individual finalization, because they are included
/// in globals and finalized when globals is finalized.
//===================================================
void finit_vars( )
{
  DEUSE( globals );
  DEUSE( root );
  DEUSE( repeat_chain );

  if( REF(last_error)>1 ) REF(last_error)=1; // patch

#ifdef SAFEMODE
  //printf( "REF(unbound)=%d\n",REF(unbound) );
  //printf( "REF(false_true[0])=%d\n",REF(false_true[0]) );
  //printf( "REF(last_error=%x)=%d\n",(int)last_error,REF(last_error) );
  //printf( "REF(word_run=%x)=%d\n",(int)word_run,REF(word_run) );
  assert( REF(unbound)==1 );
  assert( REF(stopped)==1 );
  assert( REF(word_error)==1 );
  assert( REF(word_system)==1 );
  assert( REF(word_toplevel)==1 );
  assert( REF(word_to)==1 );
  assert( REF(word_to_syn)==1 );
  assert( REF(word_end)==1 );
  assert( REF(false_true[0])==1 );
  assert( REF(false_true[1])==1 );
  assert( REF(word_run)==1 );
  assert( REF(word_make)==1 );
  assert( REF(word_plus)==1 );
  assert( REF(last_error)==1 );
#endif

  // IMPORTANT! The unbound atom is considered
  // not use-able amd deuse-able, thus its reference
  // count is always 1. Because of this DEUSE will
  // not automatically delete it, thus we delete is
  // manually and adjust statistics manually.
  DEUSE( unbound );
  DEUSE( stopped );

  #ifdef ADVANCED
  stats[ID(unbound)].deallocs++;
  stats_free++;
  stats[ID(stopped)].deallocs++;
  stats_free++;
  #endif //ADVANCED
  delete_numeric( unbound ); // special case
  delete_numeric( stopped ); // special case
  DEUSE( word_error );
  DEUSE( word_system );
  DEUSE( word_toplevel );
  DEUSE( word_to );
  DEUSE( word_to_syn );
  DEUSE( word_end );
  DEUSE( false_true[0] );
  DEUSE( false_true[1] );
  DEUSE( last_error );
  DEUSE( delayed_free );
  DEUSE( word_run );
  DEUSE( word_make );
  DEUSE( word_plus );

  #ifdef DEBUG_VAR
    printf("<VAR> Vars finalized\n");
  #endif //DEBUG_VAR
}




//===================================================
/// \fn    atom_t new_var( atom_t name, atom_t parent, int attach )
/// \brief    creates a new var atom
/// \param  name  word atom for the name of the variable
/// \param  parent  var atom for the parent of the variable
/// \param  attach  1=attach to parent, 0=do not attach
/// \return    var atom
///
/// Creates a var atom describing a variable with given
/// name and parent. The reference count of the var is
/// set to 1, the reference count of \c name is
/// increased, the reference count of \c parent is
/// not changed. The function automatically creates
/// the first descriptor of the var atom. The second
/// descriptor is left uninitialized - it could be
/// later created by \c ::need_descr2() if needed.
///
/// The newly created var atom is included in the
/// list of local variables of the parent only if
/// \c attach != 0. Otherwise var has a parent, but
/// the parent does not know about the child var.
//===================================================
atom_t new_var( atom_t name, atom_t parent, int attach )
{
  #ifdef SAFEMODE
    assert( name );
    assert( IS_WORD(name)||IS_SUBWORD(name)||IS_EMPTY(name) );
    assert( !parent||IS_VARATOM(parent) );
  #endif

  atom_t a = take_from_pool( &data_pool );
  DESCR1(a) = take_from_pool( &data_pool );
  DESCR2(a) = 0;

  REF(a) = 1;
  ID(a)  = VAR_ID;

  NAME(a) = USE(name);
  FLAGS(a) = 0;
  PARENT(a) = parent; // weak link, no ref++
  VARTYPE(a) = VAR_TYPE_NORMAL;
  if( parent )
    {
      need_descr2( parent );
      if( attach ) LOCALS(parent) = new_list( a, LOCALS(parent) );
      LEVEL(a) = LEVEL(parent)+1;
      OFFSET(a) = 0;
    }
  else
    LEVEL(a) = 0;

  #ifdef DEBUG_ATOM
    printf("<ATOM>  [%08x] var="STR"\n",(int)a,STRING(name));
  #endif //DEBUG_ATOM

    #ifdef ADVANCED
    stats[ID(a)].allocs+=2;  // it's 2 because of 
    stats_free-=2;    // the 1st descriptor
    if( stats[ID(a)].max<(stats[ID(a)].allocs-stats[ID(a)].deallocs) )
      stats[ID(a)].max=(stats[ID(a)].allocs-stats[ID(a)].deallocs);
  #endif //ADVANCED


  #ifdef DEBUG_RUNTIME_ATOMS
  if( running_compiled_code )
    {
      outter( TEXT("<RUNTIME> new  "), -1 );
      dump_atom_address( a );
      dump_atom( a, 1 );
      outter( TEXT("\n"), -1 );
    }
  #endif
  #ifdef DEBUG_COMPILETIME_ATOMS
  if( compiling_code )
    {
      outter( TEXT("<COMPILETIME> new  "), -1 );
      dump_atom_address( a );
      dump_atom( a, 1 );
      outter( TEXT("\n"), -1 );
    }
  #endif

  return a;
}




//===================================================
/// \fn    int need_descr2( atom_t var )
/// \brief    creates var descriptor if needed
/// \param  var  var which descriptor will be created
/// \return    1 if descriptor was created
///
/// Creates the second descriptor of a var atom if it
/// does not exist. All the values of the new second
/// descriptor are set to empty lists.
//===================================================
int need_descr2( atom_t var )
{
  // create descr2 if it does not exist
  if( DESCR2(var) ) return 0;
  DESCR2(var) = take_from_pool( &data_pool );

  // initialize descr2
  atom_t def = empty_list;
  def = new_list( empty_list, def ); // TREE
  def = new_list( empty_list, def ); // BODY
  def = new_list( empty_list, def ); // SOURCE
  def = new_list( empty_list, def ); // FULLSOURCE
  DEFINITIONS(var) = def;
  LOCALS(var) = empty_list;
  BINARY(var) = empty_list;

  #ifdef ADVANCED
    stats[ID(var)].allocs++;
    if( stats[ID(var)].max<(stats[ID(var)].allocs-stats[ID(var)].deallocs) )
      stats[ID(var)].max=(stats[ID(var)].allocs-stats[ID(var)].deallocs);
    stats_free--;
  #endif //ADVANCED

  return 1;
}




//===================================================
/// \fn    void delete_var( atom_t a ) 
/// \brief  deletes var atom
/// \param a  atom to delete
///
/// Deletes var atom by returning it back to the data
/// pool. All structures pointed to by the var atom
/// are dereferences (and most-likely) deleted.
//===================================================
void delete_var( atom_t a )
{
  //printf(">>>DELETEVAR "); dumpln(NAME(a));
  //if( DESCR2(a) ) { printf(">>>      DEFINITIONS "); dump_atom(DEFINITIONS(a),1); printf("\n\n"); }

  // dereference value of primitive/global variables
  if( (IS_PRIMITIVE(a) || IS_GLOBAL(a) || IS_TAG(a) || IS_RUNTIME(a)) && IS_VARIABLE(a) )
    {
      //printf(">>>DELETEVARVALUE "); dumpln(VALUE(a));
      DEUSE( VALUE(a) );
    }

  //if(a==root){printf(">>>DELETEVARNAME "); dumpln(NAME(a));}
  DEUSE( NAME(a) );

  // descriptors have no reference counts
  return_to_pool( &data_pool, DESCR1(a) );
  #ifdef ADVANCED
    stats[ID(a)].deallocs++;
    stats_free++;
  #endif //ADVANCED

  if( DESCR2(a) )
    {
      //printf(">>>DELETEVARLOCALS\n");
      //printf(">>>id=%d ref=%d\n",ID(a),REF(a));
      //dumpln(LOCALS(a));
      DEUSE( LOCALS(a) );
      //no-DEUSE( BODY(a) );
      //no-DEUSE( TREE(a) );
      //printf(">>>DELETEVARBINARY\n");
      DEUSE( BINARY(a) );
      //no-DEUSE( SOURCE(a) );
      //if( a==root )
      //{
      //printf(">>>DELETEVARDEFINITIONS\n");
      //printf(">>>BODY OF ROOT ID=%d\n",ID(root));
      //dumpln(BODY(root)); printf("========\n");
      //printf(">>>ROOT FULLSOURCE="); dumpln(FULLSOURCE(a));
      //printf(">>>ROOT SOURCE="); dumpln(SOURCE(a));
      //printf(">>>ROOT BODY="); dumpln(BODY(a));
      //printf(">>>ROOT TREE="); dumpln(TREE(a));
      //}
      //printf("vvvvvvvvvvvvvvvvvvvvvvvv\n");
      DEUSE( DEFINITIONS(a) );
      //printf("^^^^^^^^^^^^^^^^^^^^^^^^\n");
      //if( a==root )
      //{
      //printf(">>>DONE\n");
      //}
      //printf(">>>DELETEVARDEFINITIONS2\n");
      return_to_pool( &data_pool, DESCR2(a) );

      #ifdef ADVANCED
        stats[ID(a)].deallocs++;
        stats_free++;
      #endif //ADVANCED
    }
  return_to_pool( &data_pool, a );
  //if(a==root) printf(">>>          DONE!\n");
}




//===================================================
/// \fn    void dump_var( atom_t a, int level ) 
/// \brief  dumps error atom
/// \param a  atom to dump
/// \param level dump level
///
/// Dumps var atom through the current outter function.
//===================================================
void dump_var( atom_t a, int level )
{
#ifdef ADVANCED
  #define DUMP_BUF_SIZE 128
  char_t buf[DUMP_BUF_SIZE];
  int n;
  int i;

  if( OPTION_USER_VARIABLES && IS_PRIMITIVE(a) ) return;

  // print required number of spaces
  for( i=0; i<level; i++ ) outter( TEXT("   "), 3 );

  // print type, name and additional info
  if( IS_PRIMITIVE(a) ) outter( TEXT("PRIM"), -1 );
  if( IS_VARIABLE(a) )  outter( TEXT("VAR"), -1 );
  if( IS_FUNCTION(a) )  outter( TEXT("FUN"), -1 );
  if( IS_COMMAND(a) )   outter( TEXT("CMD"), -1 );

  if( IS_FUNCTION(a)||IS_COMMAND(a) )
    {
      n = SPRINTF( buf, DUMP_BUF_SIZE, TEXT("[%d"), LARGS(a) );
      outter( buf, n );
      n = SPRINTF( buf, DUMP_BUF_SIZE, TEXT(":%d]"), RARGS(a) );
      outter( buf, n );
    }

  outter( TEXT(" "), 1 );
  outter( STRING(NAME(a)), LENGTH(NAME(a)) );
  //if( !IS_PRIMITIVE(a) && (IS_FUNCTION(a)||IS_COMMAND(a)) )
  //{
  //  outter( TEXT("="), 1 );
  //  dump( TREE(a) );
  //}

  if( IS_VARIABLE(a) && a!=root && a!=globals )
    if( IS_RUNTIME(a) )
      {
        outter( TEXT(" = "), 3 );
        dump_atom( VALUE(a), 1 );
      }

  outter( TEXT("\n"), 1 );

  if( DESCR2(a) )
    {
      atom_t locals = LOCALS(a);
      for( ; IS_NOT_EMPTY(locals); locals=CDR(locals) )
        dump_atom( CAR(locals), level+1 );
    }

  #undef DUMP_BUF_SIZE

#endif //ADVANCED
}




//===================================================
/// \fn    atom_t find_var( atom_t name, atom_t parent )
/// \brief    searches a variable
/// \param  name  word atom containing the searched name
/// \param  parent  var atom where to start the search
/// \return    found var atom or \c NULL if not found
/// \note    search is syntax-scope based
///
/// Searches a variable named \c name starting from
/// variable \c parent. If not found found the search
/// continues with the parent of \c parent, then with
/// its grandparent, and so on untill the root is
/// reached. If still not found search continues
/// within the globals variable.
///
/// This search schema can find only variables known
/// at the time of compilation. Also, the search is
/// strictly syntax-scope based.
//===================================================
atom_t find_var( atom_t name, atom_t parent )
{
#ifdef SAFEMODE
  assert( IS_WORD(name) || IS_SUBWORD(name) );
  assert( parent );
  assert( IS_VARATOM(parent) );
#endif

  #ifdef DEBUG_FIND_VAR
    printf("<FINDVAR> Search "); dumpln(name);
    printf("<FINDVAR> Current var tree "); dumpln(root);
  #endif

  // scan parent and its parents
  atom_t a;
  for( ; parent; parent=PARENT(parent) )
    {
      #ifdef DEBUG_FIND_VAR
        printf("<FINDVAR> Search it in parent "); dumpln(NAME(parent));
      #endif
      a = find_local_var( name, parent );
      #ifdef DEBUG_FIND_VAR
        if( a ) { printf("<FINDVAR> Found in "); dumpln(NAME(parent)); }
      #endif
      if( a ) return a;
    }

  #ifdef DEBUG_FIND_VAR
    printf("<FINDVAR> Search it in parent "); dumpln(NAME(globals));
  #endif
  a = find_local_var( name, globals );
  #ifdef DEBUG_FIND_VAR
    if( a ) { printf("<FINDVAR> Found in "); dumpln(NAME(globals)); }
  #endif
  if( a ) return a;

  #ifdef DEBUG_FIND_VAR
    printf("<FINDVAR> Not found\n");
  #endif
  return NULL; // not found
}




//===================================================
/// \fn    atom_t find_local_runtime_var( atom_t name, int frame )
/// \brief    searches a variable at runtime
/// \param  name  word atom containing the searched name
/// \param      frame  starting frame for the search
/// \return    found var atom or \c NULL if not found
///
/// Searches a variable named \c name in the
/// the given stack frame. Search is done in the
/// list of variables created at run-time.
///
/// If the variable is not found in the runtimers,
/// then search continues with compile-time vars.
//===================================================
atom_t find_local_runtime_var( atom_t name, int frame )
{
  atom_t parent;
  atom_t var;

  #ifdef SAFEMODE
    assert( IS_WORD(name) || IS_SUBWORD(name) );
  #endif

  #ifdef DEBUG_FIND_RUNTIME_VAR
    printf("<FIND_LOCAL_RUNTIME_VAR> Search "); dumpln(name);
  #endif

  // first scan variables created at run-time
  parent = *(atom_t*)(frame+BASE_OFFSET_LOCALS);
  var = find_local_var( name, parent );
  #ifdef DEBUG_FIND_RUNTIME_VAR
    if( var ) { printf("<FIND_LOCAL_RUNTIME_VAR> Found runtimer "); dumpln(NAME(var)); }
  #endif
  if( var ) return var;

  // then scan variables created at compile-time
  parent = *(atom_t*)(frame+BASE_OFFSET_PARENT);
  var = find_local_var( name, parent );
  #ifdef DEBUG_FIND_RUNTIME_VAR
    if( var ) { printf("<FIND_LOCAL_RUNTIME_VAR> Found local "); dumpln(NAME(var)); }
  #endif
  if( var ) return var;

  #ifdef DEBUG_FIND_RUNTIME_VAR
    printf("<FIND_LOCAL_RUNTIME_VAR> Not found\n");
  #endif
  return NULL; // not found
}




//===================================================
/// \fn    atom_t find_runtime_var( atom_t name, int frame )
/// \brief    searches a variable at runtime
/// \param  name  word atom containing the searched name
/// \param      frame  starting frame for the search
/// \return    found var atom or \c NULL if not found
/// \note    search is syntax-scope based
///
/// Searches a variable named \c name starting from
/// the given stack frame. Search is done in the
/// list of variables created at run-time.
///
/// If the variable is not found in the runtimers,
/// then search continues with compile-time vars.
/// 
/// If still not found, the search moves to the
/// parent frame.
///
/// If not found in all frames up to the root, then
/// scan the globals var.
//===================================================
atom_t find_runtime_var( atom_t name, int frame )
{
  atom_t parent;
  atom_t var;

  #ifdef SAFEMODE
    assert( IS_WORD(name) || IS_SUBWORD(name) );
  #endif

  #ifdef DEBUG_FIND_RUNTIME_VAR
    printf("<FIND_RUNTIME_VAR> Search "); dumpln(name);
  #endif

  // scan parent and its parents
  while( frame )
    {
      // first scan variables created at run-time
      parent = *(atom_t*)(frame+BASE_OFFSET_LOCALS);
      var = find_local_var( name, parent );
      #ifdef DEBUG_FIND_RUNTIME_VAR
        if( var ) { printf("<FIND_RUNTIME_VAR> Found runtimer "); dumpln(NAME(var)); }
      #endif
      if( var ) return var;

      // then scan variables created at compile-time
      parent = *(atom_t*)(frame+BASE_OFFSET_PARENT);
      var = find_local_var( name, parent );
      #ifdef DEBUG_FIND_RUNTIME_VAR
        if( var ) { printf("<FIND_RUNTIME_VAR> Found local "); dumpln(NAME(var)); }
      #endif
      if( var ) return var;

      // exit the loop if we reached the root variable
      if( parent==root ) break;

      // go to upper frame
      frame = *((int*)(frame));
    }

  // the variable is not found, thus now
  // scan the global variables
  var = find_local_var( name, globals );
  #ifdef DEBUG_FIND_RUNTIME_VAR
     if( var ) { printf("<FIND_RUNTIME_VAR> Found global "); dumpln(NAME(var)); }
  #endif
  if( var ) return var;


  #ifdef DEBUG_FIND_RUNTIME_VAR
    printf("<FIND_RUNTIME_VAR> Not found\n");
  #endif
  return NULL; // not found
}




//===================================================
/// \fn    atom_t find_local_var( atom_t name, atom_t parent )
/// \brief    searches a local variable
/// \param  name  word atom containing the searched name
/// \param  parent  var atom where to start the search
/// \return    found var atom or \c NULL if not found
/// \note    search is syntax-scope based
///
/// Searches a variable named \c name starting from
/// variable \c parent. If found returns the var atom,
/// otherwise returns \c NULL. The search scans only
/// the variables \c parent. It does not scan its parents.
///
/// This search schema can find only variables known
/// at the time of compilation. Also, the search is
/// strictly syntax-scope based.
///
/// If the parent is a list atom, then just scan its
/// elements (as if this is the LOCALS field of a var)
//===================================================
atom_t find_local_var( atom_t name, atom_t parent )
{
#ifdef SAFEMODE
  assert( IS_WORD(name) || IS_SUBWORD(name) );
  assert( parent );
  assert( IS_VARATOM(parent) || IS_LIST(parent) );
#endif

  atom_t a;

  if( IS_LIST(parent) )
    {
      a = parent;
    }
  else
    {
      if( !DESCR2(parent) ) return NULL;
      a = LOCALS(parent);
      if( !a ) return NULL;
    }

  // scan all elements in the LOCALS
  for( ; IS_NOT_EMPTY(a); a=CDR(a) )
    if( same_words(name,NAME(CAR(a))) )
      {
        return CAR(a); // found
      }

  return NULL; // not found
}


//===================================================
/// \fn    atom_t new_local_var( atom_t name, atom_t function, int quoted )
/// \brief    creates a new local var
/// \param   name  word atom for the name of the variable
/// \param   function  var atom for the parent of the variable
/// \param   quoted     shows whether the name is quoted
/// \return    var or error atom
///
/// Creates a local variable in a function. The input
/// \c name contains the name of the local variable
/// together with the : or &quot; character (if quoted!=0).
///  If such local variable does not exist in the
/// function, then it is created and returned to the
/// caller. Otherwise an error atom of ERROR_DUPLICATE_INPUT
/// error is returned.
//===================================================
atom_t new_local_var( atom_t name, atom_t function, int quoted )
{
  #ifdef SAFE_MODE
  assert( IS_VAR(function) );
  assert( IS_ANY_WORD(name) );
  if( quoted )
    {
      assert( LENGTH(name)>1 );
      assert( *STRING(name)==TEXT(':') || *STRING(name)==TEXT('"') );
    }
  #endif

  atom_t real_name;
  if( quoted )
    real_name = new_subword( name, STRING(name)+1, LENGTH(name)-1 );
  else
    real_name = USE( name );

  if( find_local_var(real_name,function) )
    {
      //printf("THERE IS "); dump(real_name); printf(" IN FUNC "); dumpln(function);
      DEUSE( real_name );
      return new_error( ERROR_DUPLICATE_INPUT, name );
    }

  //printf("CREATED "); dump(real_name); printf(" IN FUNC "); dumpln(function);
  atom_t a = new_var( real_name, function, 1 );
  DEUSE( real_name );
  return a;
}



//===================================================
/// \fn    atom_t copy_local_vars( int frame )
/// \brief    copy local vars to parent
/// \param   frame current frame pointer
/// \return    same atom as input
///
/// This function is called from the generated code.
/// It copies all local variables of the current frame
/// into the dynamic parent (i.e. caller).
///
/// Copying variables consideres these cases:
///
/// No  Flag        Type      Action
///  ---  ---------    ----      ------
///  1.  primitive    *          not copied
///  2.  *            tag        not copied
///  3.  variable    normal
///  4.  variable    runtime
///  5.  variable    else      not copied
///  7.  func/cmd    normal
///  8.  func/cmd    runtime
///  9.  func/cmd    external
///  10.  func/cmd    internal
//===================================================
void copy_local_vars( int frame )
{
  // get frame and var atom of the callee
  int callee_frame = frame;
  atom_t callee = *(atom_t*)(callee_frame+BASE_OFFSET_PARENT);
  
  // get frame and var atom of the caller
  int caller_frame = *(int*)(callee_frame+BASE_OFFSET_DYNAMIC);
  atom_t caller = *(atom_t*)(caller_frame+BASE_OFFSET_PARENT);;
    
  //printf( "}->callee = "); dumpln(NAME(callee));
  //printf( "}->caller = "); dumpln(NAME(caller));
  
//#define FLAG_VARIABLE    0x0002
//#define FLAG_FUNCTION    0x0004
//#define FLAG_COMMAND    0x0008

//#define VAR_TYPE_NORMAL    0x00  ///< variable value is in stack
//#define VAR_TYPE_RUNTIME  0x01  ///< variable value is in the variable

  // Copies pure variable var from the callee to the caller.
  // If the variable exists in the caller, only its value
  // is transfered. If the variable does not exist, then
  // it is created as a runtime variable in the caller.
  // If to_atom is null, then there is no target variable
  void copy_var( atom_t from_var, atom_t to_var )
  {
    if( !to_var )
    {
      if( caller==root )
      {
        to_var = new_var( NAME(from_var), globals, 1 ); 
      }
      else
      {
        to_var = new_var( NAME(from_var), caller, 0 ); 
        atom_t* localsp = (atom_t*) (caller_frame + BASE_OFFSET_LOCALS);
        *localsp = new_list( to_var, *localsp ); // attach to other runtimers
      }
      VARTYPE( to_var ) = VAR_TYPE_RUNTIME;
      VALUE( to_var ) = USE( unbound );
    }
    
    //printf("copy "); dump_atom(NAME(PARENT(from_var)),1);
    //printf("."); dump_atom(NAME(from_var),1);
    //printf(" -> "); dump_atom(NAME(PARENT(to_var)),1);
    //printf("."); dump_atom(NAME(to_var),1);
    
    //printf("\n-->from_var=<|"); dump_atom((to_var),1);
    //printf("|>\n-->to_var=<|"); dump_atom((to_var),1);
    //printf("|>\n");
    
    // Continue with copying
    atom_t value;
      
    // get the value from the source variable variable
    SET_FLAGS( to_var, FLAG_VARIABLE );
    if( IS_NORMAL(from_var) )
    {  // CASE 3: value is in the current stack 
      value = *(atom_t*)((char*)callee_frame+OFFSET(from_var));
    }
    else
    {  // CASE 4: value pointed by var's atom
      value = VALUE( from_var );
    }
      
    // put the value in the target variable
    if( IS_NORMAL(to_var) )
    {
      //printf("normal var\n");
      atom_t* varptr = (atom_t*) ((char*)caller_frame + OFFSET( to_var ));
      //printf(" old="); dump_atom(*varptr,1);
      DEUSE( *varptr );
      *varptr = USE( value );
    }
    else
    {
      //printf("runtime var %x %x\n",(unsigned int)to_var,(unsigned int)VALUE(to_var));
      //printf(" old="); dump_atom(VALUE(to_var),1);
      DEUSE( VALUE(to_var) );
      VALUE( to_var ) = USE(value);
    }
  
    //printf(" new="); dump_atom(value,1);
    //printf("\n");
  } //copy_var()

  // Copies function/command var from the callee to the caller.
  // If to_atom is null, then there is no target variable
  void copy_func( atom_t from_var, atom_t to_var )
  {
    // if target does not exist - attach the var to the target parent
    // decrease the level of all static locals and recompile
    if( !to_var )
    {
        to_var = USE( from_var );
        LOCALS( caller ) = new_list( to_var, LOCALS( caller ) );
    }
    else
    {
      // if definitions are incompatible (different number of
      // local parameters) then exit without copying
      if (LARGS( from_var ) != LARGS( to_var ) || RARGS( from_var ) != RARGS( to_var ))
      {
        return;
      }
      
      FLAGS( to_var ) = FLAGS( from_var );
      VARTYPE( to_var ) = VARTYPE( from_var );
      PRIORITY( to_var ) = PRIORITY( from_var );
      ADDRESS( to_var ) = ADDRESS( from_var );

      DEUSE( LOCALS( to_var ) );
      LOCALS( to_var ) = USE( LOCALS( from_var ) );

      DEUSE( DEFINITIONS( to_var ) );
      DEFINITIONS( to_var ) = USE( DEFINITIONS( from_var ) );

      DEUSE( BINARY( to_var ) );
      BINARY( to_var ) = USE( BINARY( from_var ) );
    }
  }

  void copy_var_or_func( atom_t from_var )
  {
    // primitives cannot be copied
    if( IS_PRIMITIVE(from_var) ) return;
    
    // not notmals and not runtimes cannot be copies
    if( !IS_NORMAL(from_var) && !IS_RUNTIME(from_var) ) return;

    // search the destination variable
    atom_t to_var = find_local_runtime_var( NAME(from_var), caller_frame );
    
    // destination var must be non-primitive and (normal or runtime)
    if( to_var )
    {
      if( IS_PRIMITIVE(to_var) ) return;
      if( !IS_NORMAL(to_var) && !IS_RUNTIME(to_var) ) return;
    }
    
    // if variable or function/command then copy the var
    if( IS_VARIABLE( from_var ) ) copy_var( from_var, to_var );
    if( IS_FUNCTION( from_var ) || IS_COMMAND( from_var ) ) copy_func( from_var, to_var );

    return;
  }
  
  atom_t vars;
  
  // first scan variables created at run-time
  vars = *(atom_t*)(callee_frame+BASE_OFFSET_LOCALS);
  for( ; IS_NOT_EMPTY(vars); vars=CDR(vars) ) copy_var_or_func( CAR(vars) );

  // then scan variables created at compile-time
  vars = LOCALS(callee);
  for( ; IS_NOT_EMPTY(vars); vars=CDR(vars) ) copy_var_or_func( CAR(vars) );
}
