//
// Project:	Lhogho
// File:	vars.h	
//
// Copyright (C) 2007 P.Boytchev
//
// Revision history:
//   2007-02-27	- file created
//   2007-03-19 - PRIORITY_*
//   2007-05-17 - license info
//   2007-05-22	- doxygen-friendly documentation
//   2007-05-29 - find_local_var()
//   2007-06-06 - definitions spread to where they belong
//   2007-06-09 - added dump_var()
//   2007-06-18 - new_local_var()
//   2007-08-07 - BASE_OFFSET_PARAMS, BASE_OFFSET_LOCALS
//   2007-08-13 - globals, printdepthlimit, printwidthlimit,
//		  fullprintp;
//   2007-09-15 - caseignoredp
//   2007-12-14 - Support for FULLTEXT
//   2011-10-13 - Fixed Doxygen warnings
//   2011-12-26 - Support for SUBSTRING
//   2012-01-05 - More synonyms for primitives
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#ifndef LHOGHO_VARS_H
#define LHOGHO_VARS_H



//===================================================
/// \name Priority constants
//===================================================
#define PRIORITY_VAR	6	///< priority of variables
#define PRIORITY_MUL	5	///< priority of * and / operators
#define PRIORITY_ADD	4	///< priority of + and - operators
#define PRIORITY_FUN	3	///< priority of functions
#define PRIORITY_CMP	2	///< priority of <, >, =, <= and >= operators
#define PRIORITY_LOG	1	///< priority of and, or and not
#define PRIORITY_CMD	0	///< priproy of commands

#define PRIORITY_MAX	PRIORITY_VAR ///< maximal priority
#define PRIORITY_MIN	PRIORITY_CMD ///< minimal priority




//===================================================
/// \name Fields macros
/// These macros are used to access var atoms' fields.
/// Details about each macro for each variable type
/// are in \ref atoms.
//===================================================
#define OFFSET_DESCR1	8
#define OFFSET_ADDRESS	12

#define DESCR1(x)	((x)->unode.a[2])
#define DESCR2(x)	((x)->unode.a[3])
#define PARENT(x)	(DESCR1(x)->unode.a[0])
#define NAME(x)		(DESCR1(x)->unode.a[2])
#define ADDRESS(x)	(DESCR1(x)->unode.n[3])
#define OFFSET(x)	(DESCR1(x)->unode.n[3])
#define VALUE(x)	(DESCR1(x)->unode.a[3])
#define LEVEL(x)	(DESCR1(x)->unode.b[4])
#define CTYPE(x)	(DESCR1(x)->unode.b[5])
#define PRIORITY(x)	(DESCR1(x)->unode.b[5])
#define LARGS(x)	(DESCR1(x)->unode.b[6])
#define RARGS(x)	(DESCR1(x)->unode.b[7])
#define DEFINITIONS(x)	(DESCR2(x)->unode.a[0])
#define FULLSOURCE(x)	(CAR(DEFINITIONS(x)))
#define SOURCE(x)	(CAR(CDR(DEFINITIONS(x))))
#define BODY(x)		(CAR(CDR(CDR(DEFINITIONS(x)))))
#define TREE(x)		(CAR(CDR(CDR(CDR(DEFINITIONS(x))))))
#define LOCALS(x)	(DESCR2(x)->unode.a[1])
#define BINARY(x)	(DESCR2(x)->unode.a[3])
#define VARTYPE(x)	((x)->unode.b[5])
#define VARCLASS(x)     ((VARTYPE(x)-VAR_TYPE_EXTERNAL)&0x3F)


//===================================================
/// \name Flags
/// These bitmask flags determine the type of the
/// variable and its properties. The \c FLAG_PRIORITY_*
/// flags are not encoded in the var atom - they are
/// only present in the array with definition of
/// primitives. See also: \ref vars_flags.
//===================================================
#define FLAG_PRIMITIVE		0x0001
#define FLAG_VARIABLE		0x0002
#define FLAG_FUNCTION		0x0004
#define FLAG_COMMAND		0x0008 ///< <div></div>

#define FLAG_CAN_BE_UNARY	0x0010
#define FLAG_INFINITE_ARGS	0x0020
#define FLAG_MAY_SKIP_LAST_ARG	0x0040
#define FLAG_PROCESS_ARGS	0x0080
#define FLAG_PUSH_PARENT	0x0100
#define FLAG_SET_ONE_VAR	0x0200
#define FLAG_SET_ALL_VARS	0x0400
#define FLAG_MAY_HAVE_EXTRA_ARG 0x0800
#define FLAG_PUSH_FRAME		0x1000
#define FLAG_PRINT_VARS		0x2000
#define FLAG_EQUAL_VARS		0x4000
#define FLAG_PUSH_MODE		0x8000
#define ALL_VAR_FLAGS		0xFFFF ///< <div></div>

//===================================================
/// \name Types
/// These values determine the type of a variable.
/// A variable could belong to only one type/
//===================================================
#define VAR_TYPE_NORMAL		0x00	///< variable value is in stack
#define VAR_TYPE_RUNTIME	0x01	///< variable value is in the variable
#define VAR_TYPE_TAG		0x02	///< variable is tag <div></div>
#define VAR_TYPE_EXTERNAL       0x80    ///< 0x80-0xBF variable is an external function
#define VAR_TYPE_INTERNAL       0xC0    ///< 0xC0-0xFF variable is an internal function

#define FLAG_PRIORITY_MUL	0x010000
#define FLAG_PRIORITY_ADD	0x020000
#define FLAG_PRIORITY_LOG	0x040000
#define FLAG_PRIORITY_CMP	0x080000


//===================================================
/// \name Offsets
/// Base offsets determine the initial offset of
/// variables relative to the current stack frame.
/// Function parameters start from offset BASE_OFFSET_PARAMS
/// and go upward. Function local variablse start
/// from offset BASE_OFFSET_LOCALS and go downward.
//===================================================
#define BASE_OFFSET_PARAMS	    12	///< base offset for parameters
#define BASE_OFFSET_PARAMSCOUNT 8	  ///< base offset for number of parameters
#define BASE_OFFSET_DYNAMIC	    0 	///< base offset for the dynamic pointer
#define BASE_OFFSET_STATIC	    -4	///< base offset for the static pointer
#define BASE_OFFSET_PARENT	    -8	///< base offset for the parent atom
#define BASE_OFFSET_REPEATCHAIN -12	///< base offset for repeat-chain
#define BASE_OFFSET_TEST	      -16	///< base offset for test
#define BASE_OFFSET_LOCALS	    -20	///< base offset for locals



//===================================================
/// \name Constants
/// String constants are used to initialize some
/// system-defined variables.
//===================================================
#ifdef WINDOWS
#define LOGO_PLATFORM	TEXT("Windows")	///< Logo platform
#endif
#ifdef LINUX
#define LOGO_PLATFORM	TEXT("Linux")	///< Logo platform
#endif
#ifdef APPLE
#define LOGO_PLATFORM	TEXT("Mac OS X")///< Logo platform
#endif
#define LOGO_VERSION	TEXT("0.0")	///< Logo version
#define LOGO_DIALECT	TEXT("Lhogho")	///< Logo dialect



//===================================================
/// \name Testers
//===================================================
// @{
#define IS_PRIMITIVE(x)	GET_FLAGS(x,FLAG_PRIMITIVE)
#define IS_VARIABLE(x)	GET_FLAGS(x,FLAG_VARIABLE)
#define IS_FUNCTION(x)	GET_FLAGS(x,FLAG_FUNCTION)
#define IS_COMMAND(x)	GET_FLAGS(x,FLAG_COMMAND)
#define IS_VARATOM(x)	(ID(x)==VAR_ID)
#define IS_UNBOUND(x)	((x)==unbound)
#define IS_STOPPED(x)	((x)==stopped)
#define IS_GLOBAL(x)	(PARENT(x)==globals)
#define IS_NORMAL(x)	(VARTYPE(x)==VAR_TYPE_NORMAL)
#define IS_TAG(x)	(VARTYPE(x)==VAR_TYPE_TAG)
#define IS_RUNTIME(x)	(VARTYPE(x)==VAR_TYPE_RUNTIME)
#define IS_EXTERNAL(x)  ((VARTYPE(x)&0xC0)==VAR_TYPE_EXTERNAL)
#define IS_INTERNAL(x)  ((VARTYPE(x)&0xC0)==VAR_TYPE_INTERNAL)
// @}




#define ROOT_VAR_NAME		TEXT("%root%")	  ///< name of the root variable
#define GLOBALS_VAR_NAME	TEXT("%globals%") ///< name for the globals variable



extern atom_t root;	
extern atom_t word_to;
extern atom_t word_to_syn;
extern atom_t word_end;	
extern atom_t word_run;
extern atom_t word_make;
extern atom_t word_plus;
extern atom_t word_toplevel;	
extern atom_t word_system;	
extern atom_t word_error;
extern atom_t unbound;
extern atom_t stopped;
extern atom_t repeat_chain;
extern atom_t globals;
extern atom_t printdepthlimit;
extern atom_t printwidthlimit;
extern atom_t fullprintp;
extern atom_t caseignoredp;
extern atom_t last_error;
extern atom_t delayed_free;

extern int root_frame;
extern int backup_frame;
extern atom_t error_flag;
extern int catch_output_flag;	///< 1 = there was output/stop in catch

extern void init_vars( );
extern void finit_vars( );
extern atom_t new_var( atom_t name, atom_t parent, int attach );
extern atom_t new_local_var( atom_t name, atom_t function, int quoted ); 
extern void delete_var( atom_t a );
extern void dump_var( atom_t a, int level );
extern atom_t find_var( atom_t name, atom_t parent );
extern atom_t find_local_var( atom_t name, atom_t parent );
extern atom_t find_runtime_var( atom_t name, int static_link );
extern int need_descr2( atom_t var );
//extern void __attribute__((used,noinline,regparm(0),stdcall)) copy_local_vars( int frame );
extern void copy_local_vars( int frame );


#endif //LHOGHO_VARS_H
