
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class GuiTask extends Thread {

    public GuiTask(TextFrame frame, MDIMain gui) {
        this.frame = frame;
        this.gui = gui;
        this.connector = new JNIConnector(frame);
    }

    @Override
    public void run() {
        gui.disableCall();
        connector.callCompiler(command);
        gui.enableCall();
    }

    public void setCommand(String cmd) {
        command = cmd;
    }
    private JNIConnector connector;
    private TextFrame frame;
    private MDIMain gui;
    private String command = "";
}

@SuppressWarnings("serial")
public class MDIMain extends JFrame {

    private MDIDesktopPane desktop = new MDIDesktopPane();
    private JMenuBar menuBar = new JMenuBar();
    private JMenu fileMenu = new JMenu("File");
    private JMenuItem newMenu = new JMenuItem("New");
    private JMenuItem openMenu = new JMenuItem("Open");
    private JMenuItem saveMenu = new JMenuItem("Save");
    private JMenuItem exitMenu = new JMenuItem("Exit");
    private JMenu runMenu = new JMenu("Run");
    private JMenuItem run = new JMenuItem("Run");
    private JScrollPane scrollPane = new JScrollPane();

    public MDIMain() {
        menuBar.add(fileMenu);
        fileMenu.add(newMenu);
        fileMenu.add(openMenu);
        fileMenu.add(saveMenu);
        fileMenu.addSeparator();
        fileMenu.add(exitMenu);

        menuBar.add(runMenu);
        runMenu.add(run);

        menuBar.add(new WindowMenu(desktop));


        setJMenuBar(menuBar);
        setTitle("MDI Test");
        scrollPane.getViewport().add(desktop);
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(scrollPane, BorderLayout.CENTER);

        addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                close();
            }
        });
        newMenu.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                desktop.add(new TextFrame(desktop.getAllFrames().length));
            }
        });

        openMenu.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                open();
            }
        });

        saveMenu.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                save();
            }
        });

        exitMenu.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
                close();
            }
        });

        run.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent ae) {
               sendCommand();
            }
        });

    }

    public void open() {
        TextFrame tf = new TextFrame(desktop.getAllFrames().length);
        if (tf.load()) {
            desktop.add(tf);
        }
    }

    public void save() {
        if (desktop.getAllFrames().length > 0) {
            TextFrame tf = (TextFrame) desktop.getAllFrames()[0];
            tf.save();
        }
    }

    public void close() {
        JInternalFrame[] array = desktop.getAllFrames();
        for (JInternalFrame jf : array) {
            if (!((TextFrame) jf).close()) {
                return;
            }
        }
        System.exit(0);
    }

    public void disableCall() {
        runMenu.setEnabled(false);
    }

    public void enableCall() {
        runMenu.setEnabled(true);
    }

    private void sendCommand() {
        if (desktop.getAllFrames().length > 0) {
            TextFrame tf = (TextFrame) desktop.getAllFrames()[0];
            GuiTask task = new GuiTask(tf, this);
            task.setCommand(tf.getSource());
            task.start();
        }
    }

    public static void main(String[] args) {
        MDIMain lhogho = new MDIMain();
        lhogho.setSize(600, 400);
        lhogho.setVisible(true);
    }
}




