
import java.awt.BorderLayout;

import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

@SuppressWarnings("serial")
public class TextFrame extends JInternalFrame {

    private JTextArea textArea = new JTextArea();
    private JTextArea resultArea = new JTextArea();
    private boolean dirty = false;
    private String fileName;
    private JScrollPane scrollPaneText = new JScrollPane();
    private JScrollPane scrollPaneResult = new JScrollPane();

    public boolean close() {
        if (dirty) {
            int save = JOptionPane.showConfirmDialog(null, "Save file " + getTitle() + "?", "Save",
                    JOptionPane.YES_NO_OPTION);
            if (save == JOptionPane.YES_OPTION) {
                return save();
            } else {
                return true;
            }
        }
        return true;
    }

    public boolean isDirty() {
        return dirty;
    }

    public String getFileName() {
        return fileName;
    }

    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    public String getSource(){
        return textArea.getText();
    }

    public String getResult(){
        return resultArea.getText();
    }

    public void setResult(String result){
        resultArea.setText(result);
    }

    public boolean load() {
        String wd = System.getProperty("user.dir");
        JFileChooser fc = new JFileChooser(wd);
        fc.setApproveButtonText("Load");
        int rc = fc.showDialog(null, "Open File");
        if (rc == JFileChooser.APPROVE_OPTION) {
            fileName = fc.getSelectedFile().getAbsolutePath();
        } else {
            return false;
        }
        try {
            BufferedReader file = new BufferedReader(new FileReader(fileName));
            StringBuilder contents = new StringBuilder();
            String line;
            while ((line = file.readLine())!= null) {
                contents.append(line);
                contents.append(System.getProperty("line.separator"));
            }
            file.close();
            textArea.setText(contents.toString());
            dirty = false;
            setTitle(fileName);
            return true;
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "Cannot open file " + fileName, "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }
    }

    public boolean save() {
        if (fileName == null) {
            String wd = System.getProperty("user.dir");
            JFileChooser fc = new JFileChooser(wd);
            fc.setApproveButtonText("Save");
            int rc = fc.showDialog(null, "Save file to...");
            if (rc == JFileChooser.APPROVE_OPTION) {
                fileName = fc.getSelectedFile().getAbsolutePath();
            } else {
                return false;
            }
        }
        try {
            Writer file = new BufferedWriter(new FileWriter(fileName));
            file.write(textArea.getText());
            file.close();
            setTitle(fileName);
            dirty = false;
            return true;
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "Cannot open file " + fileName, "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }
    }

    public TextFrame(int index) {
        setSize(200, 300);
        setTitle("New document " + index);
        setMaximizable(true);
        setIconifiable(true);
        setClosable(true);
        setResizable(true);
        textArea.getDocument().addDocumentListener(new DocumentListener() {

            public void insertUpdate(DocumentEvent e) {
                dirty = true;
            }

            public void removeUpdate(DocumentEvent e) {
                dirty = true;
            }

            public void changedUpdate(DocumentEvent e) {
                dirty = true;
            }
        });

        getContentPane().setLayout(new BorderLayout());
        scrollPaneText.getViewport().add(textArea);
        getContentPane().add(scrollPaneText, BorderLayout.CENTER);
        scrollPaneResult.getViewport().add(resultArea);
        Dimension d = getSize();
        d.height/=3;
        scrollPaneResult.setPreferredSize(d);
        getContentPane().add(scrollPaneResult, BorderLayout.SOUTH);
    }
}

