#include <string.h>
#include <stdlib.h>

#include "JNIConnector.h"
#include "CCaller.h"

ENVIRONMENT environment = {0,};


JNIEXPORT void JNICALL Java_JNIConnector_initCLib
  (JNIEnv * env, jobject caller)
{
	environment.env = env;
	environment.interfaceCls = caller;
}

int createWindowHandle(DrawingInfo** handle)
{
	jclass wrapperTypeCls;
	jmethodID getWindowMid;
	jobject canvas;

    JAWT awt;
    JAWT_DrawingSurface* ds;
    JAWT_DrawingSurfaceInfo* dsi;

	wrapperTypeCls = (*environment.env)->FindClass(environment.env, "JNIConnector");
	if (!wrapperTypeCls)
	{
		return 1;
	}
    getWindowMid = (*environment.env)->GetStaticMethodID(environment.env, wrapperTypeCls, "getWindowCanvas", "(V)Ljava/awt/Canvas;");
	if (!getWindowMid)
	{
		return 2;
	}
	canvas = (*environment.env)->CallObjectMethod(environment.env, wrapperTypeCls, getWindowMid);
	if (!canvas)
	{
		return 3;
	}

	// Get the AWT
	awt.version = JAWT_VERSION_1_3;
 	if (JAWT_GetAWT(environment.env, &awt) == JNI_FALSE)
	{
		return 4;
	}

 	// Get the drawing surface
 	ds = awt.GetDrawingSurface(environment.env, canvas);
 	if (!ds)
	{
		return 5;
	}

 	// Get the drawing surface info
 	dsi = ds->GetDrawingSurfaceInfo(ds);
 	// Get the platform-specific drawing info
 	*handle = (DrawingInfo*)dsi->platformInfo;

	return 0;
}


int getGLContext(DrawingInfo* handle, GLContext* context)
{
#ifdef WINDOWS
    *context = wglCreateContext(handle->hdc);
    if (!*context)
    {
        return 1;
    }
    wglMakeCurrent(handle->hdc, *context);

#else
    XVisualInfo *vinfo;
    int          screen;
    int          glx_attr[32];
    int          glx_nattr;
    XSetWindowAttributes  win_attr;

    screen = DefaultScreen(handle->display);
    glx_nattr=0;
    glx_attr[glx_nattr++]=GLX_RGBA;
    glx_attr[glx_nattr++]=GLX_RED_SIZE;
    glx_attr[glx_nattr++]=1;
    glx_attr[glx_nattr++]=GLX_GREEN_SIZE;
    glx_attr[glx_nattr++]=1;
    glx_attr[glx_nattr++]=GLX_BLUE_SIZE;
    glx_attr[glx_nattr++]=1;
    glx_attr[glx_nattr++]=GLX_ALPHA_SIZE;
    glx_attr[glx_nattr++]=1;
    glx_attr[glx_nattr++]=GLX_DEPTH_SIZE;
    glx_attr[glx_nattr++]=12;
    glx_attr[glx_nattr++]=GLX_STENCIL_SIZE;
    glx_attr[glx_nattr++]=1;
    glx_attr[glx_nattr++]=GLX_DOUBLEBUFFER;
    glx_attr[glx_nattr++]=None;

    vinfo=glXChooseVisual(handle->display, screen, glx_attr);
    *context=glXCreateContext(handle->display, vinfo, NULL, True);
    XFree(vinfo);

    glXMakeCurrent(handle->display,handle->visualID,*context);
    XMapWindow(handle, handle->drawable);
    XFlush(handle);
#endif
    return 0;
}

