//===================================================
// Project:TestSystem   Author: Peter Armianov
//===================================================
//
// TESTMAKER.H
//
// This file contains basic types and definitions 
//  for the Lhogho test system
//
// Revision history:
//   2007-05-14	- file created
//   2007-07-01 - additional working modes
//   2007-07-12 - additional comments in Doxygen style
//   2007-09-08 - Implementing strict mode option
//   2013-06-27 - Adding INTEST flag /P. Boytchev/
//===================================================

#if defined(_MSC_VER)
#   pragma once
#endif

#ifndef __TEST_MAKER_H_07D957DB_2718_48C7_9981_03F095B9F7F9_INCLUDED
#define __TEST_MAKER_H_07D957DB_2718_48C7_9981_03F095B9F7F9_INCLUDED

//===================================================
/// \name Helper macro definitions
//===================================================
/// @{
#ifdef UNICODE
#   define TEXT(a)	L##a
#else
#   define TEXT(a)	a
#endif

#define SAFE_FREE(X)                                \
    {                                               \
        if (X){ free(X); X = NULL; }                \
    }
/// @}


//===================================================
/// \name Fixed width and sign data types
//===================================================
/// @{
#define	BYTE        unsigned char
#define UINT8       unsigned char
#define INT8        signed char
#define UINT16      unsigned short
#define INT16       signed short
#define UINT32      unsigned int
#define INT32       signed int
#define UINT64      unsigned long long
#define INT364      signed long long

#if defined(_MSC_VER)
#   include <limits.h>
#   undef  UINT32
#   define UINT32      unsigned long
#   undef  INT32
#   define INT32       signed long
#   undef  UINT64
#   define UINT64      unsigned __int64
#   undef  INT64
#   define INT64       signed __int64
#endif


#ifndef NULL
# define NULL	(void *)0
#endif

#ifndef BOOL
#define BOOL    UINT32
#endif

#ifndef TRUE
#define TRUE    ((BOOL)1)
#endif

#ifndef FALSE
#define FALSE   ((BOOL)0)
#endif

#ifdef UNICODE
#   define TCHAR    wchar_t
#else
#   define TCHAR    char
#endif

/// @}

//===================================================
/// \name Basic program defines
/// Defines of constants used in the main ptrogram
//===================================================
/// @{
#define ARG_DIR         "dir"
#define ARG_STRICT_MODE "strict"
#define ARG_PARAM       "param"
#define ARG_OUT         "out"
#define ARG_EXEC_MODE   "exec"
#define ARG_BUILD_MODE  "build"
#define ARG_COMPLIER    "compiler"
#define ARG_VERBOSE     "verbose"
#define ARG_INTEST      "intest"
#define ARG_HELP        "help"

#define MODE_TEST       0x01
#define MODE_BUILD      0x02
#define MODE_EXECUTE    0x04
/// @}

//----------------------------------------------------
/// \name Basic data struct definitions
//----------------------------------------------------
/// @{
typedef
struct program_parameters_tag
{
    char            * input_directory;
    char 	        * compiler_name;
    char            * global_params;
    FILE            * output_file;
    BOOL              verbose_flag;
    BOOL              intest_flag;
    BOOL              strict_flag;
    UINT32            tester_mode;
    UINT32            num_failed;
    UINT32            num_passed;
}program_parameters;
/// @}

//----------------------------------------------------
/// \name Global variable definitions
//----------------------------------------------------
/// @{
program_parameters    g_parameters;
/// @}

//----------------------------------------------------
/// \name extern function definitions
//----------------------------------------------------
/// @{
extern RESULT process_directory();
/// @}
#endif  /* __TEST_MAKER_H_07D957DB_2718_48C7_9981_03F095B9F7F9_INCLUDED */

