//===================================================
// Project:TestSystem   Author: Peter Armianov
//===================================================
//
// ERROR.H
//
// This file contains error codes for the system
//
// Revision history:
//   2007-05-14	- file created
//   2007-07-12 - additional comments in Doxygen style
//===================================================

#if defined(_MSC_VER)
#   pragma once
#endif

#ifndef __ERROR_H_EF982CFB_9F07_4AD5_888A_BFF8566F6DD6_INCLUDED
#define __ERROR_H_EF982CFB_9F07_4AD5_888A_BFF8566F6DD6_INCLUDED

#define ERROR_FLAG          0x80000000


//===================================================
/// \name Error code process macros
/// These macros are used to access errors.
/// 
//===================================================
/// @{
#define MAKE_ERROR(X)       (ERROR_FLAG | X)
#define MAKE_SUCCESS(X)     (X & (~ERROR_FLAG))
#define IS_ERROR(X)         (!!(X & ERROR_FLAG))
#define IS_SUCCESS(X)       (!(X & ERROR_FLAG))
#define GET_CODE(X)         (X & (~ERROR_FLAG))
/// @}

//===================================================
/// \name Error code definitions
//===================================================
/// @{
#define CODE_GENERIC        0x0000
#define CODE_INVALID_ARG    0x0001
#define CODE_FILE           0x0002
#define CODE_NOT_IMPL       0x0003
#define CODE_MEMORY         0x0004
#define CODE_EMPTY          0x0005
/// @}

#define SUCCESS_FULL        MAKE_SUCCESS(CODE_GENERIC)
#define SUCCESS_EMPTY       MAKE_SUCCESS(CODE_EMPTY)
#define SUCCESS_FILE        MAKE_SUCCESS(CODE_FILE)

#define ERR_GENERIC         MAKE_ERROR(CODE_GENERIC)
#define ERR_INVALID_ARG     MAKE_ERROR(CODE_INVALID_ARG)
#define ERR_FILE            MAKE_ERROR(CODE_FILE)
#define ERR_NOT_IMPL        MAKE_ERROR(CODE_NOT_IMPL)
#define ERR_MEMORY          MAKE_ERROR(CODE_MEMORY)

#define RESULT              UINT32

//===================================================
/// \name Logging macro definition
/// The macro is used to log debug info
//===================================================
/// @{
#ifdef LOG_STDERR
#   define INIT_LOGER
#   define LOG_ERROR(X) fprintf(stderr, "%s - %s : %s\n",__FILE__, __LINE__, X);
#   define UNINIT_LOGER
#else
#   define INIT_LOGER
#   define LOG_ERROR(X)
#   define UNINIT_LOGER
#endif
/// @}


#endif  /* __ERROR_H_EF982CFB_9F07_4AD5_888A_BFF8566F6DD6_INCLUDED */

