//===================================================
// Project:TestSystem   Author: Peter Armianov
//===================================================
//
// INTERNAL.H
//
// This file contains basic types and definitions 
//  for the test system.
//
// Revision history:
//   2007-05-14	- file created
//   2007-07-01 - additional working modes
//   2007-07-12 - additional comments in Doxygen style
//   2007-10-13 - Support for memory tests (-Zm option)
//   2009-05-30 - Support for shell tests
//===================================================

#if defined(_MSC_VER)
#   pragma once
#endif

#ifndef __INTERNAL_H_8DED3586_3931_4BD5_A578_3B8B6082391C_INCLUDED
#define __INTERNAL_H_8DED3586_3931_4BD5_A578_3B8B6082391C_INCLUDED

//===================================================
/// \name Basic definitions
/// These macros are used to define basic constants in
/// the code. 
//===================================================
/// @{
#define LOGO_NAME               "lhogho"
#define TARGET_FILE_EXTENSION   "lgo"
#define SHELL_FILE_EXTENSION    "sh"
#define EXPECTED_RESULTS_EXT    "expected"
#define REAL_RESULTS_EXT        "real"
#define ERROR_RESULTS_EXT       "stderr"
#define LTEMP_EXT               "ltemp"
#define MAX_LINE_LENGTH         2048
#define MAX_NAME_LEN            1024

#define EXPECTED_MEMORY         TEXT("{MEM#0:0}")
#define MEMORY_CHECK_OPTION     "-Zm"

#define PARAM_SET_SYMBOL        TEXT('=')
#define SPACE_ESC_SYMBOL        TEXT('\"')
#define COMMENT_SYMBOL          TEXT(';')
/// @}

//----------------------------------------------------
/// \name Config file data types
/// These struct contains data from parameters written 
/// into test case file. 
//----------------------------------------------------
/// @{
typedef
struct test_case_info_tag
{
    TCHAR            * test_name;
    char             * command_line_param;
    UINT16             flags;
} test_case_info;
/// @}


typedef RESULT (* parse_config_attribute_function) (const TCHAR * parameter_name, 
                                                    const TCHAR * parameter_optipons,
                                                    test_case_info * test_case_params);

//----------------------------------------------------
/// \name String functions prototypes
/// See source code for details
//----------------------------------------------------
/// @{
extern int     m_strcmp(const TCHAR * str1, const TCHAR * str2);
extern TCHAR * m_strcpy(TCHAR * dest, const TCHAR * src);
extern BOOL    m_isspace(TCHAR chr);
extern RESULT  m_strdup(TCHAR ** dest_ptr, const TCHAR * src);
extern RESULT  m_strndup(TCHAR ** dest_ptr, const TCHAR * src, size_t num_chars);
extern size_t  m_strlen(const TCHAR * str);

extern TCHAR * m_fgets(FILE* file, TCHAR * buffer, size_t buffer_size, size_t * out_size_ptr); 
extern void    m_fputs(FILE* file, const TCHAR * buffer);
extern void    m_fputs_ascii(FILE* file, const char * buffer);
extern void    m_fputc(FILE* file, TCHAR tchar);
/// @}

//----------------------------------------------------
/// \name Internal tester helper functions prototypes
/// See source code for details
//----------------------------------------------------
/// @{
extern RESULT parse_line(FILE * input_file, TCHAR ** parameter_name, TCHAR ** parameter_optipons);
extern RESULT extract_args(FILE * input_file, test_case_info * test_info);
extern RESULT extract_expected_results(FILE * input_file, const char * file_name);
extern RESULT execute_test(const char * file_name, test_case_info test_info);
extern RESULT compile_test(const char * file_name, test_case_info test_info);
extern RESULT check_results(const char * file_name, test_case_info test_info, UINT32 exec_result);
extern RESULT import_results(const char * file_name, test_case_info test_info);
extern void   clean_up(const char * file_name);
extern void   print_file(const char * file_name, FILE * output);
extern RESULT file_compare(const char * test_file_name, test_case_info test_info);
extern RESULT exec_shell(const char * file_name);
/// @}


extern int g_first;
#endif  /* __INTERNAL_H_8DED3586_3931_4BD5_A578_3B8B6082391C_INCLUDED */

