//===================================================
// Project:TestSystem   Author: Peter Armianov
//===================================================
//
// PARAM_PARSERS.H
//
//  Defines some macro definitions for automatic 
// registration of command parameter functions 
// prototypes of this functions and enum entries
//
// Revision history:
//   2007-05-14	- file created
//   2007-07-12 - additional comments in Doxygen style
//===================================================

#if defined(_MSC_VER)
#   pragma once
#endif

#ifndef __PARAM_PARSERS_H_931B1AE_9DFE_4E4C_8DCE_A9F2C60DD39D_INCLUDED
#define __PARAM_PARSERS_H_931B1AE_9DFE_4E4C_8DCE_A9F2C60DD39D_INCLUDED

//----------------------------------------------------
/// \name Macro definition for parser registers
///  These macro definitions automatically generates
/// enum entries and prototypes of function parsing 
/// test arguments. Also generate function that 
/// register function callbacks
//----------------------------------------------------
/// @{

/* Macros that generate enum for parsers */
#define MAKE_ONE_ENUM_ENTRY(entry_name)     entry_name,
#define MAKE_PARSERS_ENUM(List)                                         \
    enum REGISTERED_PARSERS_TAG                          	            \
    {                                                                   \
        List(MAKE_ONE_ENUM_ENTRY)                                       \
        PARSERS_COUNT                                                   \
    };       
typedef enum REGISTERED_PARSERS_TAG RegisteredParsers;

/* Macros that generate prototypes for parser functions */
#define MAKE_ONE_PROTOTYPE(param_name)                                  \
    extern RESULT param_name##_parser(const TCHAR * parameter_name,     \
                                      const TCHAR * parameter_optipons, \
                                      test_case_info * test_case_params);
#define MAKE_PROTOTYPES(List)   List(MAKE_ONE_PROTOTYPE)


/* Macros that generate code thar register parser functions */
#define MAKE_ONE_PARSER_REGISTRATION(param_name)                        \
    {                                                                   \
        g_parsers[ind++] = param_name##_parser;                         \
    }

#define IMPLEMENT_PARSERS_REGISTER_FUNCTION(List)        		        \
    RESULT  init_param_parsers()             		                    \
    {                                                                	\
        UINT32  ind = 0;                                                \
        List(MAKE_ONE_PARSER_REGISTRATION)                               \
        return SUCCESS_FULL;                    	                    \
    }


/*
*   Adding new parameter parser must start with adding line for it here.
*  _(parser_name)
*  From this line automatically will be generated for it:
*   enum entry,
*   prototype of parser function
*   code that register the function in the global array
*/
#define PARSERS_LIST(_)                                                \
    _(name)                                                            \
    _(cmd)


/* Generate parser function prototypes */
MAKE_PROTOTYPES(PARSERS_LIST);

/* Generate enum with all parsers and calculate their count */
MAKE_PARSERS_ENUM(PARSERS_LIST);

/* Global array with pointers to all parser functions */
parse_config_attribute_function g_parsers[PARSERS_COUNT];
/// @}

//----------------------------------------------------
/// \name Basic functions prototypes
//----------------------------------------------------
/// @{
RESULT  extract_string(const TCHAR * parameter_optipons, TCHAR ** extracted_value);
/// @}

#endif /* __PARAM_PARSERS_H_931B1AE_9DFE_4E4C_8DCE_A9F2C60DD39D_INCLUDED */

