//
// Project:	Lhogho
// File:	runtime.c
//
// Copyright (C) 2007-2012 P.Boytchev
//
// Revision history:
//   2007-06-19	- file created
//   2007-06-25 - rt_plus()
//   2007-06-26 - rt_minus(), rt_mul(), rt_div()
//   2007-07-04 - rt_cmdchk(), rt_funchk()
//   2007-07-06 - fixed bug #1748684 Memory leak print (45-10)*(10 + (1+"abc)*(3-4))
//   2007-07-31 - rt_make()
//		- fixed bug #1764281 Wrong error message
//   2007-08-01 - rt_local()
//   2007-08-05 - rt_makechk()
//   2007-08-07 - fixed bug #1769316 Memory leak of 1 atom in MAKE
//   2007-08-08 - Added implementations of arithmetical and trigonometric primitives
//   2007-08-10 - Fixed bug #1771145 - Trigonometrical functions precision problem
//   2007-08-10 - Handle of invalid values for arithmetic functions
//   2007-08-13 - support for PRINTDEPTHLIMIT, PRINTWIDTHLIMIT
//   2007-09-03 - Fixed bug #1787023 ARCTAN and RADARCTAN
//   2007-09-15 - Added implementations of comparison primitives
//   2007-09-17 - Added support for AND, OR, NOT
//   2007-09-18 - rt_output(), rt_stop(), rt_dump()
//   2007-09-19 - Fixing doxygen coments due to bugs #1797861 and #1797054
//   2007-09-19 - rt_ignore()
//   2007-09-19 - rt_first(), rt_butfirst()
//   2007-09-19 - rt_last(), rt_butlast(), rt_item()
//          Many fixes and optimizations on previously implemented functions
//   2007-09-23 - Support for IF
//		- Support for PARSE, RUNPARSE
//   2007-09-26 - Fixed bug #1800491 Memory leak in if-nested.lgo
//		- Fixed bug #1800961 Memory leak in 14=10
//   2007-09-26 - rt_numberp(), rt_wordp(), rt_listp(), rt_empty()
//   2007-09-27 - rt_memberp()
//   2007-09-29 - rt_repeat()
//   2007-10-06 - rt_word(), rt_list(), rt_sent(), rt_lput(), rt_fput()
//   2007-10-13 - rt_count(), rt_char(), rt_ascii(), rt_upper(), rt_lower(), rt_member()
//   2007-10-16 - Fixed bug #1814279 Errors masked by IGNORE
//   2007-10-21 - rt_repchk(), rt_forever()
//   2007-10-21 - Fixed bug #1817321
//   2007-10-23 - rt_repcount()
//   2007-10-28 - rt_random(), rt_rerandom(), rt_iseq(), rt_rseq()
//   2007-10-28 - implement #1815125
//   2007-10-28 - rt_type(), rt_show(), rt_form(), rt_format()
//   2007-11-03 - Fixed bug #1821537 repeat cause compiler crash
//   2007-11-04 - rt_while(), rt_until()
//		- Fixed bug #1825647 Various warning in dev doc
//   2007-11-05 - support for PRINTDEPTHLIMIT, PRINTWIDTHLIMIT and
//		  FULLPRINTP
//   2007-11-06 - rt_dowhile(), rt_dountil()
//		- rt_definedp(), rt_primitivep(), rt_name(),
//		- rt_procedurep(), rt_thing()
//   2007-11-09 - Fixed bug #1828216 MAKE with expression for var name
//		- Fixed bug #1828215 MAKE does not work with unknown variables
//		- Fixed bug #1828217 ':' does not work with unknown variables
//   2007-11-10 - Fixed bug #1828116 Warning compiling get_format()
//   2007-11-11 - Support for WAIT and BYE
//		- Support for NAME
//   2007-11-12 - Support for THROW
//		- rt_before optimized not to copy buffers if they are words.
//   2007-11-16 - rt_catch()
//   2007-11-17 - Fixed bug #1832660 THROW <tag> not implemented
//   2007-11-18 - Fixed bug #1833771 Test case catch-08.lgo fails
//   2007-11-19 - Support for TAG, GOTO
//   2007-11-21 - rt_test()
//   2007-11-22 - rt_iftrue(), rt_iffalse()
//   2007-12-02 - Fixed bugs #1838919, #1838911, #1838910, #1838909,
//		  #1838908, #1838907, #1838906 and #1831028
//		- rt_backslashedp()
//   2007-12-02 - Implemented featutre request #1842653 - DEBAR
//		- Support for ASHIFT, LSHIFT
//		- Support for BITAND, BITOR, BITXOR, BITNOT
//   2007-12-03 - rt_text()
//   2007-12-14 - rt_fulltext()
//   2007-12-15 - Fixed bug #1832680 - wrong error message in fput and lput
//   2007-12-16 - Support for PICK, REMDUP, REMOVE, REVERSE,
//                  RAWASCII, GENSYM, SUBSTRINGP
//   2007-12-30 - Support for COMBINE
//   2008-01-07 - Fixed bug #1856864 Defined but unset variables
//		- Support for RUNRESULT
//   2008-03-22 - Support for QUOTED
//   2008-04-28 - Fixed bug #1952921 FPUT does not work with numbers
//		  Fixed bug #1953190 ASCII.LGO and RAWASCII.LGO fail
//		  Fixed bug #1953207 In Fedora 4 tests fail
//   2008-07-19 - Changed depricated _sleep() to Win32API's Sleep()
//   2008-08-17 - rt_firsts(), rt_butfirsts()
//   2008-08-23 - Support for FOR
//   2008-09-05 - Support for LIBLOAD, LIBFREE
//              - Support for PACKSIZE, PACK, UNPACK
//   2008-09-06 - Support for PACKADDR, PACKTO
//   2008-09-09 - Support for EXTERNAL
//   2008-10-27 - Support for LOAD
//   2008-11-21 - rt_packopen(), rt_packclose(), rt_packread(), rt_packwrite()
//   2009-03-12 - Implementer request 2544111 - extend REVERSE
//   2009-05-16 - rt_abs()
//   2009-06-03 - rt_readchar(), rt_readchars(), rt_getenv(), rt_getenvs()
//   2009-07-28 - Implementer request 2776194 - extend REMDUP and REMOVE
//   2009-08-02 - Fixed bug #2794985 CALHC fails with wrong input
//   2009-08-03 - Support for READRAWLINE
//   2009-08-03 - Support for READWORD
//   2009-08-16 - Fixed bug #2838617 RUN suppresses STOP
//   2009-08-19 - Support for MAYBEOUTPUT
//   2009-08-20 - Support for _STACKFRAME, _STACKFRAMEATOM
//   2009-08-29 - Fixed bug #2845099 Memory leak in THROW
//		- Fixed bug #2845579 DEBUG_MEMORY_LEAKS finds two bad tests
//		- Fixed bug #2846281 Errors not (de)used correctly
//   2009-08-30 - Fixed bug #2847273 Crash of prematurely cleared errors
//   2009-09-01 - Fixed bug #2847596 Error in NAME?
//   2010-06-25 - Support for EOF?
//   2010-06-26 - Fixed bug #3021674 GETENVS fails when executed multiple times
//   2011-02-05 - Fixed bug #3173377 Warnings building developer's documentation
//		- Renamed file functions
//   2011-02-28 - Fixed bug #3195266 Assertion at atoms.c:310 failed
//   2011-03-03 - Fixed bug #3198402 Test cases fail to comply with the new memory test
//   2011-10-14 - Support for RUNMACRO
//   2011-10-22 - Fixed bug #3427251 Indirect MAKE and LOCAL cause a crash
//   2011-11-30 - Fixed bug #3445230 Catch doesn't catch an error
//   2011-12-01 - Fixed bug #3442800 Going beyond the end limit of FOR
//   2011-12-25 - Fixed bug #3463989 remainder 1 0 generates assertion error
//   2011-12-26 - Support for SUBSTRING
//   2012-01-02 - Command-line options are translatable
//   2012-01-15 - Fixed bug #3474182 Error in packs causes out of memory in Linux
//   2012-01-19 - Support for CURRENTFOLDER, MAKEFOLDER
//   2012-01-20 - Support for ERASEFOLDER, FOLDER?, FOLDERP
//   2012-01-21 - Support for FOLDERS, CHANGEFOLDER
//   2012-01-23 - Support for FILES, ERASEFILE, FILE?, FILEP, RENAMEFILE
//                FILESIZE, FILETIMES, FORMATTIME
//   2012-01-24 - Support for OPENREAD, OPENWRITE, OPENAPPEND, OPENUPDATE
//                SETREAD, SETWRITE
//   2012-01-25 - init_runtime(), finit_runtime(), find_file_by_name()
//                find_file_by_handle()
//              - Support for READER, WRITER, ALLOPEN
//   2012-01-27 - Support for CLOSEALL, READPOS, SETREADPOS, WRITEPOS, SETWRITEPOS
//   2012-01-31 - Support for TIMEZONE
//   2012-02-02 - Fixed bug #3482718 Closing file with invalid handle causes a crash
//   2012-02-04 - READPACK/WRITEPACK use file set by SETREAD/SETWRITE
//                Fixed bug #3484310 Wrong error when unpacking empty list
//                Fixed bug #3484326 SETWRITE [] causes crash
//                Fixed bug #3484297 Crash if error in readpack
//                Fixed bug #3484289 Open non-existing file for reading - no error
//   2012-02-05 - Renamed READPACK->READBLOCK, WRITEPACK->WRITEBLOCK, PACKADDR->DATAADDR
//                PACKSIZE->BLOCKSIZE, PACK->LISTTOBLOCK, UNPACK->BLOCKTOLIST,
//                PACKTO->LISTINTOBLOCK
//              - Support for READINBLOCK
//   2012-02-09 - Support for DRIBBLE, NODRIBBLE
//   2012-02-26 - Fixed bug #3493871 Externals do not capture errors
//   2012-02-27 - Fixed bug #3494907 Errors in user function inputs cause assertion fail
//   2013-07-04 - If rt_libload() fails, it tries in lib subfolder of the compiler 
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#include <stdio.h>
#include <math.h>
#include <limits.h>
#include <assert.h>
#include <stdlib.h> // rand, srand
#include <unistd.h> // usleep, getcwd, ...
#include <errno.h>  // EIO
#include <sys/stat.h> // stat, S_ISDIR, ...
#include <sys/types.h>
#include <dirent.h>
#include <time.h> // localtime_r
#include <limits.h>	// PATH_MAX
#include <libgen.h>	// dirname()

#ifdef WINDOWS
#include <windows.h>
#include <mmsystem.h>
#undef TEXT
#undef IS_ERROR
#define dlopen(a,b)  LoadLibrary(a)
#define dlclose(a)   FreeLibrary(a)
#define dlsym(h,a)   GetProcAddress(h,a)
#else
#include <dlfcn.h>
#endif

#include "globals.h"
#ifdef UNICODE_CHARS
#include <wctype.h>
#include <wchar.h>  // STRCHR (i.e. wcsstr)
#endif

#include "atoms.h"
#include "lists.h"
#include "words.h"
#include "numbers.h"
#include "vars.h"
#include "unicode.h"
#include "parser.h"
#include "errors.h"
#include "options.h"
#include "parser.h"
#include "compiler.h"
#include "mems.h"
#include "external.h"
#include "runtime.h"


#define test_elem_and_destroy_if_error(elem, list)          \
  {                                                           \
    if (IS_ERROR(elem))                                     \
    {                                                       \
      DEUSE (list);                                       \
      RETURN (elem);                                      \
    }                                                       \
  }   ///< Check if element is error and if destroy the list


#define ARGUMENT	*pdata
#define EACH_ARGUMENT	pdata=(atom_t*)(&data+data); (int*)pdata>&data; pdata--


#define SIGN(X)                                             \
  ((X == 0) ? 0 : (X < 0 ? -1 : 1))


#define ZERO_PRECISION 1e-10
#define MAX_NUMBER_WORD_LENGTH 64
#define MAX_WORD_LENGTH 4096

#define CHECK_PARAM(param)	if( IS_ERROR(param) ) RETURN(USE(param))

#undef RUNTIME
#define RUNTIME __attribute__((used,noinline,regparm(0),cdecl)) ///< default attributes of runtime functions

char* file_names[FOPEN_MAX]   = { [0 ... (FOPEN_MAX-1)]=NULL }; ///< Array of names of opened files
FILE* file_handles[FOPEN_MAX] = { [0 ... (FOPEN_MAX-1)]=NULL }; ///< Array of handles of opened files

void rt_set_var_value( int static_link, atom_t parent, atom_t var, atom_t value );


//===================================================
/// \fn void init_runtime( )
/// \brief	initializes the Runtime module
///
/// So far nothing to be initialized
//===================================================
void init_runtime( )
{
	srand(time(0));
}




//===================================================
/// \fn void finit_runtime( )
/// \brief	finalizes the Runtime module
///
/// Deallocates all names of unclosed files and
/// closes them.
//===================================================
void finit_runtime( )
{
  rt_closeall();
  rt_nodribble();
}



//===================================================
/// \fn		int find_file_by_filename (char* filename)
/// \brief	search log by filename
/// \param filename name of a file
/// \return index of found filename; -1 if not found
///
/// Searches ::file_names for the string in filename
/// and return its index in the array. If not found
/// returns -1.
//===================================================
int find_file_by_filename (char* filename)
{
  int i;

  for( i=0; i<FOPEN_MAX; i++ )
    if( file_names[i] )
      if( strcmp(file_names[i],filename)==0 ) // string are ASCII/UTF8
        return i;
  return -1;
}

//===================================================
/// \fn		int find_file_by_handle (FILE* handle)
/// \brief	search log by handlee
/// \param handle handle of a file
/// \return index of found file; -1 if not found
///
/// Searches ::file_handles for a given handle and
/// return its index in the array. If not found
/// returns -1.
//===================================================
int find_file_by_handle (FILE* handle)
{
  int i;
  for( i=0; i<FOPEN_MAX; i++ )
    if( file_handles[i]==handle )
      return i;
  return -1;
}




//===================================================
/// \fn		atom_t rt_makechk( atom_t source, atom_t data )
/// \brief		checks for valid result of a MAKE command
/// \param	source	source of the command
/// \param	data	value to check
/// \return		error or unbound atom
///
/// Checks whether the result of a MAKE is valid.
/// Returns error atom if the result is error or
/// unbound. Otherwise returns unbound atom.  The
/// \c source parameter points to the source where
/// the make is called. It is used to locate the
/// error position in case of errors.
//===================================================
#define rt_makechk __attribute__((used,noinline,regparm(0),stdcall)) rt_makechk

atom_t rt_makechk( atom_t source, atom_t data )
#undef rt_makechk
{
  #ifdef SAFEMODE
  assert( data );
  #endif //SAFEMODE

  //printf("---------------\n");
  //printf("inside rt_makechk\n");
  //printf("    - source="); dumpln(source);
  //printf("    - data  ="); dumpln(data);
  //printf("---------------\n");

#ifdef ADVANCED
  if (OPTION_RUNTIME)
  {
    outter( TEXT( "DONE " ), -1 );
    dumpln( source );
  }
#endif

  if (IS_UNBOUND( data ))
  {
    return new_error( ERROR_MISSING_VALUE, source );
  }

  if (IS_ERROR( data ))
  {
    add_error_source( data, source );
    return USE( data );
  }

  // this unbound should not be USE()'d, because
  // it is used only for checking -- it is not
  // stored anywhere
  return unbound;
}




//===================================================
/// \fn		atom_t rt_cmdchk( atom_t source, atom_t data )
/// \brief		checks for valid result of a command
/// \param	source	source of the command
/// \param	data	value to check
/// \return		error or unbound atom
///
/// Checks whether the result of a command is valid.
/// Returns error atom if the result is error or
/// not unbound. Otherwise returns unbound atom.  The
/// \c source parameter points to the source where
/// the command is called. It is used to locate the
/// error position in case of errors.
//===================================================
#define rt_cmdchk __attribute__((used,noinline,regparm(0),stdcall)) rt_cmdchk

atom_t rt_cmdchk( atom_t source, atom_t data )
#undef rt_cmdchk
{
  #ifdef SAFEMODE
  assert( data );
  #endif //SAFEMODE

  //printf("---------------\n");
  //printf("inside rt_cmdchk\n");
  //printf("    - source="); dumpln(source);
  //printf("    - datax =%x\n",(int)data);
  //printf("    - data  ="); dumpln(data);
  //printf("---------------\n");

#ifdef ADVANCED
  if (OPTION_RUNTIME)
  {
    outter( TEXT( "DONE " ), -1 );
    dumpln( source );
  }
#endif
  if (IS_UNBOUND( data ) || IS_STOPPED( data ))
  return data;

  if (IS_ERROR( data ))
  {
    add_error_source( data, source );
    return USE( data );
  }

  DEUSE( data );
  return new_error( ERROR_UNUSED_VALUE, source );
}




//===================================================
/// \fn		atom_t rt_exprchk( atom_t source, atom_t data )
/// \brief		checks for valid result of an expression
/// \param	source	source of the expression
/// \param	data	value to check
/// \return		error or unbound atom
///
/// Checks whether the result of an expression is
/// valid. Actually any result is valid. Thus this
/// function is used only to add new error position
/// or dump the source if -Zrt option is on.
//===================================================
#define rt_exprchk __attribute__((used,noinline,regparm(0),stdcall)) rt_exprchk

atom_t rt_exprchk( atom_t source, atom_t data )
#undef rt_exprchk
{
  #ifdef SAFEMODE
  assert( data );
  #endif //SAFEMODE

  //printf("---------------\n");
  //printf("inside rt_exprchk\n");
  //printf("    - source="); dumpln(source);
  //printf("    - data  ="); dumpln(data);
  //printf("---------------\n");

#ifdef ADVANCED
  if (OPTION_RUNTIME)
  {
    outter( TEXT( "DONE " ), -1 );
    dumpln( source );
  }
#endif

  if (IS_ERROR( data ))
  {
    add_error_source( USE( data ), source );
  }

  return data;
}




//===================================================
/// \fn		atom_t rt_boolchk( atom_t source, atom_t data )
/// \brief		checks for valid boolean value
/// \param	source	source of the command
/// \param	data	value to check
/// \return		error, true or false atom
///
/// Checks whether the value is \c true or \c false.
/// If not then returns an error atom.
//===================================================
#define rt_boolchk __attribute__((used,noinline,regparm(0),stdcall)) rt_boolchk

atom_t rt_boolchk( atom_t source, atom_t data )
#undef rt_boolchk
{
  #ifdef SAFEMODE
  assert( data );
  #endif //SAFEMODE

  if (IS_ERROR( data ))
  {
    add_error_source( data, source );
    return USE( data );
  }

  int b;
  if (atom_to_boolean( data, &b ))
  {
    DEUSE( data );
    return ( false_true[b]);
  }

  DEUSE( data );
  return new_error( ERROR_BOOLEAN_EXPECTED, source );
}




//===================================================
/// \fn		atom_t rt_funchk( atom_t source, atom_t data )
/// \brief		checks for valid result of a function
/// \param	data	value to check
/// \param	source	source of the function
/// \return		error or unbound atom
///
/// Checks whether the result of a function is valid.
/// Returns error atom if the result is error or
/// unbound. Otherwise returns the same atom. The
/// \c source parameter points to the source where
/// the function is called. It is used to locate the
/// error position in case of errors.
//===================================================
#define rt_funchk __attribute__((used,noinline,regparm(0),stdcall)) rt_funchk

atom_t rt_funchk( atom_t source, atom_t data )
#undef rt_funchk
{
  #ifdef SAFEMODE
  assert( data );
  #endif //SAFEMODE

  //printf("---------------\n");
  //printf("inside rt_funchk\n");
  //printf("    - source="); dumpln(source);
  //printf("    - data  ="); dumpln(data);
  //printf("---------------\n");

#ifdef ADVANCED
  if (OPTION_RUNTIME)
  {
    outter( TEXT( "EVAL " ), -1 );
    dump( source );
    outter( TEXT( " = " ), 3 );
    dumpln( data );
  }
#endif

  if (IS_ERROR( data ))
  {
    add_error_source( data, source );
    return ( USE(data) );
  }

  if (!IS_UNBOUND( data ))
  return data;


  DEUSE( data );
  return new_error( ERROR_MISSING_VALUE, source );
}




//===================================================
/// \fn		atom_t rt_repchk( atom_t source, atom_t data )
/// \brief		checks for valid repetition count
/// \param	data	value to check
/// \param	source	source of the expression
/// \return		error or unbound atom
///
/// Checks whether the number of repetition in a
/// \c repeat command is valid. Returns error atom if
/// the result is error, unbound or invalid. Otherwise
/// returns an integer atom containing the number of
/// repetitions. The \c source parameter points to
/// the source where the expression is called. It is
/// used to locate the error position in case of errors.
//===================================================
#define rt_repchk __attribute__((used,noinline,regparm(0),stdcall)) rt_repchk

atom_t rt_repchk( atom_t source, atom_t data )
#undef rt_repchk
{
  #ifdef SAFEMODE
  assert( data );
  #endif //SAFEMODE

  //printf("data=%x\n",data);
  //printf("atom type=%d\n",ID(data));
  //printf("repeat atom="); dumpln(data);

  if (IS_ERROR( data ))
  {
    add_error_source( data, source );
    return USE( data );
  }

  if (IS_UNBOUND( data ))
  {
    DEUSE( data );
    return new_error( ERROR_MISSING_VALUE, source );
  }

  int64_t cnt;
  if (!atom_to_int( data, &cnt ))
  {
    DEUSE( data );
    return new_error( ERROR_NOT_AN_INTEGER, source );
  }

  DEUSE( data );

  if (cnt > INT_MAX)
  return new_error( ERROR_TOO_BIG_NUMBER, source );

  if (cnt < 0)
  return new_error( ERROR_TOO_SMALL_NUMBER, source );

  return new_integer( cnt );
}




//===================================================
/// \fn		atom_t rt_forchk( atom_t source, atom_t* step_value, atom_t step, atom_t to, atom_t from )
/// \brief		checks for valid repetition count
/// \param	from    initial value
/// \param      to      final value
/// \param      step        step value (could be UNBOUND)
/// \param      step_value  pointer to step value
/// \param	source	source of the expression
/// \return		error or unbound atom
///
/// Calculates the number of repetition in a
/// \c for command. Returns error atom if
/// the result is error, unbound or invalid. Otherwise
/// returns an integer atom containing the number of
/// repetitions. The \c source parameter points to
/// the source where the expression is called. It is
/// used to locate the error position in case of errors.
//===================================================
#define rt_forchk __attribute__((used,noinline,regparm(0),stdcall)) rt_forchk

atom_t rt_forchk( atom_t source, atom_t* step_value, atom_t step, atom_t to, atom_t from )
#undef rt_forchk
{
  float64_t from_f;
  float64_t to_f;
  float64_t step_f;

  //printf("---<<< RT_FOR_CHK >>>---\n");
  //printf("  source="); dumpln(source);
  //printf("  from="); dumpln(from);
  //printf("  to="); dumpln(to);
  //printf("  step="); dumpln(step);
  //printf("  value="); dumpln(*step_value);

  // process the initial value
  if (IS_ERROR( from ))
  {
    DEUSE( to );
    DEUSE( step );
    add_error_source( from, source );
    return USE( from );
  }

  if (IS_UNBOUND( from ))
  {
    //DEUSE( from ); /// ???
    DEUSE( to );
    DEUSE( step );
    return new_error( ERROR_MISSING_VALUE, source );
  }

  if (!atom_to_float( from, &from_f ))
  {
    //DEUSE( from );
    DEUSE( to );
    DEUSE( step );
    return new_error( ERROR_NOT_A_NUMBER, source );
  }


  // process the final value
  if (IS_ERROR( to ))
  {
    DEUSE( from );
    DEUSE( step );
    add_error_source( to, CDR( source ) );
    return USE( to );
  }

  if (IS_UNBOUND( to ))
  {
    DEUSE( from );
    //DEUSE( to );
    DEUSE( step );
    return new_error( ERROR_MISSING_VALUE, CDR( source ) );
  }

  if (!atom_to_float( to, &to_f ))
  {
    //DEUSE( from );
    DEUSE( to );
    DEUSE( step );
    return new_error( ERROR_NOT_A_NUMBER, CDR( source ) );
  }



  // process the step
  if (IS_ERROR( step ))
  {
    DEUSE( from );
    DEUSE( to );
    add_error_source( step, CDR( CDR( source ) ) );
    return USE( step );
  }

  if (IS_UNBOUND( step ))
  {
    step_f = (to_f >= from_f) ? 1 : -1;
  }
  else
  if (!atom_to_float( step, &step_f ))
  {
    //DEUSE( from );
    DEUSE( to );
    DEUSE( step );
    return new_error( ERROR_NOT_A_NUMBER, CDR( CDR( source ) ) );
  }

  DEUSE( *step_value );
  *step_value = new_float( step_f );
  //printf("FOR from=%f to=%f step=%f\n",from_f,to_f,step_f);

  int64_t cnt;

  if ((to_f >= from_f) && (step_f > 0))
  cnt = floor( (to_f - from_f) / step_f ) + 1;
  else
  if ((to_f <= from_f) && (step_f < 0))
  cnt = floor( (to_f - from_f) / step_f ) + 1;
  else
  cnt = 0;

  //DEUSE( from );
  DEUSE( to );
  DEUSE( step );

  //printf("  repeat count=%Ld\n",cnt);
  //printf("---<<< end of RT_FOR_CHK >>>---\n");

  return new_integer( cnt );
}




//===================================================
/// \fn		atom_t rt_print( atom_t pdl, atom_t pwl, atom_t fpp, int data )
/// \brief	implementation of primitive PRINT
/// \param pdl	value of PRINTDEPTHLIMIT
/// \param pwl	value of PRINTWIDTHLIMIT
/// \param fpp	value of FULLPRINTP
/// \param data	number of inputs of PRINT
///
/// Implements the primitive \c PRINT. The only input
/// contains the number of inputs which are pulled
/// out from the stack.
//===================================================

atom_t RUNTIME rt_print( atom_t pdl, atom_t pwl, atom_t fpp, int data )
{
  //printf("pdl=");  dumpln(pdl);
  //printf("pwl=");  dumpln(pwl);
  //printf("fpp=");  dumpln(fpp);

  // check print-related variabples
  atom_to_integer( pdl, &print_depth_limit );
  atom_to_integer( pwl, &print_width_limit );
  atom_to_boolean( fpp, &full_print_p );

  // do the actual print
  atom_t ARGUMENT;
  for (EACH_ARGUMENT)
  {
    CHECK_PARAM( ARGUMENT );
    dump( ARGUMENT );
    outter( TEXT( " " ), 1 );
  };
  outter( TEXT( "\n" ), 1 );
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_plus( int count, atom_t data2, atom_t data1 )
/// \brief	implementation of primitive operator +
/// \param count	number of inputs of +
/// \param data1	first input
/// \param data2	second input
/// \return Number atom - sum of its arguments
///
/// Implements the primitive operator \c +. If \c +
/// is used as infix operator, then \c count=2,
/// otherwise \c count=1 and only data2 input is used.
//===================================================

atom_t RUNTIME rt_plus( int count, atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data2, y ); // to check whether data2 is number

  // process unary +
  if (count == 1) RETURN( USE( data2 ) );

  // process binary +
  GET_FLOAT( data1, x );
  RETURN( new_float( x + y ) );
}




//===================================================
/// \fn		atom_t rt_minus( int count, atom_t data2, atom_t data1 )
/// \brief	implementation of primitive operator -
/// \param count	number of inputs of -
/// \param data1	first input
/// \param data2	second input
/// \return Number atom - diference of its arguments
///
/// Implements the primitive operator \c -. If \c -
/// is used as infix operator, then \c count=2,
/// otherwise \c count=1 and only data2 input is used.
//===================================================

atom_t RUNTIME rt_minus( int count, atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data2, y );

  // process unary -
  if (count == 1) RETURN( new_float(-y ) );

  // process binary -
  GET_FLOAT( data1, x );
  RETURN( new_float( x - y ) );
}




//===================================================
/// \fn		atom_t rt_mul( atom_t data2, atom_t data1 )
/// \brief	implementation of primitive operator *
/// \param  data1	first input
/// \param  data2	second input
/// \return Number atom - product of it's arguments
///
/// Implements the primitive operator \c *.
//===================================================

atom_t RUNTIME rt_mul( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );

  RETURN( new_float( x * y ) );
}


//===================================================
/// \fn		atom_t rt_div( atom_t data2, atom_t data1 )
/// \brief	implementation of primitive operator /
/// \param  data1	first input
/// \param  data2	second input
/// \return Number atom - quotient of it's arguments
///
/// Implements the primitive operator \c /.
//===================================================

atom_t RUNTIME rt_div( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );

  RETURN( new_float( x / y ) );
}


//===================================================
/// \fn		atom_t rt_sum( int data )
/// \brief	implementation of aritmetic primitive SUM
/// \param  data	number of inputs of SUM
/// \return Number atom - sum of all arguments
///
/// Implements the primitive \c SUM. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Returns their sum
//===================================================

atom_t RUNTIME rt_sum( int data )
{
  float64_t acc = 0;
  float64_t x;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT) // Process each given argument
  {
    GET_FLOAT( ARGUMENT, x );
    acc += x;
  }
  RETURN( new_float( acc ) );
}


//===================================================
/// \fn		atom_t rt_difference( atom_t data2, atom_t data1 )
/// \brief	implementation of aritmetic primitive DIFFERENCE
/// \param data1	first input
/// \param data2	second input
/// \return Number atom - difference of arguments
///
/// Implements the  aritmetic primitive \c DIFFERENCE
//===================================================

atom_t RUNTIME rt_difference( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );

  RETURN( new_float( x - y ) );
}


//===================================================
/// \fn		atom_t rt_unminus (atom_t data)
/// \brief	implementation of aritmetic unary primitive MINUS
/// \param data	argument
/// \return Number atom - negate of the argument
///
/// Implements the aritmetic primitive \c MINUS
//===================================================

atom_t RUNTIME rt_unminus( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  RETURN( new_float(-x ) );
}


//===================================================
/// \fn		atom_t rt_product (int data)
/// \brief	implementation of aritmetic primitive PRODUCT
/// \param  data	number of inputs of PRODUCT
/// \return Number atom - product of all arguments
///
/// Implements the primitive \c PRODUCT. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Returns their product
//===================================================

atom_t RUNTIME rt_product( int data )
{
  float64_t acc = 1;
  float64_t x;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT) // Process each given argument
  {
    GET_FLOAT( ARGUMENT, x );
    acc *= x;
  }
  RETURN( new_float( acc ) );
}


//===================================================
/// \fn		atom_t rt_remainder( atom_t data2, atom_t data1 )
/// \brief	implementation of aritmetic primitive REMAINDER
/// \param  data1	first input
/// \param  data2	second input
/// \return Integer number atom - remainder of devision data1 / data2
///
/// Implements the aritmetic primitive \c REMAINDER
//===================================================

atom_t RUNTIME rt_remainder( atom_t data2, atom_t data1 )
{
  int64_t x, y;

  GET_INT( data1, x );
  GET_INT( data2, y );

  if( y )
  { RETURN( new_integer( x % y ) ); }
  else
  { RETURN( new_integer( 0 ) ); }
}


//===================================================
/// \fn		atom_t rt_int (atom_t data)
/// \brief	implementation of aritmetic unary primitive INT
/// \param  data	argument
/// \return Integer number atom - integer part of argument
///
/// Implements the aritmetic primitive \c INT
//===================================================

atom_t RUNTIME rt_int( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  RETURN( new_integer( (int64_t) x ) );
}


//===================================================
/// \fn     atom_t rt_round (atom_t data)
/// \brief	implementation of aritmetic unary primitive ROUND
/// \param  data	argument
/// \return Integer number atom - rounded integer nearest to argument
///
/// Implements the aritmetic primitive \c ROUND
//===================================================

atom_t RUNTIME rt_round( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );
  x += 0.5 * SIGN( x );
  RETURN( new_integer( (int64_t) x ) );
}


//===================================================
/// \fn		atom_t rt_sqrt (atom_t data)
/// \brief	implementation of aritmetic unary primitive SQRT
/// \param  data	argument
/// \return Number atom - square root of argument
///
/// Implements the aritmetic primitive \c SQRT
/// Calculates square root from the number
//===================================================

atom_t RUNTIME rt_sqrt( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  x = sqrt( x );
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_power (atom_t power, atom_t base)
/// \brief	implementation of aritmetic primitive POWER
/// \param base	base number
/// \param power	power to calculate
/// \return Number atom - \c base on \c power
///
/// Implements the  aritmetic primitive \c POWER
//===================================================

atom_t RUNTIME rt_power( atom_t power, atom_t base )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( base, x );
  GET_FLOAT( power, y );

  x = pow( x, y );
  
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_exp (atom_t power)
/// \brief	implementation of aritmetic primitive EXP
/// \param power	power to calculate
/// \return Number atom - \c e on \c power
///
/// Outputs e (2.718281828+) to the input power.
//===================================================

atom_t RUNTIME rt_exp( atom_t power )
{
  float64_t x;

  GET_FLOAT( power, x );

  x = exp( x );
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_log10 (atom_t data)
/// \brief	implementation of aritmetic primitive LOG10
/// \param data	argument
/// \return Number atom - logarithm of argument
///
/// Outputs the common logarithm of the input
//===================================================

atom_t RUNTIME rt_log10( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  x = log10( x );
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_ln (atom_t data)
/// \brief	implementation of aritmetic primitive LN
/// \param data	argument
/// \return Number atom - natural logarithm of argument
///
/// Outputs  natural logarithm of the input
//===================================================

atom_t RUNTIME rt_ln( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  x = log( x );
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_abs (atom_t data)
/// \brief	implementation of aritmetic primitive ABS
/// \param data	argument
/// \return Number atom - absolute value of argument
///
/// Outputs the absolute value of the input
//===================================================

atom_t RUNTIME rt_abs( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  x = fabs( x );
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_pi (void)
/// \brief	implementation of aritmetic primitive PI
/// \return Number atom - PI number (3.141592...)
///
/// Outputs  Value of PI
//===================================================

atom_t RUNTIME rt_pi( void )
{
  RETURN( new_float( M_PI ) );
}


//===================================================
/// \fn		atom_t rt_sin (atom_t data)
/// \brief	implementation of aritmetic primitive SIN
/// \param data	argument in degrees
/// \return Number atom - Sine of argument
///
/// Outputs the sine of its input, which is taken in degrees
//===================================================

atom_t RUNTIME rt_sin( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  // Converts degrees to radians
  x = (x / 180) * M_PI;
  x = sin( x );

  // Correct precision error from converting degrees to radians
  if (fabs( x ) < ZERO_PRECISION)
  {
    x = 0;
  }
  else
  if (fabs( 0.5 + x ) < ZERO_PRECISION)
  {
    x = -0.5;
  }
  else
  if (fabs( 0.5 - x ) < ZERO_PRECISION)
  {
    x = 0.5;
  }
  else
  if (fabs( 1 + x ) < ZERO_PRECISION)
  {
    x = -1;
  }
  else
  if (fabs( 1 - x ) < ZERO_PRECISION)
  {
    x = 1;
  }

  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_radsin (atom_t data)
/// \brief	implementation of aritmetic primitive RADSIN
/// \param data	argument in radians
/// \return Number atom - Sine of argument
///
/// Outputs the sine of its input, which is taken in radians
//===================================================

atom_t RUNTIME rt_radsin( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  x = sin( x );
  if (fabs( x ) < ZERO_PRECISION)
  {
    x = 0;
  }
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_cos (atom_t data)
/// \brief	implementation of aritmetic primitive COS
/// \param data	argument in degrees
/// \return Number atom - Cosine of argument
///
/// Outputs the cosine of its input, which is taken in degrees
//===================================================

atom_t RUNTIME rt_cos( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  // Converts degrees to radians
  x = (x / 180) * M_PI;
  x = cos( x );

  // Correct precision error from converting degrees to radians
  if (fabs( x ) < ZERO_PRECISION)
  {
    x = 0;
  }
  else
  if (fabs( 0.5 + x ) < ZERO_PRECISION)
  {
    x = -0.5;
  }
  else
  if (fabs( 0.5 - x ) < ZERO_PRECISION)
  {
    x = 0.5;
  }
  else
  if (fabs( 1 + x ) < ZERO_PRECISION)
  {
    x = -1;
  }
  else
  if (fabs( 1 - x ) < ZERO_PRECISION)
  {
    x = 1;
  }

  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_radcos (atom_t data)
/// \brief	implementation of aritmetic primitive RADCOS
/// \param data	argument in radians
/// \return Number atom - Cosine of argument
///
/// Outputs the cosine of its input, which is taken in radians
//===================================================

atom_t RUNTIME rt_radcos( atom_t data )
{
  float64_t x;

  GET_FLOAT( data, x );

  x = cos( x );
  if (fabs( x ) < ZERO_PRECISION)
  {
    x = 0;
  }

  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_arctan (int count, atom_t data2, atom_t data1)
/// \brief	implementation of aritmetic primitive ARCTAN
/// \param count	number of arguments (1 or 2)
/// \param data1	first argument
/// \param data2	second argument
/// \return Number atom - Arctangent of argument(s)
///
/// Outputs the arctangent, in degrees, of its input.
//===================================================

atom_t RUNTIME rt_arctan( int count, atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data2, y );

  if (count == 1)
  {
    x = atan( y );
  }
  else
  {
    GET_FLOAT( data1, x );
    x = atan2( x, y );
  }

  //Convert radians to degrees.
  x = (x * 180) / M_PI;
  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_radarctan (int count, atom_t data2, atom_t data1)
/// \brief	implementation of aritmetic primitive RADARCTAN
/// \param count	number of arguments (1 or 2)
/// \param data1	first argument
/// \param data2	second argument
/// \return Number atom - Arctangent of argument(s)
///
/// Outputs the arctangent, in radians, of its input.
//===================================================

atom_t RUNTIME rt_radarctan( int count, atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data2, y );

  if (count == 1)
  {
    x = atan( y );
  }
  else
  {
    GET_FLOAT( data1, x );
    x = atan2( x, y );
  }

  RETURN( new_float( x ) );
}


//===================================================
/// \fn		atom_t rt_make( int static_link, atom_t parent, atom_t value, atom_t name )
/// \brief		implementation of primitive command MAKE
/// \param static_link	static link from the current frame
/// \param parent	current parent
/// \param name		name of variable
/// \param value	new value of variable
///
/// Implements the primitive command \c MAKE. This
/// function is called when it is not possible to
/// compile \c MAKE. This happens when the name of
/// the variable is an expression or is unknown.
/// In such cases the search for the variable must
/// be done in run-time.
//===================================================

atom_t RUNTIME rt_make( int static_link, atom_t parent, atom_t value, atom_t name )
{
  //printf("rt_make, var="); dumpln(name);
  //printf("rt_make, val="); dumpln(value);

  if (IS_ERROR( value )) RETURN( USE(value) );
  if (IS_ERROR( name )) RETURN( USE(name) );

  atom_t word = atom_to_word( name );
  if (IS_ERROR( word )) RETURN( word );

  atom_t var = find_runtime_var( word, static_link );

  // not found - create a global one
  if (!var)
  {
    var = new_var( word, globals, 1 );
    SET_FLAGS( var, FLAG_VARIABLE );
    VARTYPE( var ) = VAR_TYPE_RUNTIME;
    VALUE( var ) = USE( unbound );
  }

  // var name as a word is not needed any more
  DEUSE( word );

  // not a variable (i.e. it is a function or a procedure)
  if (!IS_VARIABLE( var ))
    RETURN( new_error( ERROR_NOT_A_VAR, name ) );

  //printf("rt_make, var="); dumpln(var);
  //printf("rt_make, val="); dumpln(value);
  rt_set_var_value( static_link, parent, var, value );
  //printf("value is set, see var(%x)=",(int)var); dumpln(var);
  //printf("        its parent is="); dumpln(PARENT(var));
  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_name( int static_link, atom_t parent, atom_t name, atom_t value )
/// \brief		implementation of primitive command NAME
/// \param static_link	static link from the current frame
/// \param parent	current parent
/// \param name		name of variable
/// \param value	new value of variable
///
/// Implements the primitive command \c NAME. It
/// uses the same code as MAKE.
//===================================================

atom_t RUNTIME rt_name( int static_link, atom_t parent, atom_t name, atom_t value )
{
  // no need to use RETURN, because rt_make() has a RETURN
  return ( rt_make( static_link, parent, value, name ));
}



//===================================================
/// \fn		atom_t rt_local( int static_link, atom_t parent, int data )
/// \brief	dummy implementation of primitive command LOCAL
/// \param static_link	static link from the current frame
/// \param parent	current parent
/// \param data		count of LOCAL's arguments
///
/// Implements the primitive command \c LOCAL.
/// \c LOCAL is processed by the parser, so this
/// function is not called.
//===================================================

atom_t RUNTIME rt_local( int static_link, atom_t parent, int data )
{
  atom_t* localsp;
  localsp = (atom_t*) (static_link + BASE_OFFSET_LOCALS);
  //printf("rt_local\n");
  atom_t ARGUMENT;
  for (EACH_ARGUMENT)
  {
    //printf("creating local var: "); dumpln(ARGUMENT);
    if (IS_ERROR( ARGUMENT )) RETURN( USE( ARGUMENT ) );
    if (IS_UNBOUND( ARGUMENT ))
    {
      return new_error( ERROR_MISSING_VALUE, unbound );
    }

    // check whether this variable exists locally
    if (find_local_var( ARGUMENT, parent ) ||
        find_local_var( ARGUMENT, *localsp ))
    {
      return new_error( ERROR_DUPLICATE_INPUT, ARGUMENT );
    }

    atom_t var = new_var( ARGUMENT, parent, 0 );
    SET_FLAGS( var, FLAG_VARIABLE );
    VALUE( var ) = USE( unbound );
    VARTYPE( var ) = VAR_TYPE_RUNTIME;
    *localsp = new_list( var, *localsp ); // attach to other runtimers
  };
  //printf("new locals="); dumpln(*localsp);
  RETURN( unbound );
}


//===================================================
/// \fn		int num_compare (float64_t x, float64_t y)
/// \brief	compares two numbers.
/// \param x	first number
/// \param y	second number
/// \return     Compatison result
///
///  Compares two numbers.
/// If first is larger returns positive number,
/// if first is smaller returns negative number,
/// if both are equal returns 0
//===================================================

int num_compare( float64_t x, float64_t y )
{
  if (x < y) return -1;
  if (y < x) return 1;
  return 0; // x == y
}


//===================================================
/// \fn		int word_compare (int cip, chars_t w1, int w1_len, chars_t w2, int w2_len)
/// \brief	compares two words.
/// \param cip	to ignote or not case in comparison
/// \param w1	first word
/// \param w1_len   first word length
/// \param w2	second word
/// \param w2_len   second word length
/// \return     Compatison result
///
///  Compares two numbers.
/// If first is larger returns positive number,
/// if first is smaller returns negative number,
/// if both are equal returns 0
//===================================================

int word_compare( int cip, chars_t w1, int w1_len, chars_t w2, int w2_len )
{
  if (!cip)
  {
    while (w1_len && w2_len)
    {
      if (DEBAR( w1[0] ) != DEBAR( w2[0] ))
      return DEBAR( w1[0] ) - DEBAR( w2[0] );
      ++w1;
      ++w2;
      --w1_len;
      --w2_len;
    }
  }
  else
  {
    while (w1_len && w2_len)
    {
      if (TOLOWER( DEBAR( w1[0] ) ) != TOLOWER( DEBAR( w2[0] ) ))
      return TOLOWER( DEBAR( w1[0] ) ) - TOLOWER( DEBAR( w2[0] ) );
      ++w1;
      ++w2;
      --w1_len;
      --w2_len;
    }
  }
  return w1_len - w2_len;
}


// Function prototype
int atom_equal( atom_t cip, atom_t data2, atom_t data1 );

//===================================================
/// \fn		list_equal(atom_t cip, atom_t data1, atom_t data2)
/// \brief	compares two lists.
/// \param cip		value of CASEIGNOREDP
/// \param data1	first list
/// \param data2	second list
/// \return     != 0 number if lists are equal or 0 if they are not
///
///  Compares two lists.
/// if both are equal returns not 0 number
/// Returns 0 if lists are diferent.
//===================================================

int list_equal( atom_t cip, atom_t data1, atom_t data2 )
{
  int comp;
  //atom_t comp_res;

  while (IS_NOT_EMPTY( data1 ) && IS_NOT_EMPTY( data2 ))
  {
    comp = atom_equal( cip, CAR( data2 ), CAR( data1 ) );
    if (comp <= 0)
    {
      return comp;
    }

    data1 = CDR( data1 );
    data2 = CDR( data2 );
  }
  return (IS_EMPTY( data1 ) && IS_EMPTY( data2 ));
}


//===================================================
/// \fn		atom_equal(atom_t cip, atom_t data1, atom_t data2)
/// \brief	compares two atoms.
/// \param cip		value of CASEIGNOREDP
/// \param data1	first atom
/// \param data2	second atom
/// \return     != 0 number if lists are equal or 0 if they are not
///
///  Compares two atoms.
/// if both are same types and have equal values returns positive number
/// Returns 0 if atoms are diferent.
/// Returns -1 if some error occurs
//===================================================

int atom_equal( atom_t cip, atom_t data2, atom_t data1 )
{
  float64_t x;
  int case_ignore = 0;

  //  printf("*****00000000****"); dump_statistics();

  // test if atoms are different types and one is list
  // in that case they cannot be equal
  if ((IS_LIST( data1 ) || IS_LIST( data2 )) &&
      (ID( data1 ) != ID( data2 )))
  {
    return 0;
  }

  // Now we know that both are lists or both are not lists
  if (IS_LIST( data1 ))
  {
    return list_equal( cip, data1, data2 );
  }

  if (!atom_to_boolean( cip, &case_ignore ))
  {
    case_ignore = 1;
  }

  // If both atoms are words - best way to compare them is directly
  if (IS_ANY_WORD( data1 ) && IS_ANY_WORD( data2 ))
  {
    int xx = word_compare( case_ignore, STRING( data1 ), LENGTH( data1 ), STRING( data2 ), LENGTH( data2 ) ) == 0;
    return xx;
  }

  // Here we are in case that both are numbers or unsupported types
  if (atom_to_float( data1, &x ))
  {
    // First is a number
    float64_t y;
    if (!atom_to_float( data2, &y ))
    {
      return 0;
    }
    return num_compare( x, y ) == 0; // Both are numbers
  }

  return -1; //Different or unsupproted atom types -> error
}



//===================================================
/// \fn		atom_t rt_equal (atom_t cip, atom_t data2, atom_t data1)
/// \brief	implementation compare operation EQUALP
/// \param cip		value of CASEIGNOREDP
/// \param data1	first input
/// \param data2	second input
/// \return Boolean atom
///
/// Implements compare operation EQUALP
//===================================================

atom_t RUNTIME rt_equal( atom_t cip, atom_t data2, atom_t data1 )
{
  int comp;
  // parameter checking
  if (IS_ERROR( data1 )) RETURN( USE(data1) );
  if (IS_ERROR( data2 )) RETURN( USE(data2) );

  comp = atom_equal( cip, data2, data1 );
  if (comp < 0)
  {
    RETURN( new_error( ERROR_MISSING_VALUE, data2 ) );
  }

  RETURN( USE( false_true[comp > 0] ) );
}


//===================================================
/// \fn		atom_t rt_nequal (atom_t cip, atom_t data2, atom_t data1)
/// \brief	implementation compare operation NOTEQUALP
/// \param cip		value of CASEIGNOREDP
/// \param data1	first input
/// \param data2	second input
/// \return Boolean atom
///
/// Implements compare operation NOTEQUALP
//===================================================

atom_t RUNTIME rt_nequal( atom_t cip, atom_t data2, atom_t data1 )
{
  int comp;
  // parameter checking
  if (IS_ERROR( data1 )) RETURN( USE(data1) );
  if (IS_ERROR( data2 )) RETURN( USE(data2) );

  comp = atom_equal( cip, data2, data1 );
  if (comp < 0)
  {
    RETURN( new_error( ERROR_MISSING_VALUE, data2 ) );
  }

  RETURN( USE( false_true[comp == 0] ) );
}


//===================================================
/// \fn		atom_t rt_less (atom_t data2, atom_t data1)
/// \brief	implementation compare operation LESSP
/// \param data1	first number
/// \param data2	second number
/// \return Boolean atom
///
/// Implements compare operation LESSP
/// Arguments must be numbers
//===================================================

atom_t RUNTIME rt_less( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );

  RETURN( USE( false_true[x < y] ) );
}


//===================================================
/// \fn		atom_t rt_more (atom_t data2, atom_t data1)
/// \brief	implementation compare operation GREATERP
/// \param data1	first number
/// \param data2	second number
/// \return Boolean atom
///
/// Implements compare operation GREATERP
/// Arguments must be numbers
//===================================================

atom_t RUNTIME rt_more( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );

  RETURN( USE( false_true[x > y] ) );
}


//===================================================
/// \fn		atom_t rt_lesseq (atom_t data2, atom_t data1)
/// \brief	implementation compare operation LESSEQUALP
/// \param data1	first number
/// \param data2	second number
/// \return Boolean atom
///
/// Implements compare operation LESSEQUALP
/// Arguments must be numbers
//===================================================

atom_t RUNTIME rt_lesseq( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );

  RETURN( USE( false_true[x <= y] ) );
}


//===================================================
/// \fn		atom_t rt_moreeq (atom_t data2, atom_t data1)
/// \brief	implementation compare operation GREATEREQUALP
/// \param data1	first number
/// \param data2	second number
/// \return Boolean atom
///
/// Implements compare operation GREATEREQUALP
/// Arguments must be numbers
//===================================================

atom_t RUNTIME rt_moreeq( atom_t data2, atom_t data1 )
{
  float64_t x;
  float64_t y;

  GET_FLOAT( data1, x );
  GET_FLOAT( data2, y );
  RETURN( USE( false_true[x >= y] ) );
}


//===================================================
/// \fn		atom_t rt_before (atom_t cip, atom_t data2, atom_t data1)
/// \brief	implementation compare operation BEFOREP
/// \param cip		value of CASEIGNOREDP
/// \param data1	first word
/// \param data2	second word
/// \return Boolean atom
///
/// Implements compare operation BEFOREP
/// Arguments must be words. Note that if the inputs are
///	numbers, the result may not be the same as with LESSP;
/// for example, BEFOREP 3 12 is false because 3 collates after 1.
//===================================================

atom_t RUNTIME rt_before( atom_t cip, atom_t data2, atom_t data1 )
{
  char_t x[MAX_NUMBER_WORD_LENGTH], y[MAX_NUMBER_WORD_LENGTH];
  int len_x = MAX_NUMBER_WORD_LENGTH;
  int len_y = MAX_NUMBER_WORD_LENGTH;
  chars_t word_x;
  chars_t word_y;
  int case_ignore = 0;


  // parameter checking
  if (IS_ERROR( data1 )) RETURN( USE(data1) );
  if (IS_ERROR( data2 )) RETURN( USE(data2) );

  if (IS_ANY_WORD( data1 ))
  {
    word_x = STRING( data1 );
    len_x = LENGTH( data1 );
  }
  else if (atom_to_string( data1, x, &len_x ))
  {
    word_x = x;
  }
  else
  {
    RETURN( USE( false_true[0] ) );
  }

  if (IS_ANY_WORD( data2 ))
  {
    word_y = STRING( data2 );
    len_y = LENGTH( data2 );
  }
  else if (atom_to_string( data2, y, &len_y ))
  {
    word_y = y;
  }
  else
  {
    RETURN( USE( false_true[0] ) );
  }

  if (!atom_to_boolean( cip, &case_ignore ))
  {
    case_ignore = 1;
  }

  RETURN( USE( false_true[word_compare( case_ignore, word_x, len_x, word_y, len_y ) < 0] ) );
}



//===================================================
/// \fn		atom_t rt_output( atom_t data )
/// \brief	dummy implementation of primitive command OUTPUT
/// \param data return value of the Logo program
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for OUTPUT
/// primitive.
//===================================================

atom_t RUNTIME rt_output( atom_t data )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_maybeoutput( atom_t data )
/// \brief	dummy implementation of primitive command MAYBEOUTPUT
/// \param data return value of the Logo program
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for MAYBEOUTPUT
/// primitive.
//===================================================

atom_t RUNTIME rt_maybeoutput( atom_t data )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_stop( )
/// \brief	dummy implementation of primitive command STOP
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for STOP
/// primitive.
//===================================================

atom_t RUNTIME rt_stop( )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t  rt_dump( atom_t source )
/// \brief		dump source command
/// \param	source	source of the command
/// \return		unbound atom
///
/// This function is used to dump the source of some
/// comands (like OUTPUT).
//===================================================
#define rt_dump __attribute__((used,noinline,regparm(0),stdcall)) rt_dump

atom_t rt_dump( atom_t source )
#undef rt_dump
{
#ifdef ADVANCED
  if (OPTION_RUNTIME)
  {
    outter( TEXT( "DONE " ), -1 );
    dumpln( source );
  }
#endif

  return unbound;
}



//===================================================
/// \fn		atom_t  rt_predump( atom_t source )
/// \brief		dump source command
/// \param	source	source of the command
/// \return		unbound atom
///
/// This function is used to predump the source of
/// user-defined functions and commands. This is done
/// in order to trace the function call before the
/// tracing of function's block.
//===================================================
#define rt_predump __attribute__((used,noinline,regparm(0),stdcall)) rt_predump

atom_t rt_predump( atom_t source )
#undef rt_predump
{
#ifdef ADVANCED
  if (OPTION_RUNTIME)
  {
    outter( TEXT( "TODO " ), -1 );
    dumpln( source );
  }
#endif

  return unbound;
}



//===================================================
/// \fn		atom_t rt_and (int data)
/// \brief	implementation of boolean primitive AND
/// \param  data	number of inputs of AND
/// \return Boolean atom
///
/// Implements the primitive \c AND. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Returns \c true if all are \c true
/// \c false else.
//===================================================

atom_t RUNTIME rt_and( int data )
{
  int param;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT)
  {
    GET_BOOLEAN( ARGUMENT, param );
    if (!param)
    {
      RETURN( USE( false_true[0] ) );
    }
  }
  RETURN( USE( false_true[1] ) );
}


//===================================================
/// \fn		atom_t rt_or (int data)
/// \brief	implementation of boolean primitive OR
/// \param  data	number of inputs of OR
/// \return Boolean atom
///
/// Implements the primitive \c OR. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Returns \c true if any is \c true
/// \c false if all are \c false.
//===================================================

atom_t RUNTIME rt_or( int data )
{
  int param;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT)
  {
    GET_BOOLEAN( ARGUMENT, param );
    if (param)
    {
      RETURN( USE( false_true[1] ) );
    }
  }
  RETURN( USE( false_true[0] ) );
}


//===================================================
/// \fn		atom_t rt_not (atom_t data)
/// \brief	implementation of boolean primitive NOT
/// \param  data	boolean argument
/// \return Boolean atom
///
/// Outputs \c true is argument is \c false and
/// \c false if argument is \c true
//===================================================

atom_t RUNTIME rt_not( atom_t data )
{
  int result;
  GET_BOOLEAN( data, result );
  RETURN( USE( false_true[1 - result] ) );
}


//===================================================
/// \fn		atom_t rt_ignore (atom_t data)
/// \brief	implementation of primitive IGNORE
/// \param data	value to ignore
/// \return	unbound atom
///
/// Implementation of the primitive command IGNORE.
/// The value in data is ignored and the returned
/// value is the unbound atom
//===================================================

atom_t RUNTIME rt_ignore( atom_t data )
{
  if (IS_ERROR( data ))
  {
    RETURN( USE( data ) );
  }
  else
  {
    RETURN( USE( unbound ) );
  }
}


//===================================================
/// \fn		atom_t rt_first (atom_t data)
/// \brief	implementation of primitive FIRST
/// \param  data    word or list argument
/// \return first element of input as atom
///
/// Outputs first letter of argument if it is word
/// or first element of argument if it is list
//===================================================

atom_t RUNTIME rt_first( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  // parameter checking
  if (IS_ERROR( data )) RETURN( USE(data) );

  if (IS_LIST( data ))
  {
    if (IS_EMPTY( data ))
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( USE( CAR( data ) ) );
  }
  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_subword( data, STRING( data ), 1 ) );
  }
  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_word( buff, 1 ) );
  }
  RETURN( new_error( ERROR_MISSING_VALUE, data ) );
}


//===================================================
/// \fn		atom_t rt_butfirst (atom_t data)
/// \brief	implementation of primitive BUTFIRST
/// \param  data    word or list argument
/// \return all elements of input without first
///
/// If argument is word outputs word without first letter
/// if argument is list outputs list without first element
//===================================================

atom_t RUNTIME rt_butfirst( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  // parameter checking
  if (IS_ERROR( data )) RETURN( USE(data) );

  if (IS_LIST( data ))
  {
    if (IS_EMPTY( data ))
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( USE( CDR( data ) ) );
  }
  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( new_subword( data, STRING( data ) + 1, LENGTH( data ) - 1 ) );
  }
  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_word( buff + 1, buff_len - 1 ) );
  }
  RETURN( new_error( ERROR_MISSING_VALUE, data ) );
}


//===================================================
/// \fn	atom_t rt_firsts (atom_t data)
/// \brief	implementation of primitive FIRSTS
/// \param  data    A list
/// \return list of first element of each element of input list
///
/// Inserts in result first letter of element if it is word
/// or first element of element if it is list
//===================================================

atom_t RUNTIME rt_firsts( atom_t data )
{
  atom_t result, iter;
  result = iter = empty_list;

  // parameter checking
  if (IS_ERROR( data )) RETURN( USE(data) );
  if (!IS_LIST( data )) RETURN( new_error( ERROR_NOT_A_LIST, data ) );

  while (!IS_EMPTY( data ))
  {
    append( rt_first( CAR( data ) ), &result, &iter );
    data = CDR( data );
  }
  RETURN( result );
}

//===================================================
/// \fn		atom_t rt_butfirsts(atom_t data)
/// \brief	implementation of primitive BUTFIRSTS and BFS
/// \param  data   A list
/// \return list of all elements of each element of \c data without first
///
/// If element is word inserts in result word without first letter;
/// if element is list inserts in result the list without its first element
//===================================================

atom_t RUNTIME rt_butfirsts( atom_t data )
{
  atom_t result, iter;
  result = iter = empty_list;

  // parameter checking
  if (IS_ERROR( data )) RETURN( USE(data) );
  if (!IS_LIST( data )) RETURN( new_error( ERROR_NOT_A_LIST, data ) );

  while (!IS_EMPTY( data ))
  {
    append( rt_butfirst( CAR( data ) ), &result, &iter );
    data = CDR( data );
  }
  RETURN( result );
}


//===================================================
/// \fn		atom_t rt_last (atom_t data)
/// \brief	implementation of primitive LAST
/// \param  data    word or list argument
/// \return last element of input as atom
///
/// Outputs last letter of argument if it is word
/// or last element of argument if it is list
//===================================================

atom_t RUNTIME rt_last( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  // parameter checking
  if (IS_ERROR( data )) RETURN( USE(data) );

  if (IS_LIST( data ))
  {
    if (IS_EMPTY( data ))
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( USE( get_at_list( data, -1 ) ) );
  }
  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_subword( data, STRING( data ) + LENGTH( data ) - 1, 1 ) );
  }
  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_word( buff + buff_len - 1, 1 ) );
  }
  RETURN( new_error( ERROR_MISSING_VALUE, data ) );
}


//===================================================
/// \fn		atom_t rt_butlast (atom_t data)
/// \brief	implementation of primitive BUTALST
/// \param  data    word or list argument
/// \return all elements of input without last
///
/// If argument is word outputs word without last letter
/// if argument is list outputs list without last element
//===================================================

atom_t RUNTIME rt_butlast( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  // parameter checking
  if (IS_ERROR( data )) RETURN( USE(data) );

  if (IS_LIST( data ))
  {
    if (IS_EMPTY( data ))
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( list_copy_but_last( data ) );
  }
  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( new_subword( data, STRING( data ), LENGTH( data ) - 1 ) );
  }
  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len == 0)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_word( buff, buff_len - 1 ) );
  }
  RETURN( new_error( ERROR_MISSING_VALUE, data ) );
}


//===================================================
/// \fn		atom_t rt_item (atom_t data, atom_t index)
/// \brief	implementation of primitive ITEM
/// \param  data    word or list argument
/// \param  index   integer index of element to get
/// \return The element at sepcified index
///
/// If argument is word outputs the char at position \c index
/// if argument is list outputs the element at position \c index
/// Indexing starts at 1 in both cases
//===================================================

atom_t RUNTIME rt_item( atom_t data, atom_t index )
{
  int64_t ind;
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  // parameters check
  if (IS_ERROR( data )) RETURN( USE(data) );
  GET_INT( index, ind );

  if (ind <= 0)
  {
    RETURN( new_error( ERROR_NOT_A_NUMBER, index ) );
  }

  if (IS_LIST( data ))
  {
    if (IS_EMPTY( data ))
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }
    RETURN( USE( get_at_list( data, ind - 1 ) ) ); // -1 cause indexing starts from 1
  }
  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) < ind)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_subword( data, STRING( data ) + ind - 1, 1 ) );
  }
  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len < ind)
    {
      RETURN( new_error( ERROR_MISSING_VALUE, data ) );
    }

    RETURN( new_word( buff + ind - 1, 1 ) );
  }
  RETURN( new_error( ERROR_MISSING_VALUE, data ) );
}




//===================================================
/// \fn		atom_t rt_if( int count )
/// \brief	dummy implementation of primitive command IF
/// \param count number of parameters
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for IF
/// primitive.
//===================================================

atom_t RUNTIME rt_if( int count )
{
  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_repeat( atom_t repcount, atom_t commands )
/// \brief	dummy implementation of primitive command REPEAT
/// \param repcount number of repetitions
/// \param commands commands to repeat
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for REPEAT
/// primitive.
//===================================================

atom_t RUNTIME rt_repeat( atom_t repcount, atom_t commands )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_while( atom_t condition, atom_t commands )
/// \brief	dummy implementation of primitive command WHILE
/// \param condition	repetition condition
/// \param commands	commands to repeat
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for WHILE
/// primitive.
//===================================================

atom_t RUNTIME rt_while( atom_t condition, atom_t commands )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_while( atom_t commands, atom_t condition )
/// \brief	dummy implementation of primitive command DO.WHILE
/// \param commands	commands to repeat
/// \param condition	repetition condition
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for DO.WHILE
/// primitive.
//===================================================

atom_t RUNTIME rt_dowhile( atom_t commands, atom_t condition )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_until( atom_t condition, atom_t commands )
/// \brief	dummy implementation of primitive command UNTIL
/// \param condition	repetition condition
/// \param commands	commands to repeat
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for UNTIL
/// primitive.
//===================================================

atom_t RUNTIME rt_until( atom_t condition, atom_t commands )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_dountil( atom_t commands, atom_t condition )
/// \brief	dummy implementation of primitive command UNTIL
/// \param commands	commands to repeat
/// \param condition	repetition condition
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for DO.UNTIL
/// primitive.
//===================================================

atom_t RUNTIME rt_dountil( atom_t commands, atom_t condition )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_whlchk( atom_t source, atom_t data )
/// \brief		checks for valid while condition
/// \param	data	value to check
/// \param	source	source of the expression
/// \return		error or unbound atom
///
/// Checks whether the condition of a \c while command
/// is valid. Returns error atom is the result is error,
/// or not boolean. Otherwise returns integer atoms
/// containing 0 if the condition is false, and 1
/// otherwise. The \c source parameter points to
/// the source where the expression is called. It is
/// used to locate the error position in case of errors.
//===================================================
#define rt_whlchk __attribute__((used,noinline,regparm(0),stdcall)) rt_whlchk

atom_t rt_whlchk( atom_t source, atom_t data )
#undef rt_whlchk
{
  if (IS_ERROR( data ))
  {
    add_error_source( data, source );
    return USE( data );
  }

  if (IS_UNBOUND( data ))
  {
    DEUSE( data );
    return new_error( ERROR_MISSING_VALUE, source );
  }

  int b;
  if (!atom_to_boolean( data, &b ))
  {
    DEUSE( data );
    return new_error( ERROR_BOOLEAN_EXPECTED, source );
  }

  DEUSE( data );

  return new_integer( b );
}




//===================================================
/// \fn		void rt_repeat_enter( int frame, int count )
/// \brief	initializes a repeat loop
/// \param frame		base frame pointer
/// \param count	number of requested repetitions
///
/// This function creates a new repeat node and
/// inserts it in the beginning of the repeat chain.
/// The repeat-node contains the number of repetitions
/// done so far (::REPCOUNT) and left to do (::REPLIMIT).
/// The base pointer is used to access the repeat chain.
//===================================================
#define void void __attribute__ ((used,noinline,regparm(0),stdcall))

void rt_repeat_enter( int frame, int count )
#undef void
{
  atom_t* chain_ptr;
  atom_t node;

  node = new_integer( 0 );
  REPCOUNT( node ) = 1;
  REPLIMIT( node ) = count;

  chain_ptr = (atom_t*) (frame + BASE_OFFSET_REPEATCHAIN);
  if (IS_UNBOUND( *chain_ptr ))
  {
    // we create the first repeat-node
    DEUSE( *chain_ptr );
    *chain_ptr = new_list( node, empty_list );
  }
  else
  {
    // we add a new repeat-node
    *chain_ptr = new_list( node, *chain_ptr );
  }
}



//===================================================
/// \fn		atom_t rt_repeat_exit( int frame )
/// \brief	finalizes a repeat loop
/// \param frame	base frame pointer
/// \return		unbound atom
///
/// This function removes the top-most repeat-node
/// and returns unbound atom.
//===================================================
#define void atom_t __attribute__ ((used,noinline,regparm(0),stdcall))

void rt_repeat_exit( int frame )
#undef void
{
  atom_t* chain_ptr;

  chain_ptr = (atom_t*) (frame + BASE_OFFSET_REPEATCHAIN);

  if (!IS_UNBOUND( *chain_ptr ))
  {
#ifdef SAFEMODE
    assert( IS_LIST( *chain_ptr ) );
#endif

    *chain_ptr = behead( *chain_ptr );

    if (IS_EMPTY( *chain_ptr ))
    *chain_ptr = USE( unbound );
  }

  //printf("exitus repeatus\n");
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_repcount( int frame )
/// \brief	implementation of primitive function REPCOUNT
/// \param frame	base frame pointer
/// \return	numeric atom containing repcount value
///
/// Repetition count is always stored at the top of
/// the stack, so the implementation of \c repcount
/// naturally treats the top of the stack as its
/// parameter.
//===================================================

atom_t RUNTIME rt_repcount( int frame )
{
  atom_t* chain_ptr;

  while (frame)
  {
    chain_ptr = (atom_t*) (frame + BASE_OFFSET_REPEATCHAIN);

    if (IS_LIST( *chain_ptr ))
    RETURN( new_integer( REPCOUNT( CAR( *chain_ptr ) ) ) );

    frame = *((int*) (frame));
  }

  RETURN( new_integer(-1 ) );
}



//===================================================
/// \fn		atom_t rt_forever( atom_t commands )
/// \brief	dummy implementation of primitive command FOREVER
/// \param commands commands to repeat
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for FOREVER
/// primitive.
//===================================================

atom_t RUNTIME rt_forever( atom_t commands )
{
  RETURN( unbound );
}



//===================================================
/// \fn		void rt_forever_enter( int frame )
/// \brief	initializes a forever loop
/// \param frame	base frame pointer
///
/// This function creates a new repeat node and
/// inserts it in the beginning of the repeat chain.
/// The repeat-node contains the number of repetitions
/// done so far (::REPCOUNT). The limit of repetitions
/// (::REPLIMIT) is set to 1, although this value is
/// not used.
///
/// The base pointer is used to access the repeat chain.
//===================================================
#define void void __attribute__ ((used,noinline,regparm(0),stdcall))

void rt_forever_enter( int frame )
#undef void
{
  atom_t* chain_ptr;
  atom_t node;

  node = new_integer( 0 );
  REPCOUNT( node ) = 1;
  REPLIMIT( node ) = 1;

  chain_ptr = ((atom_t*) (frame + BASE_OFFSET_REPEATCHAIN));
  if (IS_UNBOUND( *chain_ptr ))
  {
    // we create the first repeat-node
    DEUSE( *chain_ptr );
    *chain_ptr = new_list( node, empty_list );
  }
  else
  {
    // we add a new repeat-node
    *chain_ptr = new_list( node, *chain_ptr );
  }
}



//===================================================
/// \fn		atom_t rt_parse (atom_t data)
/// \brief	implementation of PARSE
/// \param data	data to parse
/// \return parsed data
///
/// Implements primitive function PARSE. Returns the
/// input parsed as data.
//===================================================

atom_t RUNTIME rt_parse( atom_t data )
{
  if (IS_ERROR( data ))
  RETURN( USE(data) );

  RETURN( tokenize( data, TOKENIZE_DATA ) );
}




//===================================================
/// \fn		atom_t rt_runparse (atom_t data)
/// \brief	implementation of RUNPARSE
/// \param data	data to parse
/// \return parsed data
///
/// Implements primitive function RUNPARSE. Returns
/// the input parsed as commands.
//===================================================

atom_t RUNTIME rt_runparse( atom_t data )
{
  if (IS_ERROR( data ))
  RETURN( USE(data) );

  //printf("dddaaataa="); dumpln(data);
  atom_t tokens1 = tokenize( data, TOKENIZE_DATA );
  //printf("reresultt="); dumpln(tokens1);
  if (IS_ERROR( tokens1 )) RETURN( tokens1 );

  atom_t tokens2 = tokenize( tokens1, TOKENIZE_COMMANDS );
  DEUSE( tokens1 );
  if (IS_ERROR( tokens2 )) RETURN( tokens2 );

  RETURN( tokens2 );
}


//===================================================
/// \fn		atom_t rt_wordp (atom_t data)
/// \brief	implementation of WORDP
/// \param  data data to test
/// \return \c true if data is word or \c false if is not
///
/// Implements primitive predicate WORDP. Returns
/// \c true if argument is any word or \c false if it is not
//===================================================

atom_t RUNTIME rt_wordp( atom_t data )
{
  if (IS_ANY_WORD( data ) || IS_INTEGER( data ) || IS_FLOAT( data ))
  {
    RETURN( USE( false_true[1] ) );
  }
  else
  {
    char_t buff[MAX_NUMBER_WORD_LENGTH];
    int buff_len = MAX_NUMBER_WORD_LENGTH;

    if (atom_to_string( data, buff, &buff_len ))
    {
      RETURN( USE( false_true[1] ) );
    }
  }
  RETURN( USE( false_true[0] ) );
}


//===================================================
/// \fn		atom_t rt_listp (atom_t data)
/// \brief	implementation of LISTP
/// \param  data data to test
/// \return \c true if data is list or \c false if is not
///
/// Implements primitive predicate LISTP. Returns
/// \c true if argument is list or \c false if it is not
//===================================================

atom_t RUNTIME rt_listp( atom_t data )
{
  RETURN( USE( false_true[IS_LIST( data )] ) );
}


//===================================================
/// \fn		atom_t rt_numberp (atom_t data)
/// \brief	implementation of NUMBERP
/// \param  data data to test
/// \return \c true if data is number or \c false if is not
///
/// Implements primitive predicate NUMBERP. Returns
/// \c true if argument is any number or \c false if it is not
//===================================================

atom_t RUNTIME rt_numberp( atom_t data )
{
  float64_t x;

  // number atom atom
  if (IS_INTEGER( data ) || IS_FLOAT( data ))
  {
    RETURN( USE( false_true[1] ) );
  }

  // Can be converted to num
  if (atom_to_float( data, &x ))
  {
    RETURN( USE( false_true[1] ) );
  }

  // Not a number
  RETURN( USE( false_true[0] ) );
}


//===================================================
/// \fn		atom_t rt_empty (atom_t data)
/// \brief	implementation of EMPTYP
/// \param  data data to test
/// \return \c true if data is empty or \c false if is not
///
/// Implements primitive predicate \c EMPTYP. Outputs
/// \c true if argument is empty list or empty word.
/// Outputs \c false if it is not.
//===================================================

atom_t RUNTIME rt_empty( atom_t data )
{
  if (IS_LIST( data ))
  {
    RETURN( USE( false_true[IS_EMPTY( data )] ) );
  }

  // Empty word
  if (IS_ANY_WORD( data ) && LENGTH( data ) == 0)
  {
    RETURN( USE( false_true[1] ) );
  }

  // Not a list or empty word
  RETURN( USE( false_true[0] ) );
}


//===================================================
/// \fn		atom_t rt_memberp (atom_t cip, atom_t data, atom_t elem)
/// \brief	implementation of MEMBERP
/// \param  cip	 value of CASEIGNOREDP
/// \param  data Data where will search
/// \param  elem Element which will be searched
/// \return \c true if \c elem is in \c data or \c false if is not
///
/// Implements primitive predicate MEMBERP.
/// If \c data is list outputs \c true if \c data contains
/// \c elem as an element and \c false if not
/// If \c data is a word outputs \c true if \c elem
/// is char that is contained in the word \c data
//===================================================

atom_t RUNTIME rt_memberp( atom_t cip, atom_t data, atom_t elem )
{

  if (IS_LIST( data ))
  { // Test all elements of the list for equality with elem
    while (!IS_EMPTY( data ))
    {
      if (atom_equal( cip, CAR( data ), elem ))
      RETURN( USE( false_true[1] ) );
      data = CDR( data );
    }
    RETURN( USE( false_true[0] ) );
  }
  else
  {
    char_t buff[MAX_NUMBER_WORD_LENGTH];
    chars_t comp_buffer;
    int buff_len = MAX_NUMBER_WORD_LENGTH; // We start with 3 symbols. Actually need 1, but test if there is more
    char_t elem_ch;

    // If data is not a list, elem must be one character word
    if (IS_ANY_WORD( elem ) && LENGTH( elem ) != 1)
    {
      RETURN( USE( false_true[0] ) );
    }
    if (!IS_ANY_WORD( elem ) && (!atom_to_string( elem, buff, &buff_len ) || buff_len != 1))
    {
      RETURN( USE( false_true[0] ) );
    }

    if (buff_len == 1) // elem is one char word -> Store it
    {
      elem_ch = DEBAR( buff[0] );
    }
    else
    {
      elem_ch = DEBAR( STRING( elem )[0] );
    }

    if (IS_ANY_WORD( data ))
    {
      comp_buffer = STRING( data );
      buff_len = LENGTH( data );
    }
    else
    {
      buff_len = MAX_NUMBER_WORD_LENGTH;
      if (!atom_to_string( data, buff, &buff_len ) || buff_len == 0)
      {
        RETURN( USE( false_true[0] ) ); // data is not a word or empty
      }
      comp_buffer = buff;
    }
    while (buff_len--)
    {
      if (DEBAR( comp_buffer[buff_len] ) == elem_ch)
      {
        RETURN( USE( false_true[1] ) );
      }
    }
    RETURN( USE( false_true[0] ) );
  }
}


//===================================================
/// \fn		atom_t rt_word (int data)
/// \brief	implementation of constructor primitive WORD
/// \param  data 	number of inputs of WORD
/// \return New word concatenation of arguments
///
/// Implements the primitive \c WORD. The only input
/// contains the number of inputs which are pulled
/// out from the stack.
/// Returns new word whish is concatenation of all arguments.
/// All arguments must be words.
//===================================================

atom_t RUNTIME rt_word( int data )
{
  atom_t ARGUMENT;
  atom_t res;

  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;
  int total_length = 0, real_length = 0;

  // First calculate estimate length of result
  for (EACH_ARGUMENT)
  {
    //printf("=======arg="); dumpln(ARGUMENT);
    if (IS_ERROR( ARGUMENT ))
    {
      RETURN( USE(ARGUMENT) );
    }

    if (IS_ANY_WORD( ARGUMENT ))
    {
      total_length += LENGTH( ARGUMENT );
    }
    else
    {
      // Assume data can be translated to string.
      total_length += MAX_NUMBER_WORD_LENGTH;
    }
  }

  // Allocate memory
  res = create_word( total_length );

  // And copy elements one by one
  for (EACH_ARGUMENT)
  {
    if (IS_ANY_WORD( ARGUMENT ))
    {
      STRNCPY( STRING( res ) + real_length, STRING( ARGUMENT ), LENGTH( ARGUMENT ) );
      real_length += LENGTH( ARGUMENT );
    }
    else
    {
      buff_len = MAX_NUMBER_WORD_LENGTH;
      if (!atom_to_string( ARGUMENT, buff, &buff_len ))
      {
        DEUSE( res );
        RETURN( new_error( ERROR_NOT_A_WORD, ARGUMENT ) );
      }

      STRNCPY( STRING( res ) + real_length, buff, buff_len );
      real_length += buff_len;
    }
  }

  // If allocated more memory than needed reallocate to free unused
  if (real_length != total_length)
  {
    STRING( res ) = REALLOC( STRING( res ), (real_length + 1) * sizeof (char_t) );
    IDLENGTH( res ) = WORD_ID | (real_length << 8);
  }

  STRING( res )[real_length] = NULL_CHAR;
  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_list (int data)
/// \brief	implementation of constructor primitive LIST
/// \param  data	number of inputs of LIST
/// \return New list containing all arguments
///
/// Implements the primitive \c LIST. The only input
/// contains the number of inputs which are pulled
/// out from the stack.
/// Returns new list with elements all arguments passed
/// to the function.
//===================================================

atom_t RUNTIME rt_list( int data )
{
  atom_t ARGUMENT;
  atom_t res, iter;

  iter = res = empty_list;

  for (EACH_ARGUMENT)
  {
    if (IS_ERROR( ARGUMENT ))
    {
      RETURN( USE(ARGUMENT) );
    }
  }

  for (EACH_ARGUMENT)
  {
    append( USE( ARGUMENT ), &res, &iter );
  }

  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_sent (int data)
/// \brief	implementation of constructor primitive SE
/// \param  data	number of inputs of SE
/// \return New list containing all arguments
///
/// Implements the primitive \c SE. The only input
/// contains the number of inputs which are pulled
/// out from the stack.
/// Returns new list with elements all arguments passed
/// to the function if they are not lists.
/// If an argument is list its members are added to result
//===================================================

atom_t RUNTIME rt_sent( int data )
{
  atom_t ARGUMENT;
  atom_t res, iter;

  iter = res = empty_list;
  for (EACH_ARGUMENT)
  {
    if (IS_ERROR( ARGUMENT ))
    {
      RETURN( USE(ARGUMENT) );
    }
  }

  for (EACH_ARGUMENT)
  {
    if (IS_LIST( ARGUMENT ))
    {
      atom_t curr = ARGUMENT;

      while (IS_NOT_EMPTY( curr ))
      {
        test_elem_and_destroy_if_error( curr, res );
        append( USE( CAR( curr ) ), &res, &iter );
        curr = CDR( curr );
      }
    }
    else
    {
      append( USE( ARGUMENT ), &res, &iter );
    }
  }

  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_fput (atom_t data2, atom_t data1)
/// \brief	implementation of constructor primitive FPUT
/// \param  data1   Element to add
/// \param  data2   List or word
/// \return New list or word containing data2 with data1 at start
///
/// Implements the primitive \c FPUT.
/// Returns \c data2 with \c data1 inserted at first position.
/// If data2 is a word, \c data1 must be one char word.
//===================================================

atom_t RUNTIME rt_fput( atom_t data2, atom_t data1 )
{
  if (IS_ERROR( data1 )) RETURN( USE(data1) );
  if (IS_ERROR( data2 )) RETURN( USE(data2) );

  if (IS_LIST( data2 ))
  {
    RETURN( new_list( USE( data1 ), USE( data2 ) ) );
  }
  else
  {
    char_t buff2[MAX_NUMBER_WORD_LENGTH];
    int buff_len2 = MAX_NUMBER_WORD_LENGTH;

    char_t buff1[MAX_NUMBER_WORD_LENGTH];
    int buff_len1 = MAX_NUMBER_WORD_LENGTH;

    atom_t res;

    /* check parametr correctness */
    if (!IS_ANY_WORD( data2 ) && !atom_to_string( data2, buff2, &buff_len2 ))
    {
      RETURN( new_error( ERROR_NOT_A_WORD, data2 ) );
    }

    if ((IS_ANY_WORD( data1 ) && LENGTH( data1 ) != 1) ||
        (!atom_to_string( data1, buff1, &buff_len1 ) || buff_len1 != 1))
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data1 ) );
    }

    if (IS_ANY_WORD( data2 ))
    {
      res = create_word( 1 + LENGTH( data2 ) );
      STRING( res )[0] = IS_ANY_WORD( data1 ) ? STRING( data1 )[0] : buff1[0];
      STRNCPY( STRING( res ) + 1, STRING( data2 ), LENGTH( data2 ) );
    }
    else
    {
      res = create_word( 1 + buff_len2 );
      STRING( res )[0] = IS_ANY_WORD( data1 ) ? STRING( data1 )[0] : buff1[0];
      STRNCPY( STRING( res ) + 1, buff2, buff_len2 );
    }

    STRING( res )[LENGTH( res )] = NULL_CHAR;
    RETURN( res );
  }
}


//===================================================
/// \fn		atom_t rt_lput (atom_t data2, atom_t data1)
/// \brief	implementation of constructor primitive LPUT
/// \param  data1   Element to add
/// \param  data2   List or word
/// \return New list or word containing data2 with data1 at end
///
/// Implements the primitive \c LPUT.
/// Returns \c data2 with \c data1 appended to last position.
/// If data2 is a word, \c data1 must be one char word.
//===================================================

atom_t RUNTIME rt_lput( atom_t data2, atom_t data1 )
{
  if (IS_ERROR( data1 )) RETURN( USE(data1) );
  if (IS_ERROR( data2 )) RETURN( USE(data2) );

  if (IS_LIST( data2 ))
  {
    RETURN( copy_append( data2, data1 ) );
  }
  else
  {
    char_t buff2[MAX_NUMBER_WORD_LENGTH];
    int buff_len2 = MAX_NUMBER_WORD_LENGTH;

    char_t buff1[MAX_NUMBER_WORD_LENGTH];
    int buff_len1 = MAX_NUMBER_WORD_LENGTH;

    atom_t res;

    /* check parametr correctness */
    if (!IS_ANY_WORD( data2 ) && !atom_to_string( data2, buff2, &buff_len2 ))
    {
      RETURN( new_error( ERROR_NOT_A_WORD, data2 ) );
    }

    if ((IS_ANY_WORD( data1 ) && LENGTH( data1 ) != 1) ||
        (!atom_to_string( data1, buff1, &buff_len1 ) || buff_len1 != 1))
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data1 ) );
    }

    if (IS_ANY_WORD( data2 ))
    {
      res = create_word( 1 + LENGTH( data2 ) );
      STRNCPY( STRING( res ), STRING( data2 ), LENGTH( data2 ) );
      STRING( res )[LENGTH( data2 )] = IS_ANY_WORD( data1 ) ? STRING( data1 )[0] : buff1[0];
    }
    else
    {
      res = create_word( 1 + buff_len2 );
      STRNCPY( STRING( res ), buff2, buff_len2 );
      STRING( res )[buff_len2] = IS_ANY_WORD( data1 ) ? STRING( data1 )[0] : buff1[0];
    }

    STRING( res )[LENGTH( res )] = NULL_CHAR;
    RETURN( res );
  }
}


//===================================================
/// \fn		atom_t rt_count (atom_t data)
/// \brief	implementation of querie primitive COUNT
/// \param  data Atom to inspect
/// \return Number of elements in data
///
/// Implements the primitive \c COUNT.
/// Returns number of chars in \c data if data is word
/// or number of elements in \c data if data is list
//===================================================

atom_t RUNTIME rt_count( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }
  if (IS_ANY_WORD( data ))
  {
    RETURN( new_integer( LENGTH( data ) ) );
  }
  if (IS_LIST( data ))
  {
    RETURN( new_integer( list_length( data ) ) );
  }
  if (atom_to_string( data, buff, &buff_len ))
  {
    RETURN( new_integer( buff_len ) );
  }
  RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
}


//===================================================
/// \fn		atom_t rt_char (atom_t data)
/// \brief	implementation of querie primitive CHAR
/// \param  data ASCII code
/// \return Char coresponding to data
///
/// Implements the primitive \c CHAR.
/// Returns one char word containing symbol with given ASCII code
//===================================================

atom_t RUNTIME rt_char( atom_t data )
{
  int64_t x;
  char_t str[2];
  GET_INT( data, x );

#ifdef UNICODE_CHARS
  if (!OPTION_TRADITIONAL)
  {
    if (x < 0 || x > ((1 << 16) - 1))
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
    }
    str[0] = (char_t) x;
    str[1] = NULL_CHAR;
    RETURN( new_word( str, 1 ) );
  }
#endif

  if (x < 0 || x > 255)
  {
    RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
  }

  str[0] = DEBAR( (char_t) x );
  str[1] = NULL_CHAR;
  RETURN( new_word( str, 1 ) );
}



//===================================================
/// \fn		atom_t rt_ascii (atom_t data)
/// \brief	implementation of querie primitive ASCII
/// \param  data one char word
/// \return ASCII code of data
///
/// Implements the primitive \c ASCII.
/// Returns ASCII code of given char
//===================================================

atom_t RUNTIME rt_ascii( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) != 1)
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
    }
#ifdef UNICODE_CHARS
    if (!OPTION_TRADITIONAL)
    {
      RETURN( new_integer( (ushort_t) DEBAR( STRING( data )[0] ) ) );
    }
#endif
    RETURN( new_integer( (byte_t) DEBAR( STRING( data )[0] ) ) );
  }

  if (IS_LIST( data ))
  {
    RETURN( new_error( ERROR_NOT_A_WORD, data ) );
  }

  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len == 1)
    {
#ifdef UNICODE_CHARS
      if (!OPTION_TRADITIONAL)
      {
        RETURN( new_integer( (ushort_t) DEBAR( buff[0] ) ) );
      }
#endif
      RETURN( new_integer( (byte_t) DEBAR( buff[0] ) ) );
    }

    // Word but too long
    RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
  }
  RETURN( new_error( ERROR_NOT_A_WORD, data ) );
}



//===================================================
/// \fn		atom_t rt_lower (atom_t data)
/// \brief	implementation of querie primitive LOWERCASE
/// \param  data word
/// \return New word all in lowercase letters
///
/// Implements the primitive \c LOWERCASE.
/// Returns word produced from \c data with replacing all
/// uppercase letters with lowercase letters
//===================================================

atom_t RUNTIME rt_lower( atom_t data )
{
  atom_t res;
  int i;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_ANY_WORD( data ))
  {
    res = new_word( STRING( data ), LENGTH( data ) );
  }
  else
  {
    char_t buff[MAX_NUMBER_WORD_LENGTH];
    int buff_len = MAX_NUMBER_WORD_LENGTH;

    if (IS_LIST( data ) || !atom_to_string( data, buff, &buff_len ))
    {
      RETURN( new_error( ERROR_NOT_A_WORD, data ) );
    }
    // in case of boolean of scientific float mey be necessery to convert
    res = new_word( buff, buff_len );
  }

  // proces word and make convertion
  for (i = 0; i < LENGTH( res ); ++i)
  {
    STRING( res )[i] = TOLOWER( DEBAR( STRING( res )[i] ) );
  }

  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_upper (atom_t data)
/// \brief	implementation of querie primitive UPPERCASE
/// \param  data word
/// \return New word all in lowercase letters
///
/// Implements the primitive \c UPPERCASE.
/// Returns word produced from \c data with replacing all
/// lowercase letters with uppercase letters
//===================================================

atom_t RUNTIME rt_upper( atom_t data )
{
  atom_t res;
  int i;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_ANY_WORD( data ))
  {
    res = new_word( STRING( data ), LENGTH( data ) );
  }
  else
  {
    char_t buff[MAX_NUMBER_WORD_LENGTH];
    int buff_len = MAX_NUMBER_WORD_LENGTH;

    if (IS_LIST( data ) || !atom_to_string( data, buff, &buff_len ))
    {
      RETURN( new_error( ERROR_NOT_A_WORD, data ) );
    }
    // in case of boolean of scientific float mey be necessery to convert
    res = new_word( buff, buff_len );
  }

  // proces word and make convertion
  for (i = 0; i < LENGTH( res ); ++i)
  {
    STRING( res )[i] = TOUPPER( DEBAR( STRING( res )[i] ) );
  }

  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_member (atom_t cip, atom_t elem, atom_t data)
/// \brief	implementation of MEMBER
/// \param  cip	 value of CASEIGNOREDP
/// \param  data Data where will search
/// \param  elem Element which will be searched
/// \return Elements of \c data form first occurence of \c elem to the end
///
/// Implements primitive predicate \c MEMBER.
/// If \c data is list outputs list containing all elements of
/// \c data after first occurence of \c elem or empty list
/// if \c elem is not a member of \c data
/// If \c data is a word outputs subword of \c data
/// starting from first occurence of \c elem to the end
/// or empty word if \c elem is not a member of \c data
//===================================================

atom_t RUNTIME rt_member( atom_t cip, atom_t data, atom_t elem )
{
  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_LIST( data ))
  { // Test all elements of the list for equality with elem
    while (!IS_EMPTY( data ))
    {
      if (atom_equal( cip, CAR( data ), elem ))
      RETURN( USE( data ) );
      data = CDR( data );
    }
    RETURN( USE( empty_list ) );
  }
  else
  {
    char_t buff[MAX_NUMBER_WORD_LENGTH];
    chars_t comp_buffer;
    int buff_len = MAX_NUMBER_WORD_LENGTH; // We start with 3 symbols. Actually need 1, but test if there is more
    int pos;
    char_t elem_ch;

    // If data is not a list, elem must be one character word
    if (IS_ANY_WORD( elem ) && LENGTH( elem ) != 1)
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, elem ) );
    }
    if (!IS_ANY_WORD( elem ) && (!atom_to_string( elem, buff, &buff_len ) || buff_len != 1))
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, elem ) );
    }

    if (buff_len == 1) // elem is one char word -> Store it
    {
      elem_ch = DEBAR( buff[0] );
    }
    else
    {
      elem_ch = DEBAR( STRING( elem )[0] );
    }

    if (IS_ANY_WORD( data ))
    {
      comp_buffer = STRING( data );
      buff_len = LENGTH( data );
    }
    else
    {
      buff_len = MAX_NUMBER_WORD_LENGTH;
      if (!atom_to_string( data, buff, &buff_len ) || buff_len == 0)
      {
        RETURN( new_error( ERROR_NOT_A_WORD, data ) );
      }
      comp_buffer = buff;
    }
    pos = 0;
    while (pos < buff_len)
    {
      if (DEBAR( comp_buffer[pos] ) == elem_ch)
      {
        if (IS_ANY_WORD( data ))
        {
          RETURN( new_subword( data, comp_buffer + pos, buff_len - pos ) );
        }
        else
        {
          RETURN( new_word( comp_buffer + pos, buff_len - pos ) );
        }
      }
      ++pos;
    }
    RETURN( new_word( buff, 0 ) );
  }
}


//===================================================
/// \fn		atom_t rt_iseq (atom_t to, atom_t from)
/// \brief	implementation of ISEQ
/// \param  from First element of the sequence
/// \param  to Last element of the sequence
/// \return List form all elements between \c from and \c to
///
/// Implements primitive predicate \c ISEQ.
/// Returns a list containing all numbers between \c from
/// and \c to. Parameters must be integers.
//===================================================

atom_t RUNTIME rt_iseq( atom_t to, atom_t from )
{
  int64_t first;
  int64_t last;
  atom_t list_start, list_end;

  GET_INT( from, first );
  GET_INT( to, last );

  list_start = list_end = empty_list;
  if (first > last)
  {
    for (; first >= last; --first)
    append( new_integer( first ), &list_start, &list_end );
  }
  else
  {
    for (; first <= last; ++first)
    append( new_integer( first ), &list_start, &list_end );
  }
  RETURN( list_start );
}


//===================================================
/// \fn		atom_t rt_rseq (atom_t count, atom_t to, atom_t from)
/// \brief	implementation of RSEQ
/// \param  from First element of the sequence
/// \param  to Last element of the sequence
/// \param  count Number of elements in the sequence
/// \return List form \c count elements between \c from and \c to
///
/// Implements primitive predicate \c RSEQ.
/// Returns a list containing \c count numbers between \c from
/// and \c to. All numbers are equally spaced rational.
//===================================================

atom_t RUNTIME rt_rseq( atom_t count, atom_t to, atom_t from )
{
  float64_t first;
  float64_t last;
  float64_t step;
  int64_t cnt;

  atom_t list_start, list_end;

  GET_FLOAT( from, first );
  GET_FLOAT( to, last );
  GET_INT( count, cnt );

  if (cnt <= 0 || (cnt == 1 && first != last))
  {
    RETURN( new_error( ERROR_INCOMPATIBLE_DATA, count ) );
  }

  list_start = list_end = empty_list;
  if (first > last)
  {
    for (step = cnt > 1 ? (first - last) / (cnt - 1) : 0; cnt > 0; --cnt, first -= step)
    {
      append( new_float( first ), &list_start, &list_end );
    }
  }
  else
  {
    for (step = cnt > 1 ? (last - first) / (cnt - 1) : 0; cnt > 0; --cnt, first += step)
    {
      append( new_float( first ), &list_start, &list_end );
    }
  }
  RETURN( list_start );
}


//===================================================
/// \fn		rt_random (int count, atom_t data2, atom_t data1)
/// \brief	implementation of aritmetic primitive RANDOM
/// \param count	number of arguments (1 or 2)
/// \param data1	first argument
/// \param data2	second argument
/// \return Random number.
///
/// Implements the primitive \c RANDOM
/// If called with one arg outputs a nonnegative integer
/// less than its input \c first. If input is a list
/// outputs randomly selected element from it
/// If called with two arguments outputs a nonnegative
/// random integer greater than or equal to \c data1,
/// and less than or equal to \c data2
//===================================================

atom_t RUNTIME rt_random( int count, atom_t data2, atom_t data1 )
{
  int rand_num = rand( );
  if (count == 1)
  {
    if (!OPTION_TRADITIONAL && IS_LIST( data2 ))
    {
      int len;
      if (IS_EMPTY( data2 ))
      {
        RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data2 ) );
      }

      len = list_length( data2 );
      RETURN( USE( get_at_list( data2, rand_num % len ) ) );
    }
    else
    {
      int64_t num;
      GET_INT( data2, num );

      if (num <= 0)
      {
        RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data2 ) );
      }
      RETURN( new_integer( rand_num % num ) );
    }
  }
  else if (count == 2)
  {
    int64_t start;
    int64_t end;

    GET_INT( data1, start );
    GET_INT( data2, end );

    if (start > end || start < 0)
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data1 ) );
    }
    RETURN( new_integer( rand_num % (end - start + 1) + start ) );
  }
  else
  {
    RETURN( new_error( ERROR_CROWDED_EXPRESSION, data1 ) );
  }
}


//===================================================
/// \fn		atom_t rt_rerandom(int count, atom_t seed)
/// \brief	implementation of primitive RERANDOM
/// \param  count number of arguments (0 or 1)
/// \param  seed number to set random seed
///
/// Implements the primitive \c RERANDOM.
/// Makes the results of \c RANDOM reproducible.
//===================================================

atom_t RUNTIME rt_rerandom( int count, atom_t seed )
{
  int64_t rseed;
  if (count == 1)
  {
    GET_INT( seed, rseed );
  }
  else if (count == 0)
  {
    rseed = 0;
  }
  else
  {
    RETURN( new_error( ERROR_CROWDED_EXPRESSION, seed ) );
  }

  srand( (uint_t) rseed );
  RETURN( unbound );
}


//===================================================
/// \fn		atom_t rt_show (atom_t pdl, atom_t pwl, atom_t fpp, int data)
/// \brief	implementation of primitive SHOW
/// \param pdl	value of PRINTDEPTHLIMIT
/// \param pwl	value of PRINTWIDTHLIMIT
/// \param fpp	value of FULLPRINTP
/// \param data	number of inputs of SHOW
///
/// Implements the primitive \c SHOW. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Prints input like \c PRINT
/// except that if an input is a list it is printed
/// inside square brackets
//===================================================

atom_t RUNTIME rt_show( atom_t pdl, atom_t pwl, atom_t fpp, int data )
{
  // check print-related variabples
  atom_to_integer( pdl, &print_depth_limit );
  atom_to_integer( pwl, &print_width_limit );
  atom_to_boolean( fpp, &full_print_p );

  // do the actual print
  atom_t ARGUMENT;
  for (EACH_ARGUMENT)
  {
    if (IS_ERROR( ARGUMENT ))
    {
      RETURN( USE(ARGUMENT) );
    }

    // If list adds square brackets
    if (IS_LIST( ARGUMENT ))
    {
      outter( TEXT( "[" ), 1 );
      dump( ARGUMENT );
      outter( TEXT( "]" ), 1 );
    }
    else
    {
      dump( ARGUMENT );
    }
    outter( TEXT( " " ), 1 );
  }

  outter( TEXT( "\n" ), 1 );
  RETURN( unbound );
}


//===================================================
/// \fn		atom_t rt_type (atom_t pdl, atom_t pwl, atom_t fpp, int data)
/// \brief	implementation of primitive TYPE
/// \param pdl	value of PRINTDEPTHLIMIT
/// \param pwl	value of PRINTWIDTHLIMIT
/// \param fpp	value of FULLPRINTP
/// \param data	number of inputs of TYPE
///
/// Implements the primitive \c TYPE. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Prints input like \c PRINT
/// except that no new line is printed at the end
/// and no spaces is printed between inputs.
//===================================================

atom_t RUNTIME rt_type( atom_t pdl, atom_t pwl, atom_t fpp, int data )
{
  // check print-related variabples
  atom_to_integer( pdl, &print_depth_limit );
  atom_to_integer( pwl, &print_width_limit );
  atom_to_boolean( fpp, &full_print_p );

  // do the actual print
  atom_t ARGUMENT;
  for (EACH_ARGUMENT)
  {
    if (IS_ERROR( ARGUMENT ))
    {
      RETURN( USE(ARGUMENT) );
    }
    dump( ARGUMENT );
  }
  RETURN( unbound );
}


//===================================================
/// \fn		atom_t rt_form (atom_t precision, atom_t width, atom_t num)
/// \brief	implementation of primitive FORM
/// \param num number to be converted
/// \param width width in wich number will be printed
/// \param precision precision with wich number will be printed
/// \return Word representing given num.
///
/// Implements the primitive \c FORM. Outputs string
/// representation of \c num printed with \c precision
/// digits after decimal point and in at least \c width chars
/// If \c width is more than number length some spaces are
/// inserted in front of the string to fill \c width chars
//===================================================

atom_t RUNTIME rt_form( atom_t precision, atom_t width, atom_t num )
{
  int64_t num_prec;
  int64_t num_width;
  float64_t number;

  chars_t format;
  char_t buff[MAX_WORD_LENGTH];
  int buff_len = MAX_WORD_LENGTH;

  GET_FLOAT( num, number );
  GET_INT( precision, num_prec );
  if (num_prec >= 0)
  {
    GET_INT( width, num_width );
    format = TEXT( "%*.*lf" );
    SPRINT( buff, buff_len, format, (uint_t) num_width, (uint_t) num_prec, number );
  }
  else
  {
    if (IS_ANY_WORD( width ))
    {
      format = STRING( width );
      SPRINTF( buff, buff_len, format, number );
    }
    else
    {
      RETURN( new_error( ERROR_NOT_A_WORD, width ) );
    }
  }

  buff_len = STRLEN( buff );
  RETURN( new_word( buff, buff_len ) );
}


//===================================================
/// \fn		char_t get_format(chars_t string)
/// \brief	extracts format specifier
/// \param  string  a format string
/// \return Format char.
///
/// Returns format char or 0 if no format is passed
/// or more than one format specifiers occur.
//===================================================

char_t RUNTIME get_format( chars_t string )
{
  chars_t elem = string;
  char_t type = 0;
  while (elem)
  {
    elem = STRCHR( elem, TEXT( '%' ) );
    if (elem)
    {
      ++elem;
      if (elem[0] == TEXT( '%' ))
      {
        elem += 1;
      }
      else
      {
        while (ISDIGIT( *elem ) || *elem == TEXT( '.' ))
        {
          ++elem;
        }
        while (*elem == TEXT( 'l' ) || *elem == TEXT( 'u' ) || *elem == TEXT( 'h' ))
        ++elem;
      }
      if (type) return 0;
      type = *elem;
    }
  }
  return type;
}


//===================================================
/// \fn		atom_t rt_format (atom_t format, atom_t data)
/// \brief	implementation of primitive FORMAT
/// \param data data to be formated
/// \param format formating string
/// \return Word representing given data according to format string.
///
/// Implements the primitive \c FORMAT. Outputs string
/// representation of \c data according to \c format
/// formating string using printf sintax
//===================================================

atom_t RUNTIME rt_format( atom_t format, atom_t data )
{
  if (!IS_ANY_WORD( format ))
  {
    RETURN( new_error( ERROR_NOT_A_WORD, format ) );
  }
  else
  {
    char_t buff[MAX_WORD_LENGTH];
    int buff_len = MAX_WORD_LENGTH;
    char_t end = STRING( format )[LENGTH( format )];

    STRING( format )[LENGTH( format )] = NULL_CHAR;
    char_t format_chr = get_format( STRING( format ) );

    switch (format_chr)
    {
      // Integer number
    case TEXT( 'd' ) : case TEXT( 'i' ) : case TEXT( 'u' ) :

    case TEXT( 'x' ) : case TEXT( 'X' ) : case TEXT( 'o' ) :
      {
        int64_t i_num;
        STRING( format )[LENGTH( format )] = end;
        GET_INT( data, i_num );
        STRING( format )[LENGTH( format )] = NULL_CHAR;
        SPRINTF( buff, buff_len, STRING( format ), i_num );
        break;
      }
      // Floating point number
    case TEXT( 'f' ) :
    case TEXT( 'e' ) : case TEXT( 'E' ) :
    case TEXT( 'g' ) : case TEXT( 'G' ) :

    case TEXT( 'a' ) : case TEXT( 'A' ) :
      {
        float64_t fl_num;
        STRING( format )[LENGTH( format )] = end;
        GET_FLOAT( data, fl_num );
        STRING( format )[LENGTH( format )] = NULL_CHAR;
        SPRINTF( buff, buff_len, STRING( format ), fl_num );
        break;
      }

      // char

    case TEXT( 'c' ) : case TEXT( 'C' ) :
      {
        char_t ch;
        if (IS_ANY_WORD( data ))
        {
          if (LENGTH( data ) == 0)
          {
            STRING( format )[LENGTH( format )] = end;
            RETURN( new_error( ERROR_MISSING_VALUE, data ) );
          }
          ch = STRING( data )[0];
        }
        else
        if (atom_to_string( data, buff, &buff_len ))
        {
          if (buff_len == 0)
          {
            STRING( format )[LENGTH( format )] = end;
            RETURN( new_error( ERROR_MISSING_VALUE, data ) );
          }
          ch = buff[0];
        }
        else
        {
          STRING( format )[LENGTH( format )] = end;
          RETURN( new_error( ERROR_NOT_A_WORD, data ) );
        }
        buff_len = MAX_WORD_LENGTH;
        SPRINTF( buff, buff_len, STRING( format ), ch );
        break;
      }

      // string

    case TEXT( 's' ) : case TEXT( 'S' ) :
      {
        if (IS_ANY_WORD( data ))
        {
          char_t term = STRING( data )[LENGTH( data )];
          STRING( data )[LENGTH( data )] = NULL_CHAR;
          SPRINTF( buff, buff_len, STRING( format ), STRING( data ) );
          STRING( data )[LENGTH( data )] = term;
        }
        else
        {
          char_t num_buff[MAX_NUMBER_WORD_LENGTH];
          int num_buff_len;

          if (atom_to_string( data, num_buff, &num_buff_len ))
          {
            SPRINTF( buff, buff_len, STRING( format ), num_buff );
          }
          else
          {
            STRING( format )[LENGTH( format )] = end;
            RETURN( new_error( ERROR_NOT_A_WORD, data ) );
          }
        }
        break;
      }
      // pointer

    case TEXT( 'p' ) :
      {
        SPRINTF( buff, buff_len, STRING( format ), data );
        break;
      }
    default:
      {
        STRING( format )[LENGTH( format )] = end;
        RETURN( new_error( ERROR_INCOMPATIBLE_DATA, format ) );
      }
    }

    STRING( format )[LENGTH( format )] = end;
    buff_len = STRLEN( buff );
    RETURN( new_word( buff, buff_len ) );
  }
}



//===================================================
/// \fn		atom_t rt_formattime (atom_t format, atom_t data)
/// \brief	implementation of primitive FORMATTIME
/// \param data data to be formated
/// \param format formating string
/// \return Word representing given data according to format string.
///
/// Implements the primitive \c FORMATTIME. Outputs
/// string representation of \c data according to
/// date/time \c format formating string using
/// strftime sintax.
//===================================================

atom_t RUNTIME rt_formattime( atom_t format, atom_t data )
{
  atom_t res = unbound;
  
  CHECK_PARAM(format);
  CHECK_PARAM(data);
  
  if (!IS_ANY_WORD( format ))
  {
    RETURN( new_error( ERROR_NOT_A_WORD, format ) );
  }

  int64_t time64;
  if( !atom_to_int( data, &time64 ) )
  {
    RETURN( new_error( ERROR_NOT_AN_INTEGER, data ) );
  }
  time_t time = time64;

  atom_t formatz = atom_to_real_word( format );

  struct tm *presult;
  presult = gmtime( &time );
  char_t buf[MAX_WORD_LENGTH];
  int len;
  if( presult )
    len = STRFTIME( buf, MAX_WORD_LENGTH, STRING(formatz), presult ); 
  else
    len = 0;
 
  res = new_word( buf, len );
  DEUSE( formatz );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_definedp( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of DEFINED?
/// \param  data	data to test
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return \c true if \c data contains the name of a user-defined function
///
/// Implements primitive predicate DEFINED?. Returns
/// \c true if \c data contains the name of a
/// user-defined function or command.
//===================================================

atom_t RUNTIME rt_definedp( int static_link, atom_t parent, atom_t data )
{
  atom_t word = atom_to_word( data );
  if (IS_ERROR( data )) RETURN( USE(data) );

  //printf("looking for "); dump(word); printf(" in "); dumpln(NAME(parent));
  atom_t var = find_var( word, parent );
  DEUSE( word );
  RETURN( USE( false_true[ var && !IS_PRIMITIVE( var ) && !IS_VARIABLE( var ) ] ) );
}




//===================================================
/// \fn		atom_t rt_primitivep( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of PRIMITIVE?
/// \param  data	data to test
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return \c true if \c data contains the name of a primitive function
///
/// Implements primitive predicate PRIMiTIVE?. Returns
/// \c true if \c data contains the name of a
/// primitive function or command.
//===================================================

atom_t RUNTIME rt_primitivep( int static_link, atom_t parent, atom_t data )
{
  atom_t word = atom_to_word( data );
  if (IS_ERROR( data )) RETURN( USE(data) );

  atom_t var = find_var( word, parent );
  DEUSE( word );

  RETURN( USE( false_true[ var && IS_PRIMITIVE( var ) && !IS_VARIABLE( var ) ] ) );
}




//===================================================
/// \fn		atom_t rt_namep( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of NAME?
/// \param  data	data to test
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return \c true if \c data contains the name of a variable
///
/// Implements primitive predicate NAME?. Returns
/// \c true if \c data contains the name of a
/// variable.
//===================================================

atom_t RUNTIME rt_namep( int static_link, atom_t parent, atom_t data )
{
  atom_t word = atom_to_word( data );
  if (IS_ERROR( data )) RETURN( USE(data) );

  atom_t var = find_var( word, parent );
  DEUSE( word );

  if( !var ) RETURN( USE(false_true[0]) );
  if( !IS_VARIABLE(var) ) RETURN( USE(false_true[0]) );

  atom_t value;

  // is primitive variable
  if( IS_RUNTIME(var) )
  value = VALUE(var);
  else
  value = rt_var_value( static_link, parent, var );

  RETURN( USE( false_true[!IS_UNBOUND(value)] ) );
}




//===================================================
/// \fn		atom_t rt_procedurep( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of PROCEDURE?
/// \param  data	data to test
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return \c true if \c data contains the name of a function or a command
///
/// Implements primitive predicate PROCEDURE?. Returns
/// \c true if \c data contains the name of a function
/// or a command.
//===================================================

atom_t RUNTIME rt_procedurep( int static_link, atom_t parent, atom_t data )
{
  atom_t word = atom_to_word( data );
  if (IS_ERROR( data )) RETURN( USE(data) );

  atom_t var = find_var( word, parent );
  DEUSE( word );

  RETURN( USE( false_true[ var && (IS_FUNCTION( var ) || IS_COMMAND( var )) ] ) );
}




//===================================================
/// \fn		atom_t rt_var_value( int static_link, atom_t parent, atom_t var )
/// \brief	searches var's value in the local stacks
/// \param  var		the variable
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return the value of the variable
///
/// This function looks for the value of a variable,
/// somewhere in a stack frame. If the variable is
/// global do not scan the stack.
//===================================================

atom_t rt_var_value( int static_link, atom_t parent, atom_t var )
{
  if (IS_GLOBAL( var )) return ( VALUE( var ));
  int i;
  for (i = 0; i < LEVEL( parent ) - LEVEL( var ) + 1; i++)
  static_link = *(int*) ((char*) static_link + BASE_OFFSET_STATIC);

  return *(atom_t*) ((char*) static_link + OFFSET( var ));
}



//===================================================
/// \fn		void rt_set_var_value( int static_link, atom_t parent, atom_t var, atom_t value )
/// \brief		sets var's value in the local stacks
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  var		the variable
/// \param  value	the value
///
/// This function looks for a variable somewhere in
/// stack frames and sets it value. If the variable is
/// global do not scan the stack.
//===================================================

void rt_set_var_value( int static_link, atom_t parent, atom_t var, atom_t value )
{
  //printf("in rt_set_var_value\n");
  //printf("var="); dumpln(var);
  //printf("parent="); dumpln(PARENT(var));
  //printf("is primitive=%d\n",IS_PRIMITIVE(var));
  //printf("is global=%d\n",IS_GLOBAL(var));
  //printf("is runtime=%d\n",IS_RUNTIME(var));

  if (IS_RUNTIME( var ))
  {
    //printf("VALUE IS VAR ATOM\n");
    //printf("var = "); dumpln(var);
    //printf("old value="); dumpln(VALUE(var));
    //printf("new value="); dumpln(value);
    DEUSE( VALUE( var ) );
    VALUE( var ) = USE( value );
    //printf("new value="); dumpln(VALUE(var));
    return;
  }

  //printf("VALUE IN STACK\n");
  int i;
  for (i = 0; i < LEVEL( parent ) - LEVEL( var ) + 1; i++)
  static_link = *(int*) ((char*) static_link + BASE_OFFSET_STATIC);

  atom_t* varptr = (atom_t*) ((char*) static_link + OFFSET( var ));
  DEUSE( *varptr );
  *varptr = USE( value );
}



//===================================================
/// \fn		atom_t rt_use_var( atom_t source, atom_t value )
/// \brief	checks the value of a variable
/// \param source       the source code
/// \param  value	the variable's value
/// \return the value of the variable
///
/// This function increases the reference count of
/// a variable's value and check whether it is
/// acceptible for a value -- i.e. it is neither
/// error or unbound.
//===================================================
#define rt_use_var __attribute__((used,noinline,regparm(0),stdcall)) rt_use_var

atom_t rt_use_var( atom_t source, atom_t value )
#undef rt_use_var
{
  //printf("rt_use_var="); dumpln(value);
  //printf("    source="); dumpln(source);
  if (IS_UNBOUND( value ))
  {
    RETURN( USE( new_error( ERROR_VAR_HAS_NO_VALUE, source ) ) );
  }
  //printf("------------\n");
  RETURN( USE( value ) );
}



//===================================================
/// \fn		atom_t rt_thing( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of THING
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  data	variable name
/// \return the value of variable with given name
///
/// Implements primitive THING. Returns the value of
/// the variable which name is the value of \c data.
/// If there is no variable, then return error atom.
//===================================================

atom_t RUNTIME rt_thing( int static_link, atom_t parent, atom_t data )
{
  if (IS_ERROR( data )) RETURN( USE(data) );

  atom_t word = atom_to_word( data );
  if (IS_ERROR( word )) RETURN( word );

  atom_t var = find_runtime_var( word, static_link );
  DEUSE( word );

  // not found
  if (!var)
  RETURN( new_error( ERROR_UNKNOWN_VAR, data ) );

  // not a variable (i.e. it is a function or a procedure)
  if (!IS_VARIABLE( var ))
  RETURN( new_error( ERROR_NOT_A_VAR, data ) );

  // is primitive variable
  if (IS_RUNTIME( var ))
  RETURN( USE( VALUE( var ) ) );

  // user-defined variable
  RETURN( USE( rt_var_value( static_link, parent, var ) ) );
}


//===================================================
/// \fn		atom_t rt_reference( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of :
/// \param  data	variable name
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return the value of variable with given name
///
/// Implements the : syntax. Returns the value of
/// the variable which name is following the colons.
/// If there is no variable, then return error atom.
//===================================================

atom_t RUNTIME rt_reference( int static_link, atom_t parent, atom_t data )
{
  //printf("rt_reference = "); dumpln(data);

  if (IS_ERROR( data )) RETURN( USE( data ) );

  atom_t word = atom_to_word( data );
  if (IS_ERROR( word )) RETURN( word );

  atom_t real_name = new_subword( word, STRING( word ) + 1, LENGTH( word ) - 1 );
  //printf("~~~~~~searching for var=");dumpln(real_name);
  atom_t var = find_runtime_var( real_name, static_link );
  if (IS_UNBOUND( var )) var = find_var( real_name, parent );
  DEUSE( real_name );
  DEUSE( word );

  //if( var )
  //{
  //printf("~~~~~~3~~~~~~ found var=(%x)",(int)var); dumpln(var);
  //printf("        its parent is="); dumpln(PARENT(var));
  //}
  //else
  //{
  //printf("~~~~~~3~~~~~~ not found\n");
  //}

  // not found
  if (!var)
  RETURN( new_error( ERROR_UNKNOWN_VAR, data ) );

  // not a variable (i.e. it is a function or a procedure)
  if (!IS_VARIABLE( var ))
  RETURN( new_error( ERROR_NOT_A_VAR, data ) );

  // is primitive variable
  if (IS_RUNTIME( var ))
  {
    //printf("var="); dumpln(var);
    RETURN( USE( VALUE( var ) ) );
  }

  //printf("value="); dumpln(VALUE(var));

  // user-defined variable
  RETURN( USE( rt_var_value( static_link, parent, var ) ) );
}


//===================================================
/// \fn		atom_t rt_check_inputs( int stack_frame )
/// \brief	check inputs params
/// \param  stack_frame the current stack frame
/// \return unbound or error atoms
///
/// Checks all external parameters (inputs) whether
/// any of them is error atom. If yes, then return
/// this error. Otherwise, return unbound atom.
//===================================================

#define rt_check_inputs __attribute__((used,noinline,regparm(0),stdcall)) rt_check_inputs

atom_t rt_check_inputs( int stack_frame )
#undef rt_check_inputs
{
  atom_t res = unbound;
  int inputs = *(int*)(stack_frame+BASE_OFFSET_PARAMSCOUNT);
  //printf("inputs=%d\n",inputs);
  for( ; inputs>0; inputs-- )
  {
    atom_t data = *(atom_t*)(stack_frame+BASE_OFFSET_PARAMSCOUNT+4*inputs);
    if( IS_ERROR(data) )
      {
        data = USE(data);
        if( res==unbound ) res = data;
      }
  }
  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_bye (void)
/// \brief	implementation of command BYE
///
/// Implement command \c BYE Terminate the program
//===================================================

atom_t RUNTIME rt_bye( void )
{
  //exit(0);
  RETURN( new_error( EXIT_BY_BYE, empty_list ) );
}


//===================================================
/// \fn		atom_t rt_wait (atom_t time)
/// \brief	implementation of command WAIT
/// \param  time Time to wait in 60-ths of second
///
/// Implement command \c WAIT
/// Suspend the program execution for \c time 60-ths of second
/// and flushes the output buffer.
//===================================================

atom_t RUNTIME rt_wait( atom_t time )
{
  int64_t wait_time;
  GET_INT( time, wait_time );

  fflush( stdout ); // Flushing output buffer

  if (wait_time > 0)
  {
#ifdef WINDOWS
    wait_time = (wait_time * 1000) / 60; // convert 60-ths of second to milliseconds
    Sleep( wait_time );
#else
    wait_time = (wait_time * 1000 * 1000) / 60; // convert 60-ths of second to milliseconds
    usleep( wait_time );
#endif
  }
  RETURN( unbound );
}


//===================================================
/// \fn		atom_t rt_ashift(atom_t bits, atom_t num)
/// \brief	implementation of primitive command ASHIFT
/// \param bits	Number of bits to shift
/// \param num  Number to be shifted
/// \return	result of num << bits.
///
/// Implement logical operation \c ASHIFT.
/// \c num is shifted \c bits bits to the left.
/// if \c bits is negative the shift is to the right
/// with sign bit fill
//===================================================

atom_t RUNTIME rt_ashift( atom_t bits, atom_t num )
{
  int64_t number;
  int64_t bits_num;

  GET_INT( bits, bits_num );
  GET_INT( num, number );

  if (bits_num < 0)
  {
    RETURN( new_integer( number >> -bits_num ) );
  }
  else
  {
    RETURN( new_integer( number << bits_num ) );
  }
}


//===================================================
/// \fn		atom_t rt_lshift(atom_t bits, atom_t num)
/// \brief	implementation of primitive command LSHIFT
/// \param bits	Number of bits to shift
/// \param num  Number to be shifted
/// \return	result of num << bits.
///
/// Implement logical operation \c LSHIFT.
/// \c num is shifted \c bits bits to the left.
/// if \c bits is negative the shift is to the right
/// with zero bits fill
//===================================================

atom_t RUNTIME rt_lshift( atom_t bits, atom_t num )
{
  int64_t number;
  int64_t bits_num;

  GET_INT( bits, bits_num );
  GET_INT( num, number );

  if (bits_num < 0)
  {
    RETURN( new_integer( (int64_t) (((uint64_t) number) >> -bits_num) ) );
  }
  else
  {
    RETURN( new_integer( number << bits_num ) );
  }
}


//===================================================
/// \fn		atom_t rt_bitand(int data)
/// \brief	implementation of primitive command BITAND
/// \param  data	number of inputs of BITAND
/// \return	binary AND of all arguments
///
/// Implement binary operation \c BITAND. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Returns their binary product (AND)
//===================================================

atom_t RUNTIME rt_bitand( int data )
{
  int64_t acc = -1;
  int64_t x;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT) // Process each given argument
  {
    GET_INT( ARGUMENT, x );
    acc &= x;
  }
  RETURN( new_integer( acc ) );
}


//===================================================
/// \fn		atom_t rt_bitor(int data)
/// \brief	implementation of primitive command BITOR
/// \param  data	number of inputs of BITOR
/// \return	binary OR of all arguments
///
/// Implement binary operation \c BITOR. The only input
/// contains the number of inputs which are pulled
/// out from the stack. Returns their binary sum (OR)
//===================================================

atom_t RUNTIME rt_bitor( int data )
{
  int64_t acc = 0;
  int64_t x;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT) // Process each given argument
  {
    GET_INT( ARGUMENT, x );
    acc |= x;
  }
  RETURN( new_integer( acc ) );
}



//===================================================
/// \fn		atom_t rt_bitxor(int data)
/// \brief	implementation of primitive command BITXOR
/// \param  data	number of inputs of BITXOR
/// \return	binary XOR of all arguments
///
/// Implement binary operation \c BITXOR. The only input
/// contains the number of inputs which are pulled
/// out from the stack.
/// Returns their binary sum by modulo 2(XOR)
//===================================================

atom_t RUNTIME rt_bitxor( int data )
{
  int64_t acc = 0;
  int64_t x;
  atom_t ARGUMENT;

  for (EACH_ARGUMENT) // Process each given argument
  {
    GET_INT( ARGUMENT, x );
    acc ^= x;
  }
  RETURN( new_integer( acc ) );
}


//===================================================
/// \fn		atom_t rt_bitnot (atom_t data)
/// \brief	implementation of primitive command BITNOT
/// \param  data	integer number.
/// \return	binary NOT of the argument
///
/// Implement binary operation \c BITNOT. The only input
/// contains the number to be negated.
/// Returns its binary negation (NOT)
//===================================================

atom_t RUNTIME rt_bitnot( atom_t data )
{
  int64_t x;
  GET_INT( data, x );

  RETURN( new_integer(~x ) );
}


//===================================================
/// \fn		atom_t rt_pick(atom_t list)
/// \brief	implementation of primitive command PICK
/// \param  list   a list
/// \return	randomly selected element of list
///
/// Implement primitiwe function \c PICK.
/// Returns randomly selected element of \c list
//===================================================

atom_t RUNTIME rt_pick( atom_t list )
{
  if (IS_ERROR( list ))
  {
    RETURN( USE(list) );
  }
  if (IS_LIST( list ))
  {
    int len;
    int rand_num;
    if (IS_EMPTY( list ))
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, list ) );
    }
    rand_num = rand( );
    len = list_length( list );
    RETURN( USE( get_at_list( list, rand_num % len ) ) );
  }
  RETURN( new_error( ERROR_NOT_A_LIST, list ) );
}


//===================================================
/// \fn		atom_t rt_remdup(atom_t cip, atom_t data)
/// \brief	implementation of primitive command REMDUP
/// \param  cip	 value of CASEIGNOREDP
/// \param  data   a list or a word
/// \return	the list without duplicate elements
///
/// Implement primitiwe function \c REMDUP.
/// Remove all duplicate elements of its input.
/// The element which remain is rightmost.
//===================================================

atom_t RUNTIME rt_remdup( atom_t cip, atom_t data )
{
  atom_t result;

  if (IS_ERROR(data))
  {
    RETURN(USE(data));
  }

  if (IS_LIST(data))
  {
    atom_t iter;
    atom_t res_iter;

    result = empty_list;
    res_iter = result;

    while (!IS_EMPTY(data))
    {
      iter = CDR(data);
      while (!IS_EMPTY(iter))
      {
        if (atom_equal( cip, CAR(data), CAR( iter ) ))
        {
          break;
        }
        iter = CDR( iter );
      }
      if (IS_EMPTY( iter ))
      {
        if (IS_EMPTY( res_iter ))
        {
          result = res_iter = new_list( USE( CAR(data) ), empty_list );
        }
        else
        {
          CDR( res_iter ) = new_list( USE( CAR(data) ), empty_list );
          res_iter = CDR( res_iter );
        }
      }
      data = CDR(data);
    }
  }
  else // Data is a WORD
  {
    size_t len, total;
    size_t i, j;
    int case_ignore;
    chars_t word;
    atom_t data1;
    if (!atom_to_boolean( cip, &case_ignore ))
    {
      case_ignore = 1;
    }
    data1 = atom_to_word(data);
    if (IS_ERROR(data1))
    {
      RETURN(data1);
    }

    total = len = LENGTH(data1);
    word = STRING(data1);
    if (case_ignore)
    {
      for (i = 0; i < total-1; ++i)
      {
        for (j = i+1; j < total; ++j)
        {
          if (TOLOWER(DEBAR(word[i]))==TOLOWER(DEBAR(word[j])))
          {
            --len;
            break;
          }
        }
      }
      result = create_word(len);
      len = 0;
      for (i = 0; i < total; ++i)
      {
        for (j = i+1; j < total; ++j)
        {
          if (TOLOWER(DEBAR(word[i]))==TOLOWER(DEBAR(word[j])))
          {
            break;
          }
        }
        if (j >= total)
        {
          STRING(result)[len++] = word[i];
        }
      }
    }
    else
    {
      for (i = 0; i < total-1; ++i)
      {
        for (j = i+1; j < total; ++j)
        {
          if (DEBAR(word[i])==DEBAR(word[j]))
          {
            --len;
            break;
          }
        }
      }
      result = create_word(len);
      len = 0;
      for (i = 0; i < total; ++i)
      {
        for (j = i+1; j < total; ++j)
        {
          if (DEBAR(word[i])==DEBAR(word[j]))
          {
            break;
          }
        }
        if (j >= total)
        {
          STRING(result)[len++] = word[i];
        }
      }
    }
    DEUSE(data1);
  }
  RETURN(result);
}


//===================================================
/// \fn		atom_t rt_remove (atom_t cip, atom_t data, atom_t elem)
/// \brief	implementation of primitive command REMOVE
/// \param  cip	 value of CASEIGNOREDP
/// \param  data a list or word
/// \param  elem an atom to be remowed from list or char to be removed from word
/// \return	the input without any occurrence of elem
///
/// Implement primitiwe function \c REMOVE.
/// Return a copy of \c data where all
/// occurrences of \c elem are removed
//===================================================

atom_t RUNTIME rt_remove( atom_t cip, atom_t data, atom_t elem )
{
  atom_t result;
  atom_t res_iter;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_LIST(data))
  {
    result = empty_list;
    res_iter = result;

    while (!IS_EMPTY( data ))
    {
      if (!atom_equal( cip, CAR( data ), elem ))
      {
        if (IS_EMPTY( res_iter ))
        {
          result = res_iter = new_list( USE( CAR( data ) ), empty_list );
        }
        else
        {
          CDR( res_iter ) = new_list( USE( CAR( data ) ), empty_list );
          res_iter = CDR( res_iter );
        }
      }
      data = CDR( data );
    }
  }
  else  // data is a word
  {
    size_t len, total;
    size_t i;
    int case_ignore;
    chars_t word;
    atom_t data1;
    char_t ch;
    if (IS_ERROR(elem))
    {
      RETURN(USE(elem));
    }

    data1 = atom_to_word(elem);
    if (IS_ERROR(data1) || LENGTH(data1) != 1)
    {
      DEUSE(data1);
      RETURN(new_error(ERROR_INCOMPATIBLE_DATA, elem));
    }
    ch = (STRING(data1))[0];
    DEUSE(data1);

    if (!atom_to_boolean( cip, &case_ignore ))
    {
      case_ignore = 1;
    }
    data1 = atom_to_word(data);
    if (IS_ERROR(data1))
    {
      DEUSE(data1);
      RETURN(data);
    }
    total = len = LENGTH(data1);
    word = STRING(data1);
    if (case_ignore)
    {
      ch = TOLOWER(DEBAR(ch));
      for (i = 0; i < total; ++i)
      {
        if (TOLOWER(DEBAR(word[i]))==ch)
        {
          --len;
        }
      }
      result = create_word(len);
      len = 0;
      for (i = 0; i < total; ++i)
      {
        if (TOLOWER(DEBAR(word[i]))!=ch)
        {
          STRING(result)[len++] = word[i];
        }
      }
    }
    else
    {
      ch = DEBAR(ch);
      for (i = 0; i < total; ++i)
      {
        if (DEBAR(word[i])==ch)
        {
          --len;
        }
      }
      result = create_word(len);
      len = 0;
      for (i = 0; i < total; ++i)
      {
        if (DEBAR(word[i])!=ch)
        {
          STRING(result)[len++] = word[i];
        }
      }
    }
    DEUSE(data1);
  }
  RETURN(result);
}


//===================================================
/// \fn		atom_t rt_reverse (atom_t data)
/// \brief	implementation of primitive command REVERSE
/// \param  data a list or a word
/// \return	a copy of data, but elements are reversed
///
/// Implements primitive function \c REVERSE.
/// Creates a reverse copy of a list or word
//===================================================

atom_t RUNTIME rt_reverse( atom_t data )
{
  atom_t result;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_LIST(data))
  {
    if (IS_EMPTY( data ))
    {
      RETURN( data );
    }

    result = empty_list;
    while (!IS_EMPTY( data ))
    {
      result = new_list( USE( CAR( data ) ), result );
      data = CDR( data );
    }
  }
  else  // data must be word
  {
    int i, n;
    data = atom_to_word( data );
    if (IS_ERROR( data ))
    {
      RETURN( data );
    }
    result = create_word(LENGTH( data ));
    n = LENGTH( data );
    for (i = 0; i < n; ++i)
    {
      STRING( result )[n - i - 1] = STRING( data )[i];
    }
    DEUSE (data);
  }

  RETURN( result );
}


//===================================================
/// \fn		atom_t rt_rawascii (atom_t data)
/// \brief	implementation of querie primitive RAWASCII
/// \param  data one char word
/// \return ASCII code of data
///
/// Implements the primitive \c RAWASCII.
/// Returns ASCII code of given char, but interpred
/// control symbols as themselves
//===================================================

atom_t RUNTIME rt_rawascii( atom_t data )
{
  char_t buff[MAX_NUMBER_WORD_LENGTH];
  int buff_len = MAX_NUMBER_WORD_LENGTH;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (IS_ANY_WORD( data ))
  {
    if (LENGTH( data ) != 1)
    {
      RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
    }
#ifdef UNICODE_CHARS
    if (!OPTION_TRADITIONAL)
    {
      RETURN( new_integer( (ushort_t) STRING( data )[0] ) );
    }
#endif
    RETURN( new_integer( (byte_t) STRING( data )[0] ) );
  }

  if (IS_LIST( data ))
  {
    RETURN( new_error( ERROR_NOT_A_WORD, data ) );
  }

  if (atom_to_string( data, buff, &buff_len ))
  {
    if (buff_len == 1)
    {
#ifdef UNICODE_CHARS
      if (!OPTION_TRADITIONAL)
      {
        RETURN( new_integer( (ushort_t) buff[0] ) );
      }
#endif
      RETURN( new_integer( (byte_t) buff[0] ) );
    }

    // Word but too long
    RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
  }
  RETURN( new_error( ERROR_NOT_A_WORD, data ) );
}


//===================================================
/// \fn		atom_t rt_gensym()
/// \brief	implements primitive function GENSYM
/// \return unique word each time it's called
///
/// Implements the primitive \c GENSYM.
/// Returns unique word each time it's called
/// The words are like that \c G1, \c G2, \c G3...
//===================================================

atom_t RUNTIME rt_gensym( )
{
  static int num = 1;
  int num_len, temp, temp_len;
  char_t sym[MAX_NUMBER_WORD_LENGTH];

  sym[0] = TEXT( 'G' );

  num_len = (int) log10( num ) + 1;
  sym[num_len + 1] = NULL_CHAR;

  for (temp = num, temp_len = num_len; temp_len > 0; --temp_len, temp /= 10)
  {
    sym[temp_len] = TEXT( '0' ) + temp % 10;
  }
  ++num;
  RETURN( new_word( sym, num_len + 1 ) );
}


//===================================================
/// \fn		atom_t rt_substringp(atom_t cip, atom_t data2, atom_t data1)
/// \brief	implements primitive predicate SUBSTRINGP
/// \param  cip	 value of CASEIGNOREDP
/// \param  data1 String to be search
/// \param  data2 String to search in
/// \return boolean atom
///
/// Implements the primitive precicate \c SUBSTRINGP
/// Returns \c true if data1 is substring of data2
/// \c false if it is not, or any of inputs is not word
//===================================================

atom_t RUNTIME rt_substringp( atom_t cip, atom_t data2, atom_t data1 )
{
  int case_ignore = 0;
  int result = 0;
  int pos;

  // Chek for error args
  if (IS_ERROR( data1 ))
  {
    RETURN( USE(data1) );
  }
  if (IS_ERROR( data2 ))
  {
    RETURN( USE(data2) );
  }

  // Check for lists - easy way to return FALSE
  if (IS_LIST( data1 ) || IS_LIST( data2 ))
  {
    RETURN( USE( false_true[0] ) );
  }

  // Get words from atoms
  data1 = atom_to_word( data1 );
  if (IS_ERROR( data1 ))
  {
    RETURN( data1 );
  }
  data2 = atom_to_word( data2 );
  if (IS_ERROR( data2 ))
  {
    DEUSE( data1 );
    RETURN( data2 );
  }

  if (!atom_to_boolean( cip, &case_ignore ))
  {
    case_ignore = 1;
  }

  // Here is the comparison
  for (pos = 0; pos + LENGTH( data1 ) <= LENGTH( data2 ); ++pos)
  {
    if (!word_compare( case_ignore, STRING( data2 ) + pos, LENGTH( data1 ),
          STRING( data1 ), LENGTH( data1 ) ))
    {
      result = 1;
      break;
    }
  }

  //clear and return
  DEUSE( data1 );
  DEUSE( data2 );
  RETURN( USE( false_true[result] ) );
}


//===================================================
/// \fn		atom_t rt_substring(atom_t cip, atom_t data2, atom_t data1)
/// \brief	implements primitive SUBSTRING
/// \param  cip	 value of CASEIGNOREDP
/// \param  data1 String to be search
/// \param  data2 String to search in
/// \return boolean atom
///
/// Implements the primitive \c SUBSTRING
/// Returns the position of data1 in data2 if they are
/// words (0 if not found), or generates an error is
/// any of the inputs is not a word.
//===================================================

atom_t RUNTIME rt_substring( atom_t cip, atom_t data2, atom_t data1 )
{
  int case_ignore = 0;
  int result = 0;
  int pos;

  // Chek for error args
  if (IS_ERROR( data1 ))
  {
    RETURN( USE(data1) );
  }
  if (IS_ERROR( data2 ))
  {
    RETURN( USE(data2) );
  }

  // Get words from atoms
  data1 = atom_to_word( data1 );
  if (IS_ERROR( data1 ))
  {
    RETURN( data1 );
  }
  data2 = atom_to_word( data2 );
  if (IS_ERROR( data2 ))
  {
    DEUSE( data1 );
    RETURN( data2 );
  }

  if (!atom_to_boolean( cip, &case_ignore ))
  {
    case_ignore = 1;
  }

  // Here is the search
  for (pos = 0; pos + LENGTH( data1 ) <= LENGTH( data2 ); ++pos)
  {
    if (!word_compare( case_ignore, STRING( data2 ) + pos, LENGTH( data1 ),
          STRING( data1 ), LENGTH( data1 ) ))
    {
      result = pos+1;
      break;
    }
  }

  //clear and return
  DEUSE( data1 );
  DEUSE( data2 );
  RETURN( new_integer(result) );
}


//===================================================
/// \fn		atom_t rt_combine (atom_t data2, atom_t data1)
/// \brief	implementation of constructor primitive COMBINE
/// \param  data1   Element to add
/// \param  data2   List or word
/// \return New list or word containing data2 with data1 at start
///
/// Implements the primitive \c COMBINE.
/// Returns \c data2 with \c data1 inserted at front.
//===================================================

atom_t RUNTIME rt_combine( atom_t data2, atom_t data1 )
{
  atom_t res;

  if (IS_ERROR( data1 )) RETURN( USE(data1) );
  if (IS_ERROR( data2 )) RETURN( USE(data2) );

  if (IS_LIST( data2 ))
  {
    RETURN( new_list( USE( data1 ), USE( data2 ) ) );
  }
  data1 = atom_to_word( data1 );
  if (IS_ERROR( data1 ))
  {
    RETURN( data1 );
  }
  data2 = atom_to_word( data2 );
  if (IS_ERROR( data2 ))
  {
    DEUSE( data1 );
    RETURN( data2 );
  }

  // Copy the data
  res = create_word( LENGTH( data1 ) + LENGTH( data2 ) );
  STRNCPY( STRING( res ), STRING( data1 ), LENGTH( data1 ) );
  STRNCPY( STRING( res ) + LENGTH( data1 ), STRING( data2 ), LENGTH( data2 ) );
  *(STRING( res ) + LENGTH( data1 ) + LENGTH( data2 )) = NULL_CHAR;

  DEUSE( data1 );
  DEUSE( data2 );
  RETURN( res );
}


//===================================================
/// \fn     atom_t rt_quoted (atom_t data)
/// \brief  implementation of primitive function QUOTED
/// \param  data   Element to quote
/// \return quoted input if word
///
/// Implements the primitive \c QUOTED.
/// Returns \c data if it's a list or \c data with quotation if is a word.
//===================================================

atom_t RUNTIME rt_quoted( atom_t data )
{
  atom_t res;
  if (IS_ERROR( data )) RETURN( USE(data) );
  if (IS_LIST( data )) RETURN( USE( data ) );

  data = atom_to_word( data );
  if (IS_ERROR( data )) RETURN( data );

  res = create_word( LENGTH( data ) + 1 );
  STRING( res )[0] = '\"';
  STRNCPY( STRING( res ) + 1, STRING( data ), LENGTH( data ) );
  *(STRING( res ) + LENGTH( data ) + 1) = NULL_CHAR;
  DEUSE( data );
  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_throw( int count, atom_t data1, atom_t data2 )
/// \brief	implementation of primitive THROW
/// \param count	number of arguments (1 or 2)
/// \param data1	first parameter of THROW
/// \param data2	second parameter of THROW
/// \return unbound or error atom
///
/// Implements the THROW primitive:
///	THROW "TOPLEVEL
///	THROW "SYSTEM
///	THROW "ERROR
///	(THROW "ERROR message)
///	THROW tag
///	(THROW tag value)
//===================================================

atom_t RUNTIME rt_throw( int count, atom_t data1, atom_t data2 )
{
  atom_t tag;
  atom_t value;

  if (count == 1)
  {
    tag = data1;
    value = unbound;
  }
  else
  {
    tag = data2;
    value = data1;
  }

  // THROW "TOPLEVEL
  if (same_words( tag, word_toplevel ))
  RETURN( new_error( EXIT_BY_THROW_TOPLEVEL, tag ) );

  // THROW "SYSTEM
  if (same_words( tag, word_system ))
  RETURN( new_error( EXIT_BY_THROW_SYSTEM, tag ) );

  // THROW "ERROR
  // THROW "ERROR <message>
  if (same_words( tag, word_error ))
  {
    if (count == 1)
    {
      RETURN( new_error( EXIT_BY_THROW_ERROR, unbound ) );
    }
    else
    {
      RETURN( new_error( EXIT_BY_THROW_USER_ERROR, value ) );
    }
  }

  // THROW <tag>
  // THROW <tag> <value>
  atom_t list = new_list( USE( tag ), new_list( USE( value ), empty_list ) );
  atom_t result = new_error( count == 1 ? EXIT_BY_THROW_TAG : EXIT_BY_THROW_TAG_VALUE, list );
  DEUSE( list );
  RETURN( result );
}



//===================================================
/// \fn		atom_t rt_catch( atom_t commands, atom_t tag )
/// \brief	dummy implementation of primitive command CATCH
/// \param commands	commands monitored by catch
/// \param tag		catch tag
///
/// Dummy implementation of primitive \c CATCH.
//===================================================

atom_t RUNTIME rt_catch( atom_t commands, atom_t tag )
{
  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_catchchk( int status, atom_t tag, atom_t data )
/// \brief		checks for valid result of a catch
/// \param	status	output status of data
/// \param	tag	catch's tag
/// \param	data	value to check
/// \return		error or unbound atom
///
/// Checks whether the result of a catch command.
/// If the result is thrown by throw with the same
/// tag, then mask the result and return the
/// thrown data. Results are also masked if the
/// catch tag is ERROR and the thrown calue is
/// an error-meaning error atom.
///
/// In all other cases (thrown with another tag
/// or error) return the same result.
//===================================================
#define rt_catchchk __attribute__((used,noinline,regparm(0),stdcall)) rt_catchchk

atom_t rt_catchchk(  int status, atom_t tag, atom_t data )
//atom_t rt_catchchk(  atom_t tag, atom_t data )
#undef rt_catchchk
{
  //printf("---------------\n");
  //printf("inside rt_catchchk\n");
  //printf("    - data%d  = ",IS_UNBOUND(data)); dumpln(data);
  //printf("    - tag    = "); dumpln(tag);
  //printf("    - status  = %d\n",status);
  //printf("---------------\n");

  atom_t result;

  catch_output_flag = status;

  if (!IS_ERROR( data ))
  {
    //catch_output_flag = 0;
    DEUSE( tag );

    return data;
  }
  //printf("CATCH TAG=");dumpln(tag);
  //printf("ERRDATA=");dumpln((ERRDATA(data)));
  //printf("THROW TAG=");dumpln(CAR(ERRDATA(data)));

  // process true errors
  if (ERRCODE( data ) < FIRST_EXIT_CODE || ERRCODE( data ) > LAST_EXIT_CODE)
  {
    if (!IS_ANY_WORD( tag ))
    {
      DEUSE( tag );
      return data;
    }

    if (!same_words( word_error, tag ))
    {
      DEUSE( tag );
      return data;
    }

    result = unbound;
    goto exit_catch;
  }

  // return throw-exception if tags are not (sub)words
  if (!IS_ANY_WORD( tag ) || !IS_ANY_WORD( CAR( ERRDATA( data ) ) ))
  {
    DEUSE( tag );
    return data;
  }
  // return throw-exception is tags are not equal
  if (!same_words( tag, CAR( ERRDATA( data ) ) ))
  {
    DEUSE( tag );
    return data;
  }

  // tags match or error captured
  result = USE( CAR( CDR( ERRDATA( data ) ) ) );

  exit_catch:
  //while (REF( data ) > 1) DEUSE( data ); //2009
  //DEUSE( last_error );
  last_error = data;
  //clear_all_errors();
  DEUSE( tag );
  //printf("result="); dumpln(result);
  //printf("last_error(%d)=",REF(last_error)); dumpln(last_error);

  catch_output_flag = 0; // forget any OUTPUT with error
  return result;
}





//===================================================
/// \fn		atom_t rt_error( )
/// \brief	implementation of primitive function ERROR
/// \return	a list describing the last error
///
/// Implementation of the primitive function \c ERROR.
/// Returns a list describing the last error (if any)
/// and then clears the error.
//===================================================

atom_t RUNTIME rt_error( )
{
  if (IS_EMPTY( last_error ))
  RETURN( last_error );

#ifdef SAFEMODE
  assert( IS_ERROR( last_error ) );
#endif

  atom_t code;
  atom_t message;
  atom_t procedure;
  atom_t source;

  //printf("--LAST ERROR->"); dump_atom(last_error,1); printf("\n");
  

  // get error code
  code = new_integer( ERRCODE( last_error ) );
  //printf("--ERROR CODE-->"); dump_atom(code,1); printf("\n");
  
  // get error message
  if (ERRCODE( last_error ) == EXIT_BY_THROW_USER_ERROR)
  message = USE( ERRDATA( last_error ) );
  else
  message = USE( error_texts[ERRCODE( last_error )] );
  //printf("--MESSAGE-->"); dump_atom(message,1); printf("\n");

  // get error procedure/source
  source = empty_list;
  procedure = empty_list;
  atom_t p;
  for (p = ERRPOS( last_error ); IS_NOT_EMPTY( p ); p = CDR( p ))
  if (IS_LIST( CAR( p ) ))
  {
    //printf("---"); dump_atom(p,1); printf("\n");
    //printf(">>>"); dumpln(CAR(p));
    if (!IS_EXPRESSION( CAR( p ) )) continue;
    //printf("???"); dumpln(CAR(CAR(p)));
    atom_t var = find_var( CAR( CAR( p ) ), globals );
    if (var && IS_PRIMITIVE( var )) continue;
    //printf("!!!\n");
    source = CAR( p );
    procedure = CAR( source );
  }
  //printf("--AFTER FOR-->!\n");
  
  atom_t result;
  result = new_list( USE( source ), empty_list );
  result = new_list( USE( procedure ), result );
  result = new_list( message, result );
  result = new_list( code, result );

  //REF(last_error) = 1;
  //DEUSE( last_error );
  last_error = empty_list;
  //printf("--BEFORE CLEAR-->!\n");
  //clear_all_errors(); //2011.11.30 removed, because causes bug 3445230. now all test cases work fine
  //printf("--AFTER CLEAR-->!\n");

  RETURN( result );
}



//===================================================
/// \fn		atom_t rt_tag( )
/// \brief	dummy implementation of primitive command TAG
///
/// Dummy implementation of primitive \c TAG.
//===================================================

atom_t RUNTIME rt_tag( )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_goto( int static_link, atom_t parent, atom_t data, atom_t source )
/// \brief	implementation of primitive command GOTO
/// \param static_link	static link from the current frame
/// \param parent	current parent
/// \param data		name of variable
/// \param source	source of GOTO
/// \return		unbound or error atom
///
/// Implementation of primitive \c GOTO. Looks for
/// the tag in the local variables. If not found then
/// return an error atom. If found then return the
/// ::VALUE of the tag-variable.
//===================================================

atom_t RUNTIME rt_goto( int static_link, atom_t parent, atom_t data, atom_t source )
{
  if (IS_ERROR( data )) return ( USE( data ));
  if (!IS_ANY_WORD( data )) return ( new_error( ERROR_NOT_A_WORD, source ));

  atom_t var = find_local_var( data, parent );
  if (!var || !IS_TAG( var )) return ( new_error( ERROR_NOT_A_TAG, source ));

  //printf("jumpto=%x\n",(int)INTEGER(VALUE(var)));
  RETURN( USE( VALUE( var ) ) );
}




//===================================================
/// \fn		atom_t rt_iftrue( atom_t commands )
/// \brief	dummy implementation of primitive command IFTRUE
/// \param commands	commands to run
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for IFTRUE
/// primitive.
//===================================================

atom_t RUNTIME rt_iftrue( atom_t commands )
{
  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_iffalse( atom_t commands )
/// \brief	dummy implementation of primitive command IFFALSE
/// \param commands	commands to run
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for IFFALSE
/// primitive.
//===================================================

atom_t RUNTIME rt_iffalse( atom_t commands )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_test( atom_t condition, int frame )
/// \brief	implementation of primitive command TEST
/// \param frame	base frame pointer
/// \param condition	condition to store
/// \return	unbound atom
///
/// Sets the value of local variable at address
/// EBP+BASE_OFFEST_TEST to be the value of condition.
/// This value is later accessed by \c IFTRUE and
/// \c IFFALSE primitives.
//===================================================

atom_t RUNTIME rt_test( atom_t condition, int frame )
{
  int* test_ptr;

  test_ptr = (int*) (frame + BASE_OFFSET_TEST);
  GET_BOOLEAN( condition, *test_ptr );

  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_backslashedp( atom_t data )
/// \brief	implementation of primitive BACKSLASHED?
/// \param data	one char word
/// \return	\c true if \c data contains backslashed character
///
/// Implements the primitive \c BACLSLASHED?.
//===================================================

atom_t RUNTIME rt_backslashedp( atom_t data )
{
  data = atom_to_word( data );
  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (LENGTH( data ) != 1)
  {
    RETURN( new_error( ERROR_INCOMPATIBLE_DATA, data ) );
  }

  char_t ch = *STRING( data );
  DEUSE( data );
  RETURN( USE( false_true[ch != DEBAR( ch )] ) );
}




//===================================================
/// \fn		atom_t rt_text( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of TEXT
/// \param  data	function name
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return a list with the function's commands
///
/// Implements primitive \c TEXT. Returns a list containing
/// the commands of a given function. The list could
/// be accepted by \c DEFINE primitive.
//===================================================

atom_t RUNTIME rt_text( int static_link, atom_t parent, atom_t data )
{
  atom_t word = atom_to_word( data );
  if (IS_ERROR( word )) RETURN( USE(word) );

  //printf("looking for "); dump(word); printf(" in "); dumpln(NAME(parent));
  atom_t var = find_var( word, parent );
  DEUSE( word );

  //test for invalid variable types
  if (IS_PRIMITIVE( var ) || (!IS_FUNCTION( var ) && !IS_COMMAND( var )))
  RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, data ) );

  atom_t result = empty_list;
  atom_t reslast = empty_list;

  atom_t element = empty_list;
  atom_t elemlast = empty_list;

  atom_t x;
  int n;

  // prepare list of right inputs
  // because of their reverse order, we can
  // construct the list without append()
  for (x = LOCALS( var ), n = 0; IS_NOT_EMPTY( x ); x = CDR( x ))
  if (IS_NORMAL( CAR( x ) ) && IS_VARIABLE( CAR( x ) ) && OFFSET( CAR( x ) ) > 0)
  {
    n++;
    if (n > RARGS( var ))
    element = new_list( USE( NAME( CAR( x ) ) ), element ); // left inputs
    else
    result = new_list( USE( NAME( CAR( x ) ) ), result ); // right inputs
  }

  // if there were left inputs then put them as
  // a sublist in front of right inputs
  if (IS_NOT_EMPTY( element ))
  {
    result = new_list( element, result );
    element = empty_list;
  }

  // pack all inputs as the first element of the result
  result = new_list( result, empty_list );
  reslast = result;

  // group elements into lines
  for (x = BODY( var ); IS_NOT_EMPTY( x ); x = CDR( x ))
  {
    if (IS_NOT_EMPTY( element ) && GET_FLAGS( x, FLAG_NEWLINE ))
    {
      append( element, &result, &reslast );
      element = empty_list;
      elemlast = empty_list;
    }
    append( USE( CAR( x ) ), &element, &elemlast );
  }

  // process any leftovers
  if (IS_NOT_EMPTY( element ))
  {
    append( element, &result, &reslast );
  }

  RETURN( result );
}




//===================================================
/// \fn		atom_t rt_fulltext( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of FULLTEXT
/// \param  data	function name
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return a list with the function's commands
///
/// Implements primitive \c FULLTEXT. Returns a word
/// containing the function as source. If the source
/// is not avaialble, then return the same result as
/// TEXT.
//===================================================

atom_t RUNTIME rt_fulltext( int static_link, atom_t parent, atom_t data )
{
  atom_t word = atom_to_word( data );
  if (IS_ERROR( word )) RETURN( USE(word) );

  //printf("looking for "); dump(word); printf(" in "); dumpln(NAME(parent));
  atom_t var = find_var( word, parent );
  DEUSE( word );

  //test for invalid variable types
  if (IS_PRIMITIVE( var ) || (!IS_FUNCTION( var ) && !IS_COMMAND( var )))
  RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, data ) );

  atom_t result;
  if (IS_ANY_WORD( SOURCE( var ) ))
  result = USE( FULLSOURCE( var ) );
  else
  result = rt_text( static_link, parent, data );

  RETURN( result );
}




//===================================================
/// \fn		atom_t rt_run( int static_link, atom_t parent, atom_t data, int mode )
/// \brief	implementation of RUN
/// \param  data	data to run
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return var atom containing the compiled code
///
/// Implements primitive command/function RUN.
/// Creates a local function with the given body,
/// compiles it, and returns its var atom.
///
/// It is supposed that the caller of rt_run() should
/// use the result to do the actual call of the
/// newly compiled code.
///
/// If \c mode is ::COMPILE_AS_PROC then the compiled
/// code is a list of commands, otherwise it is a
/// single expression.
//===================================================

atom_t RUNTIME rt_run( int static_link, atom_t parent, atom_t data, int mode )
{
  //printf("-----------ENTER rt_run()---------------\n");
  //printf("compiling "); dump_atom(data,1); printf("\n");
  //if( mode==COMPILE_AS_PROC ) printf("--->procedure<---\n");
  //if( mode==COMPILE_AS_FUNC ) printf("--->function<---\n");
  //if( mode==COMPILE_AS_UNKNOWN ) printf("--->both proc&func<---\n");
  //printf("--->in parent= "); dump_atom(parent,1); printf("\n");

  if (IS_ERROR( data )) RETURN( USE(data) );

  // create temporary local var where to compile the code
  atom_t func = new_var( word_to, parent, 0 );
  need_descr2( func );

  SOURCE( func ) = USE( data );

  compiling_code = 1;
  atom_t x = compile_function( func, mode, COMPILE_AS_NON_MACRO );
  compiling_code = 0;
  if (IS_ERROR( x ))
  {
    DEUSE( func );
    RETURN( x );
  }

  //printf("compiled tree="); dump_atom(TREE(func),1); printf("\n");
  //printf("-----------EXIT rt_run()---------------\n");
  //printf("result="); dumpln(func);
  //printf("---------------------------------------\n");
  //printf("!!!locals ="); dump_atom(LOCALS(func),1); printf("\n");
  //printf("!!!to transfer to parent="); dump_atom(NAME(PARENT(func)),1); printf("\n");
  //printf("---------------------------------------\n");

  // wrong place:: transfer all local vars from here to the parent of run
  // it is wrong, because transfer should be done after the code is
  // executed. currently the code is only compiled
  //atom_t a;
  //for( a=LOCALS(func); IS_NOT_EMPTY(a); a=CDR(a) )
  //  {
  //    //printf(" "); dump_atom(NAME(CAR(a)),1);
  //    
  //  }
  //printf("\n");
  RETURN( func );
}




//===================================================
/// \fn		atom_t rt_runmacro( int static_link, atom_t parent, atom_t data, int mode )
/// \brief	implementation of RUNMACRO
/// \param  data	data to run
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return var atom containing the compiled code
///
/// Implements primitive command/function RUNMACRO.
/// Creates a local function with the given body,
/// compiles it as a macro, and returns its var atom.
///
/// It is supposed that the caller of rt_runmacro()
/// should use the result to do the actual call of the
/// newly compiled code.
///
/// If \c mode is ::COMPILE_AS_PROC then the compiled
/// code is a list of commands, otherwise it is a
/// single expression.
//===================================================

atom_t RUNTIME rt_runmacro( int static_link, atom_t parent, atom_t data, int mode )
{
  //printf("-----------ENTER rt_run()---------------\n");
  //printf("compiling "); dump_atom(data,1); printf("\n");
  //if( mode==COMPILE_AS_PROC ) printf("--->procedure<---\n");
  //if( mode==COMPILE_AS_FUNC ) printf("--->function<---\n");
  //if( mode==COMPILE_AS_UNKNOWN ) printf("--->both proc&func<---\n");
  //printf("--->in parent= "); dump_atom(parent,1); printf("\n");


  if (IS_ERROR( data )) RETURN( USE(data) );

  // create temporary local var where to compile the code
  atom_t func = new_var( word_to, parent, 0 );
  need_descr2( func );

  SOURCE( func ) = USE( data );

  compiling_code = 1;
  atom_t x = compile_function( func, mode, COMPILE_AS_MACRO );
  compiling_code = 0;
  if (IS_ERROR( x ))
  {
    DEUSE( func );
    RETURN( x );
  }

  RETURN( func );
}




//===================================================
/// \fn		atom_t rt_runresult( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of RUNRESULT
/// \param  data	data to run
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \return var atom containing the compiled code
///
/// Implements primitive command/function RUNRESULT
/// by reusing ::rt_run() function.
//===================================================

atom_t RUNTIME rt_runresult( int static_link, atom_t parent, atom_t data )
{
  atom_t res = rt_run( static_link, parent, data, COMPILE_AS_UNKNOWN );
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_runresult_fix( atom_t data )
/// \brief	fixes the result of RUNRESULT
/// \param  data	data to fix
/// \return fixed atom
///
/// Fixes the result of RUNRESULT. If the result is
/// error then return it without change. If it is
/// unbound then return empty list. Otherwise create
/// an one-element list and put \c data as its element.
//===================================================
#define rt_runresult_fix __attribute__((used,noinline,regparm(0),stdcall)) rt_runresult_fix

atom_t rt_runresult_fix( atom_t data )
#undef rt_runresult_fix
{
  if (IS_ERROR( data )) RETURN( USE(data) );
  if (IS_UNBOUND( data )) RETURN( empty_list );
  RETURN( new_list( data, empty_list ) );
}




//===================================================
/// \fn		atom_t rt_define( int static_link, atom_t parent, atom_t value, atom_t name )
/// \brief		implementation of primitive command DEFINE
/// \param static_link	static link from the current frame
/// \param parent	current parent
/// \param name		name of function
/// \param value	body of the function
///
/// Implements the primitive command \c DEFINE.
//===================================================

atom_t RUNTIME rt_define( int static_link, atom_t parent, atom_t value, atom_t name )
{
  if (IS_ERROR( value )) RETURN( USE(value) );
  if (IS_ERROR( name )) RETURN( USE(name) );

  atom_t word = atom_to_word( name );
  if (IS_ERROR( word )) RETURN( word );

  if (!IS_LIST( value ))
  {
    DEUSE( word );
    RETURN( new_error( ERROR_NOT_A_LIST, value ) );
  }

  // create function (similarily to define_user_function)
#ifdef DEBUG_TO_END
  printf( "<TO-END-RUNTIME> DEFINING=" );
  dumpln( name );
#endif

  // create the function
  atom_t function = new_var( word_to, parent, 1 ); //DEUSE(to);
  need_descr2( function );
  SET_FLAGS( function, FLAG_FUNCTION );
  LARGS( function ) = 0;
  RARGS( function ) = 0;

  // get lists of left and right inputs
  atom_t lefts = empty_list; // left inputs
  atom_t rights = empty_list; // right inputs

  if (IS_NOT_EMPTY( value ))
  { // there is at least one element
    // is it list?
    if (IS_LIST( CAR( value ) )) rights = CAR( value );

    // now test for left parameters
    if (IS_NOT_EMPTY( rights ) && IS_LIST( CAR( rights ) ))
    {
      lefts = CAR( rights );
      rights = CDR( rights );
    }
  }

#ifdef DEBUG_TO_END
  printf( "<TO-END-RUNTIME> LEFT PARAMS=" );
  dumpln( lefts );
  printf( "<TO-END-RUNTIME> RIGHT PARAMS=" );
  dumpln( rights );
#endif

  // process left inputs
  while (IS_NOT_EMPTY( lefts ))
  {
    if (!IS_ANY_WORD( CAR( lefts ) ))
    {
      DEUSE( word );
      LOCALS( parent ) = behead( LOCALS( parent ) ); // this DEUSEs function
      return new_error( ERROR_NOT_A_WORD, lefts );
    }

    atom_t var = new_local_var( CAR( lefts ), function, 0 );
    if (IS_ERROR( var ))
    {
      DEUSE( word );
      LOCALS( parent ) = behead( LOCALS( parent ) ); // this DEUSEs function
      return var;
    }

    SET_FLAGS( var, FLAG_VARIABLE );
    LARGS( function )++;
#ifdef SAFE_MODE
    assert( LARGS( function ) < 255 );
#endif
    lefts = CDR( lefts );
  }

  // process right inputs
  while (IS_NOT_EMPTY( rights ))
  {
    if (!IS_ANY_WORD( CAR( rights ) ))
    {
      DEUSE( word );
      LOCALS( parent ) = behead( LOCALS( parent ) ); // this DEUSEs function
      return new_error( ERROR_NOT_A_WORD, rights );
    }

    atom_t var = new_local_var( CAR( rights ), function, 0 );
    if (IS_ERROR( var ))
    {
      DEUSE( word );
      LOCALS( parent ) = behead( LOCALS( parent ) ); // this DEUSEs function
      return var;
    }

    SET_FLAGS( var, FLAG_VARIABLE );
    RARGS( function )++;
#ifdef SAFE_MODE
    assert( RARGS( function ) < 255 );
#endif
    rights = CDR( rights );
  }

#ifdef DEBUG_TO_END
  printf( "<TO-END-RUNTIME> FUNC DEF=" );
  dumpln( function );
#endif


  // set offset of parameters
  int offset = BASE_OFFSET_PARAMS; // this is the start offset
  atom_t x;
  for (x = LOCALS( function ); IS_NOT_EMPTY( x ); x = CDR( x ))
  {
    //printf("set offset of "); dump(NAME(CAR(x))); printf(" to be %d\n",offset);
    OFFSET( CAR( x ) ) = offset;
    offset += sizeof ( atom_t);
  }

  // the CDR(value) is the body of the function, but it is
  // cut into sublists -- each line is a single list. So
  // all these list should be combined into one, which will
  // become the body of the function
  atom_t body = empty_list;
  atom_t body_end = empty_list;
  atom_t a;
  atom_t b;
  for (a = CDR( value ); IS_NOT_EMPTY( a ); a = CDR( a ))
  {
    b = CAR( a );
    if (IS_NOT_EMPTY( b ))
    { // the first element should have FLAG_NEWLINE set
      append( USE( CAR( b ) ), &body, &body_end );
      SET_FLAGS( body, FLAG_NEWLINE );
    }
    for (b = CDR( b ); IS_NOT_EMPTY( b ); b = CDR( b ))
    append( USE( CAR( b ) ), &body, &body_end );
  }

#ifdef DEBUG_TO_END
  printf( "<TO-END-RUNTIME> FUNC BODY=" );
  dumpln( body );
#endif



  // check whether the function is already defined
  atom_t var = find_runtime_var( word, static_link );
  if (var)
  {
    if (LARGS( var ) != LARGS( function ) || RARGS( var ) != RARGS( function ))
    {
      LOCALS( parent ) = behead( LOCALS( parent ) ); // this DEUSEs function
      atom_t result = new_error( ERROR_INCOMPATIBLE_REDEFINITION, value );
      return result;
    }

    // forget data of the old function and
    // reuse data of the new function
    DEUSE( FULLSOURCE( var ) );
    DEUSE( SOURCE( var ) );
    DEUSE( LOCALS( var ) );
    DEUSE( BODY( var ) );
    DEUSE( TREE( var ) );
    DEUSE( BINARY( var ) );

    BODY( var ) = empty_list;
    TREE( var ) = empty_list;
    BINARY( var ) = empty_list;

    SOURCE( var ) = body;
    LOCALS( var ) = USE( LOCALS( function ) );

    LOCALS( parent ) = behead( LOCALS( parent ) );
    //DEUSE(function);
    function = var;
  }
  else
  {
    DEUSE( NAME( function ) );
    NAME( function ) = USE( word );
    BODY( function ) = empty_list;
  }

  //LEVEL( function ) = level;
  ADDRESS( function ) = 0;
  PRIORITY( function ) = PRIORITY_FUN;

  SOURCE( function ) = body;
  FULLSOURCE( function ) = unbound;

  //DEUSE( input );
#ifdef DEBUG_TO_END
  printf( "<TO-END> DEFINED FUNCTION " );
  dumpln( NAME( function ) );
  printf( "<TO-END>           SOURCE " );
  dumpln( SOURCE( function ) );
  printf( "<TO-END>           BODY   " );
  dumpln( BODY( function ) );
  printf( "<TO-END>           TREE   " );
  dumpln( TREE( function ) );
#endif

  DEUSE( word );

  atom_t y = compile_function( function, COMPILE_AS_PROC, COMPILE_AS_NON_MACRO );
  if (IS_ERROR( y )) return y;

  return unbound;
}


//===================================================
/// \fn		atom_t rt_for( atom_t body, atom_t limits, atom_t var )
/// \brief	dummy implementation of primitive command FOR
/// \param body     body of FOR command
/// \param limits   list of initial and final limits
/// \param var      name of control variable
/// \return	unbound atom
///
/// This is a dummy implementation, which is not used
/// except for reserving a unique address for FOR
/// primitive.
//===================================================

atom_t RUNTIME rt_for( atom_t body, atom_t limits, atom_t var )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_libload (atom_t data)
/// \brief	implementation of primitive LIBLOAD
/// \param  data filename of the dynamic library
/// \return Number of library handle
///
/// Implements the primitive \c LIBLOAD.
/// Returns the library handle.
//===================================================

atom_t RUNTIME rt_libload( atom_t data )
{
  void* handle;

  if (IS_ERROR( data ))
  {
    RETURN( USE(data) );
  }

  if (!IS_ANY_WORD( data ))
  {
    RETURN( new_error( ERROR_NOT_A_WORD, data ) );
  }

  // convert char_t* into char*
  int len = LENGTH( data );
  chars_t ptr = STRING( data );
  char filename[len + 1+3+1+2];
  int i=0;
  //#ifndef WINDOWS
  //filename[i++] = 'l';
  //filename[i++] = 'i';
  //filename[i++] = 'b';
  //len += 3;
  //#endif

  for (; i < len; i++) filename[i] = *(ptr++);
  filename[len] = '\0';
  //#ifndef WINDOWS
  //filename[len++] = '.';
  //filename[len++] = 's';
  //filename[len++] = 'o';
  //filename[len] = '\0';
  //#endif

  //dumpln(data);
  //fprintf(stderr, "loading library %s\n\n\n",filename);
  handle = dlopen( filename, RTLD_LAZY );
  //if (!handle) fputs(dlerror(), stderr);
  //fprintf(stderr, "\nloaded result=%d.\n",(int)handle);
  if( !handle )
	{
		// searching failed, try again looking in subfolder
		// lib of the folder where the compiler is
		char buf[PATH_MAX+1];
		strncpy(buf,option_compiler_filename,strlen(option_compiler_filename));
		char* path = dirname(buf);
		int pathlen = strlen(path);
		strncpy(buf,path,pathlen);
		
		int filelen = strlen(filename);
		if( pathlen+filelen+6 > PATH_MAX ) filelen=0;
#ifdef WINDOWS
		strncpy(buf+pathlen,"\\lib\\",5);
#else
		strncpy(buf+pathlen,"/lib/",5);
#endif
		strncpy(buf+5+pathlen,filename,filelen);
		buf[pathlen+5+filelen] = '\0';
		
		//printf("failed %s, try '%s' while compiler is %s\n",filename,buf,option_compiler_filename);
		handle = dlopen( buf, RTLD_LAZY );
	}

  return new_integer( (int) handle );
}


//===================================================
/// \fn		atom_t rt_libfree (atom_t data)
/// \brief	implementation of primitive LIBFREE
/// \param  data handle of the dynamic library
/// \return unbound atom
///
/// Implements the primitive \c LIBFREE.
//===================================================

atom_t RUNTIME rt_libfree( atom_t data )
{
  int64_t x;

  GET_INT( data, x );

  dlclose( (void*) (int) x );

  return unbound;
}


//===================================================
/// \fn		atom_t rt_blocksize( int static_link, atom_t parent, atom_t prototype )
/// \brief	implementation of PACKSIZE
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  prototype	pack prototype
/// \return the pack size in bytes
///
/// Implements primitive PACKSIZE. Returns the size
/// of a pack in bytes. The pack is defined by a
/// prototype list. If unknown type is reached then
/// return error atom.
//===================================================

atom_t RUNTIME rt_blocksize( int static_link, atom_t parent, atom_t prototype )
{
  CHECK_PARAM( prototype );

  if( IS_LIST( prototype ) )
  {
    RETURN( traverse_pack( static_link, parent, prototype, empty_list, 0, MEM_STRUCT_SIZE ) );
  }
  else if( IS_MEM(prototype) && IS_INTEGER(ATOMS(prototype)) )
  {
    RETURN( USE( ATOMS(prototype) ) );
  }
  else
  {
    RETURN( new_error(ERROR_NOT_BLOCK_OR_DEF,prototype) );
  }
}



//===================================================
/// \fn		atom_t rt_listtoblock( int static_link, atom_t parent, atom_t prototype, atom_t data )
/// \brief	implementation of LISTTOBLOCK
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  data	data to pack
/// \param  prototype	pack prototype
/// \return memory atom with packed data
///
/// Implements primitive LISTTOBLOCK. Creates a memory atom
/// big enough to hold all packed data.
//===================================================
atom_t RUNTIME rt_listtoblock( int static_link, atom_t parent, atom_t prototype, atom_t data )
{
  atom_t size = rt_blocksize( static_link, parent, prototype );
  if( IS_ERROR(size) )
  {
    RETURN( size );
  }
  
  atom_t memory = new_mem( INTEGER(size) );
  ATOMS(memory) = size;

  void* ptr = MEMORY( memory );

  atom_t res = traverse_pack( static_link, parent, prototype, data, ptr, MEM_STRUCT_PACK );
  if( IS_ERROR(res) )
  {
    DEUSE( memory );
    RETURN( res );
  }

  RETURN( memory );
}




//===================================================
/// \fn		atom_t rt_blocktolist( int static_link, atom_t parent, atom_t prototype, atom_t data )
/// \brief	implementation of PACK
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  data	data to unpack
/// \param  prototype	pack prototype
/// \return memory atom with packed data
///
/// Implements primitive BLOCKTOLIST. It 'reads' packed
/// data from memory atom and returns a list of
/// unpacked data.
//===================================================
atom_t RUNTIME rt_blocktolist( int static_link, atom_t parent, atom_t prototype, atom_t data )
{
  void* ptr;
  if( data==empty_list )
  {

    RETURN( new_error( ERROR_NOT_A_MEM, data ) );
  }
  
  if( IS_MEM(data) )
    ptr = MEMORY( data );
  else
  {
    int64_t addr;
    GET_INT( data, addr );
    ptr = (void*)(int)addr;
  }

  atom_t res = traverse_pack( static_link, parent, prototype, empty_list, ptr, MEM_STRUCT_UNPACK );
  return res;
}



//===================================================
/// \fn		atom_t rt_dataaddr( atom_t data )
/// \brief	implementation of DATAADDR
/// \param  data  atom which address is returned
/// \return the address of an atom
///
/// Implements primitive DATAADDR. Returns an integer
/// atom which is the address of the \c data atom.
//===================================================

atom_t RUNTIME rt_dataaddr( atom_t data )
{
  RETURN( new_integer((int)data) );
}




//===================================================
/// \fn		atom_t rt_listintoblock( int static_link, atom_t parent, atom_t prototype, atom_t dest, atom_t data )
/// \brief	implementation of PACKTO
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  dest        destination for packed data
/// \param  prototype	pack prototype
/// \param  data	data to pack
/// \return unbound or error atom
///
/// Implements primitive PACKTO. Packs data into
/// address specified by \c dest which must be either
/// a memory atom or an address (i.e. integer atom).
//===================================================
atom_t RUNTIME rt_listintoblock( int static_link, atom_t parent, atom_t prototype, atom_t dest, atom_t data )
{
  void* ptr;
  if( IS_MEM(dest) )
  ptr = MEMORY( dest );
  else
  {
    int64_t addr;
    GET_INT( dest, addr );
    ptr = (void*)(int)addr;
  }

  atom_t res = traverse_pack( static_link, parent, prototype, data, ptr, MEM_STRUCT_PACK );
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_funcaddr( int static_link, atom_t parent, atom_t data )
/// \brief	implementation of FUNVADDR
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  data	function name
/// \return the address of a function with given name
///
/// Implements primitive FUNCADDR. Returns the address of
/// the function which name is the value of \c data.
/// If there is no function, then return error atom.
//===================================================
atom_t RUNTIME rt_funcaddr( int static_link, atom_t parent, atom_t data )
{
  if (IS_ERROR( data )) RETURN( USE(data) );

  atom_t word = atom_to_word( data );
  if (IS_ERROR( word )) RETURN( word );

  atom_t var = find_runtime_var( word, static_link );
  DEUSE( word );

  // not found
  if (!var)
    RETURN( new_error( ERROR_UNKNOWN_VAR, data ) );

  // not a variable (i.e. it is a function or a procedure)
  if (IS_VARIABLE( var ) || IS_PRIMITIVE( var ))
    RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, data ) );

  RETURN( new_integer(ADDRESS(var)) );
}




//===================================================
/// \fn		atom_t rt_external (int static_link, atom_t parent, atom_t handle, atom_t prototype, atom_t name)
/// \brief	implementation of EXTERNAL
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  handle      handle of library
/// \param  prototype	external function prototype
/// \param  name        function to externalize
/// \return unbound or error atom
///
/// Implements primitive EXTERNAL. Creates a trampoline
/// code which prepares the stack by converting atoms
/// into C data types.
//===================================================
atom_t RUNTIME rt_external (int static_link, atom_t parent, atom_t handle, atom_t prototype, atom_t name)
{
  //printf(" handle = "); dumpln(handle);
  //printf(" prototype = "); dumpln(prototype);
  //printf(" name = "); dumpln(name);
///atom_t ppp = prototype;
//printf("enter p="); dumpln(ppp);

  if( IS_ERROR(handle) ) RETURN( USE(handle) );
  if( IS_ERROR(prototype) ) RETURN( USE(prototype) );
  if( IS_ERROR(name) ) RETURN( USE(name) );

  int hnd = 0;
  if( !atom_to_integer(handle,&hnd) )
  RETURN( new_error(ERROR_NOT_AN_INTEGER,handle) );

  // check whether the Logo function exists
  atom_t word = atom_to_word( name );
  if( IS_ERROR(word) ) RETURN( word );
  atom_t var = find_runtime_var( word, static_link );
  DEUSE( word );
  if( !var ) RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, name ) );
  if( !IS_NORMAL(var) ) RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, name ) );
  if( !IS_FUNCTION(var) && !IS_COMMAND(var) ) RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, name ) );

  // check prototype
  if( !IS_LIST(prototype) ) RETURN( new_error( ERROR_NOT_A_LIST, prototype ) );
  if( list_length(prototype) < 2 ) RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

  // process return type
  atom_t type = CAR( prototype );
  prototype = CDR( prototype );

  int c_type = get_c_type( static_link, parent, type );
  int class = c_types[c_type].class;

  // return type cannot be structure or unknown. Structures are usually
  // returned as pointer to structures.
  if( class==C_TYPE_STRUCT || class==C_TYPE_UNKNOWN )
  RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

  VARTYPE( var ) = VAR_TYPE_EXTERNAL+c_type;

  // if the return type is void, then we have external
  // command, otherwise we have external function
  CLR_FLAGS( var, FLAG_COMMAND||FLAG_FUNCTION );
  SET_FLAGS( var, (class==C_TYPE_VOID)?FLAG_COMMAND:FLAG_FUNCTION );

  // get external name
  atom_t external_name = CAR( prototype );
  prototype = CDR( prototype );
  if( !IS_ANY_WORD(external_name) )
  RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );
  //printf("external name = "); dumpln(external_name);

  // check the correct order of parameters
  if( list_length(prototype) != RARGS(var) )
  RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

  // now process parameters one-by-one
  atom_t params;
  int offset = BASE_OFFSET_PARAMS+(RARGS(var)-1)*sizeof(atom_t);
  for( ; IS_NOT_EMPTY(prototype); prototype=CDR(prototype) )
  {
    // get one type from the prototype
    type = CAR( prototype );
    c_type = get_c_type( static_link, parent, type );
    class = c_types[c_type].class;

    if( class==C_TYPE_STRUCT || class==C_TYPE_UNKNOWN )
    RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

    // now find parameter with given offset
    for( params=LOCALS(var); IS_NOT_EMPTY(params); params=CDR(params))
    {
      atom_t param = CAR( params );
      if( !IS_VARIABLE(param) ) continue; // functions/commands not allowerd
      if( !IS_NORMAL(param) )   continue; // tags/runtimes not allowed
      if( OFFSET(param)==offset )
      {
        VARTYPE( param ) = VAR_TYPE_EXTERNAL+c_type;
        //printf(" :::param "); dump(NAME(param)); printf(" is class=%d name=%S\n", c_type, c_types[c_type].name);
        break;
      }
    }

    offset -= sizeof( atom_t );
  }

#ifdef SAFE_MODE
  for( params=LOCALS(var); IS_NOT_EMPTY(params); params=CDR(params))
  assert( IS_EXTERNAL(CAR(params)) );
#endif

  //printf("search function "); dump(external_name); printf(" in handle "); dumpln(handle);

  // convert char_t* into char*
  int len = LENGTH( external_name );
  chars_t ptr = STRING( external_name );
  char func_name[len + 1];
  int i;
  for (i = 0; i < len; i++) func_name[i] = *(ptr++);
  func_name[len] = '\0';

  void* address = dlsym( (void*)hnd, func_name );

  if( !address )
  RETURN( new_error( ERROR_NOT_A_FUNCTION, external_name ) );

  //printf("external address=%x\n",(int)address);

  ADDRESS( var ) = (int)address;
  compile_external_function( var );
//printf("exit  p="); dumpln(ppp);
  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_internal (int static_link, atom_t parent, atom_t prototype, atom_t name)
/// \brief	implementation of INTERNAL
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  prototype	internal function prototype
/// \param  name        function to internalize
/// \return unbound or error atom
///
/// Implements primitive INTERNAL. Creates a trampoline
/// code which prepares the stack by converting
/// C data types into atoms.
//===================================================
atom_t RUNTIME rt_internal (int static_link, atom_t parent, atom_t prototype, atom_t name)
{
  //printf(" prototype = "); dumpln(prototype);
  //printf(" name = "); dumpln(name);

  if( IS_ERROR(prototype) ) RETURN( USE(prototype) );
  if( IS_ERROR(name) ) RETURN( USE(name) );

  // check whether the Logo function exists
  atom_t word = atom_to_word( name );
  if( IS_ERROR(word) ) RETURN( word );
  atom_t var = find_runtime_var( word, static_link );
  DEUSE( word );
  if( !var ) RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, name ) );
  if( !IS_NORMAL(var) ) RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, name ) );
  if( !IS_FUNCTION(var) && !IS_COMMAND(var) ) RETURN( new_error( ERROR_NOT_A_USER_FUNCTION, name ) );

  // check prototype
  if( !IS_LIST(prototype) ) RETURN( new_error( ERROR_NOT_A_LIST, prototype ) );
  if( list_length(prototype) < 1 ) RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

  // process return type
  atom_t type = CAR( prototype );
  prototype = CDR( prototype );

  int c_type = get_c_type( static_link, parent, type );
  int class = c_types[c_type].class;

  // return type cannot be structure or unknown. Structures are usually
  // returned as pointer to structures.
  if( class==C_TYPE_STRUCT || class==C_TYPE_UNKNOWN )
  RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );
  VARTYPE( var ) = VAR_TYPE_INTERNAL+c_type;

  // if the return type is void, then we have internal
  // command, otherwise we have internal function
  CLR_FLAGS( var, FLAG_COMMAND||FLAG_FUNCTION );
  SET_FLAGS( var, (class==C_TYPE_VOID)?FLAG_COMMAND:FLAG_FUNCTION );

  // check the correct order of parameters
  if( list_length(prototype) != RARGS(var) )
  RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

  // now process parameters one-by-one
  atom_t params;
  int offset = BASE_OFFSET_PARAMS+(RARGS(var)-1)*sizeof(atom_t);
  for( ; IS_NOT_EMPTY(prototype); prototype=CDR(prototype) )
  {
    // get one type from the prototype
    type = CAR( prototype );
    c_type = get_c_type( static_link, parent, type );
    class = c_types[c_type].class;

    if( class==C_TYPE_STRUCT || class==C_TYPE_UNKNOWN )
    RETURN( new_error( ERROR_BAD_PROTOTYPE, prototype ) );

    // now find parameter with given offset
    for( params=LOCALS(var); IS_NOT_EMPTY(params); params=CDR(params))
    {
      atom_t param = CAR( params );
      if( !IS_VARIABLE(param) ) continue; // functions/commands not allowerd
      if( !IS_NORMAL(param) )   continue; // tags/runtimes not allowed
      if( OFFSET(param)==offset )
      {
        VARTYPE( param ) = VAR_TYPE_INTERNAL+c_type;
        //printf(" :::param "); dump(NAME(param)); printf(" is class=%d name=%S offset=%d\n", c_type, c_types[c_type].name,offset);
        break;
      }
    }

    offset -= sizeof( atom_t );
  }

#ifdef SAFE_MODE
  for( params=LOCALS(var); IS_NOT_EMPTY(params); params=CDR(params))
  assert( IS_INTERNAL(CAR(params)) );
#endif

  compile_internal_function( var, static_link );
  RETURN( unbound );
}




//===================================================
/// \fn		atom_t rt_stackframe (int static_link, atom_t parent, atom_t offset, atom_t frame)
/// \brief	implementation of _STACKFRAME
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  offset	offset in the stack frame
/// \param  frame	stack frame number
/// \return value at offset of a stack frame or error atom
///
/// Implements primitive _STACKFRAME. Goes to stack frame
/// number FRAME (0 - current frame, 1 - parent frame, etc.)
/// and returns the value at given OFFSET relative to
/// the stack frame. OFFSET is given in term of words.
//===================================================
atom_t RUNTIME rt_stackframe (int static_link, atom_t parent, atom_t offset, atom_t frame)
{
  int64_t ofs, frm;
  GET_INT( offset, ofs );
  GET_INT( frame, frm );

  int base = (frm<0)?BASE_OFFSET_DYNAMIC:BASE_OFFSET_STATIC;

  frm = abs(frm);
  for (; frm; frm--)
  static_link = *(int*) ((char*) static_link + base);

  RETURN( new_integer( *(int*) ((char*) static_link + (ofs*sizeof(atom_t))) ) );
}



//===================================================
/// \fn		atom_t rt_stackframeatom (int static_link, atom_t parent, atom_t offset, atom_t frame)
/// \brief	implementation of _STACKFRAMEATOM
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  offset	offset in the stack frame
/// \param  frame	stack frame number
/// \return atom at offset of a stack frame or error atom
///
/// Implements primitive _STACKFRAMEATOM. Goes to stack frame
/// number FRAME (0 - current frame, 1 - parent frame, etc.)
/// and returns the value at given OFFSET relative to
/// the stack frame. OFFSET is given in term of words.
/// The value is assumed to be an atom.
//===================================================
atom_t RUNTIME rt_stackframeatom (int static_link, atom_t parent, atom_t offset, atom_t frame)
{
  int64_t ofs, frm;
  GET_INT( offset, ofs );
  GET_INT( frame, frm );

  int base = (frm<0)?BASE_OFFSET_DYNAMIC:BASE_OFFSET_STATIC;

  frm = abs(frm);
  for (; frm; frm--)
  static_link = *(int*) ((char*) static_link + base);

  RETURN( USE(*(atom_t*) ((char*) static_link + (ofs*sizeof(atom_t))) ) );
}




//===================================================
/// \fn		atom_t rt_int3( )
/// \brief	dummy implementation of debug command _INT3
///
/// This definition just reserves an address for
/// rt_int3, so that compile_function() can easily
/// detect it.
//===================================================

atom_t RUNTIME rt_int3( )
{
  RETURN( unbound );
}


//===================================================
/// \fn		atom_t rt_load( atom_t data )
/// \brief	implementation of LOAD
/// \param data name of file to load
/// \return var atom containing the compiled code
///
/// Implements primitive command LOAD. Creates a local
/// function with the given body, compiles it, and
//  return its var atom.
///
/// It is supposed that the caller of rt_run() should
/// use the result to do the actual call of the
/// newly compiled code.
///
/// If \c mode is ::COMPILE_AS_PROC then the compiled
/// code is a list of commands, otherwise it is a
/// single expression.
//===================================================

atom_t RUNTIME rt_load( atom_t data )
{
  RETURN( unbound );
}



//===================================================
/// \fn		atom_t rt_commandline( )
/// \brief	implementation of COMMANDLINE
/// \return list atom containing command line
///
/// Implements primitive function COMMANDLINE. Returns
/// a list containing the command-line arguments.
//===================================================

atom_t RUNTIME rt_commandline( )
{
  RETURN( USE( standalone_arguments ) );
}



//===================================================
/// \fn		atom_t rt_openfile_mode( atom_t filename, char* mode, int call_mode )
/// \brief	used by OPEN* functions
/// \param mode open mode (read, write, ...)
/// \param filename name of binary file
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return var atom containing the file handle or error atom
///
/// Opens a file stream with given \c filename and
/// \c mode.  The filename is in an atom, while
/// the mode is in a string.
//===================================================

atom_t RUNTIME rt_openfile_mode( atom_t filename, char* mode, int call_mode )
{
  CHECK_PARAM( filename );

  // find a slot for the new file
  int i = find_file_by_handle( NULL );
  #ifdef SAFEMODE
    assert( i>-1 );
  #endif

  atom_t fn = atom_to_real_word( filename );
  if( IS_ERROR(fn) ) RETURN(fn);
  
  file_names[i]   = FILENAME(STRING(fn));
  file_handles[i] = fopen( file_names[i], mode );
  if( errno )
  {
    DEUSE( fn );
    DEALLOC( file_names[i] );
    file_names[i]   = NULL;
    file_handles[i] = NULL;

    last_os_error = errno;
    RETURN( new_error( ERROR_OS_ERROR, filename ) );
  }
  
  DEUSE( fn );

  if( call_mode==COMPILE_AS_FUNC )
  {
    RETURN( new_integer( (int)file_handles[i] ) );
  }
  else
  {
    RETURN( unbound );
  }
};




//===================================================
/// \fn		atom_t rt_openfile( atom_t mode, atom_t filename, int call_mode )
/// \brief	implementation of OPENFILE
/// \param mode open mode (read, write, ...)
/// \param filename name of binary file
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return var atom containing the file handle or error atom
///
/// Implements primitive command OPENFILE. Opens a binary
/// file with given \c filename and \c mode.  Files opened
/// with PACKOPEN should be processed with other PACK-aware
/// functions and commands.
//===================================================

atom_t RUNTIME rt_openfile  (atom_t mode, atom_t filename, int call_mode)
{
  if( IS_ERROR(mode) ) RETURN(USE(mode));

  atom_t rw = atom_to_real_word( mode );
  if( IS_ERROR(rw) ) { RETURN(rw); }

  char* ascii_rw = FILENAME(STRING(rw));

  DEUSE( rw );

  atom_t res = rt_openfile_mode( filename, ascii_rw, call_mode );

  DEALLOC( ascii_rw );

  RETURN( res );
};



//===================================================
/// \fn		atom_t get_file_index( atom_t file, int* index )
/// \brief	get file index
/// \param file file handle or file name
/// \param index variable to store the index
/// \return unbound or error atom
///
/// Searches for an opened file assuming that \c file
/// is a file handle. If it cannot be converted to a
/// number, assumes it is a file name. Returns error
/// atom (if getting the handle or the filename failed)
/// or unbound atom if OK. In the latter case \c index
/// contains the index of the file. If index=-1, then
/// the file is not opened so far.
///
/// If error atom is returns, it is already USEed,
/// so the caller should not reUSE it.
//===================================================
atom_t RUNTIME get_file_index( atom_t file, int* index )
{
  *index = -1;
  if( IS_ERROR(file) ) RETURN(USE(file));
  
  int64_t handle;
  if( atom_to_int( file, &handle ) )
  { // the filename is a number
    *index = find_file_by_handle( (FILE*)(int)handle );
  }
  
  if( *index<0 )
  { // the filename may be a string
    atom_t fn = atom_to_real_word( file );
    if( IS_ERROR(fn) ) {DEUSE(fn); RETURN(USE(file))};
    char* ch = FILENAME(STRING(fn));
    *index = find_file_by_filename( ch );
    DEALLOC( ch );
    DEUSE( fn );
  }

  //is file opened?
  if( *index<0 )
  {
    RETURN( new_error( ERROR_FILE_NOT_OPENED, file ) );
  }
  
  RETURN( unbound );
};



//===================================================
/// \fn		atom_t rt_closefile( atom_t file )
/// \brief	implementation of CLOSEFILE
/// \param file file to close
/// \return unbound atom
///
/// Implements primitive command CLOSEFILE. Closes a
/// file identified by either by its handle (if
/// \c file contains a number) or by its name 
/// otherwise.
//===================================================

atom_t RUNTIME rt_closefile (atom_t file)
{
  int index;
  atom_t res = get_file_index( file, &index );
  if( IS_ERROR(res) ) RETURN(res); // USE is done by get_file_index()

  FILE *h = file_handles[index];
  if( h==output_stream ) output_stream = NULL;
  if( h==input_stream )  input_stream = NULL;
  fclose( h );

  DEALLOC( file_names[index] );
  file_names[index]   = NULL;
  file_handles[index] = NULL;

  RETURN( unbound );
};


//===================================================
/// \fn		atom_t rt_readblock( int static_link, atom_t parent, atom_t size )
/// \brief	implementation of READBLOCK
/// \param  static_link	static link from the current frame
/// \param  parent	current parent
/// \param  size size of the block and number of bytes to read
/// \return memory or error atom or empty list
///
/// Implements primitive command READBLOCK. Reads
/// block of bytes from a file opened with OPENFILE and
/// set as reading file with SETREAD. The size of
/// the data being read is measured in bytes. The
/// read data is placed in a newly created memory
/// block.
///
/// Returns an empty list if reading failed because
/// of end of file, or if the reading is from the
/// standard input.
//===================================================

atom_t RUNTIME rt_readblock( int static_link, atom_t parent, atom_t size )
{
  int64_t datasize; // data size
  atom_t sizeatom;
  
  CHECK_PARAM( size );

  if( input_stream==NULL )
    RETURN( empty_list );
  
  // try to get the block size
  if( IS_LIST( size ) )
  { // size is a prototype
    sizeatom = rt_blocksize( static_link, parent, size );
    if( IS_ERROR(sizeatom) )
    {
      RETURN( sizeatom );
    }
    datasize = INTEGER(sizeatom);
  }
  else
  { // size must be an integer
    if (!atom_to_int( size, &datasize ))
    {
      RETURN( new_error( ERROR_NOT_AN_INTEGER, size ) );
    }
    sizeatom = USE(size);
  }
  
  atom_t data = new_mem( datasize );

  if( !fread( MEMORY(data), datasize, 1, input_stream ) )
  {
    DEUSE( data );
    DEUSE( sizeatom );
    if( errno )
    {
      errno = EIO;
      last_os_error = errno;
      return new_error( ERROR_OS_ERROR, size );
    }
    else
    {
      data = empty_list;
    }
  }
  else
  {
    DEUSE( ATOMS(data) );
    ATOMS(data) = sizeatom;
  }
  RETURN( data );
};



//===================================================
/// \fn		atom_t rt_readinblock( atom_t block, int call_mode )
/// \brief	implementation of READINBLOCK
/// \param  block block to read data to
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return memory or error atom or empty list
///
/// Implements primitive command READINBLOCK. Reads
/// block of bytes from a file opened with OPENFILE and
/// set as reading file with SETREAD. The size of
/// the data is taken from the memory block where
/// the data is read.
///
/// Returns an empty list if reading failed because
/// of end of file, or if the reading is from the
/// standard input.
//===================================================

atom_t RUNTIME rt_readinblock( atom_t block, int call_mode )
{
  CHECK_PARAM( block );

  if( input_stream==NULL )
    RETURN( empty_list );
  
  // try to get the block size
  if( !IS_MEM(block) || !IS_INTEGER(ATOMS(block)) )
  {
    RETURN( new_error(ERROR_NOT_BLOCK_OR_DEF,block) );
  }

  if( !fread( MEMORY(block), INTEGER(ATOMS(block)), 1, input_stream ) )
  {
    if( errno )
    {
      errno = EIO;
      last_os_error = errno;
      return new_error( ERROR_OS_ERROR, block );
    }
    else
    {
      if( call_mode==COMPILE_AS_FUNC )
        { RETURN( empty_list ); }
      else
        { RETURN( unbound ); }
    }
  }

  if( call_mode==COMPILE_AS_FUNC )
    { RETURN( USE(block) ); }
  else
    { RETURN( unbound ); }
};



//===================================================
/// \fn		atom_t rt_writeblock( atom_t data )
/// \brief	implementation of WRITEBLOCK
/// \param data packed data to be written
/// \return unbound or error atom
///
/// Implements primitive command WRITEBLOCK. Writes
/// a block of bytes to a file opened with OPENFILE and
/// set for writing with SETWRITE. The size of the
/// data being written is taken from the memory
/// block containing the packed data.
///
/// Returns an empty list if the output file is the
/// standard output.
//===================================================

atom_t RUNTIME rt_writeblock( atom_t data )
{
  int64_t size; // data size

  if( IS_ERROR(data) ) RETURN(USE(data));
  
  if( output_stream==NULL )
    RETURN( unbound );

  if( !IS_MEM(data) )
  {
    RETURN( new_error( ERROR_NOT_A_MEM, data ) );
  }
  if (!atom_to_int( ATOMS(data), &size ))
  {
    RETURN( new_error( ERROR_NOT_AN_INTEGER, ATOMS(data) ) );
  }

  fwrite( MEMORY(data), size, 1, output_stream );
  if( errno )
  {
    last_os_error = errno;
    RETURN( new_error( ERROR_OS_ERROR, data ) );
  }

  RETURN( unbound );
};



//===================================================
/// \fn		atom_t rt_readchar( )
/// \brief	implementation of primitive function readchar
/// \return Number atom - read char as a word
///
/// Implements the primitive function \c readchar.
/// Reads a single character from the read stream and
/// outputs that character as a word.
//===================================================

atom_t RUNTIME rt_readchar( void )
{
  char_t ch = inner();
  if( ch==NO_MORE )
  {
    RETURN( empty_list );
  }
  else
  {
    RETURN( new_word( &ch, 1 ) );
  }
}



//===================================================
/// \fn		atom_t rt_readchars( atom_t data)
/// \brief	implementation of primitive function readchars
/// \param data number of characters to read
/// \return Number atom - read chars as a word
///
/// Implements the primitive function \c readchars.
/// Reads a given number of characters from the read
/// stream and outputs them as a word.
//===================================================

atom_t RUNTIME rt_readchars( atom_t data )
{
  atom_t res;
  int64_t count;
  chars_t chs;

  // parameters check
  if (IS_ERROR( data )) RETURN( USE(data) );
  GET_INT( data, count );

  if (count <= 0)
  {
    RETURN( new_error( ERROR_NOT_A_NUMBER, data ) );
  }

  // Allocate memory
  res = create_word( count+1 ); // +1 is for the terminating 0
  chs = STRING(res);
  while( count )
  {
    *chs = (char_t)inner();
    chs++;
    count--;
  }
  *chs = TEXT('\0');

  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_readrawline( )
/// \brief	implementation of primitive function readrawline
/// \return Number atom - read line as a word
///
/// Implements the primitive function \c readrawline.
/// Reads a single line from the read stream and
/// outputs that line as a word.
//===================================================

atom_t RUNTIME rt_readrawline( void )
{
  chars_t buffer = 0;		// buffer to hold the text
  int bufspace = 0;		// number of free positions in the buffer
  int bufsize = 0;		// current buffer size
  int bufstep = 16;		// step size for buffer reallocations
  int buflen = 0;
  atom_t res;

  char_t ch = inner();
  if( ch==NO_MORE )
  {
    return( empty_list );
  }

  while( (ch!=TEXT('\n')) && (ch!=NO_MORE) )
  {
    if( !bufspace )
    {
      buffer = REALLOC( buffer, (bufsize+bufstep)*sizeof(char_t) );
      bufspace = bufstep;
      bufsize = bufsize+bufstep;
      bufstep = bufstep+1;
    }

    *(buffer+buflen) = ch;
    buflen++;
    bufspace--;

    ch = inner();
  }

  if( !buffer ) return( empty_list );
  res = new_word( buffer, buflen );
  DEALLOC( buffer );
  return res;
}



//===================================================
/// \fn		atom_t rt_readword( )
/// \brief	implementation of primitive function readword
/// \return Number atom - read line as a word
///
/// Implements the primitive function \c readword.
/// Reads a single line from the read stream and
/// outputs that line as a word. Processes backslashes,
/// vertical bars.
//===================================================

atom_t RUNTIME rt_readword( void )
{
  atom_t word = rt_readrawline();
  if( IS_ERROR(word) ) return word;
  if( IS_EMPTY(word) ) return word;

  atom_t parsed = tokenize( word, TOKENIZE_READWORD );
  DEUSE( word );
  return parsed;
}



//===================================================
/// \fn		atom_t rt_readlist( )
/// \brief	implementation of primitive function readlist
/// \return Number atom - read line as a list
///
/// Implements the primitive function \c readlist.
/// Reads a single line from the read stream and
/// outputs that line as a list. Processes all special
/// characters except semicolon ";".
//===================================================

atom_t RUNTIME rt_readlist( void )
{
  atom_t word = rt_readrawline();
  if( IS_ERROR(word) ) return word;
  if( IS_EMPTY(word) ) return new_word( TEXT(""), 0);

  atom_t parsed = tokenize( word, TOKENIZE_READLIST );
  DEUSE( word );
  return parsed;
}



//===================================================
/// \fn		atom_t rt_getenv( atom_t data )
/// \brief	implementation of GETENV
/// \param  data	environment variable name
/// \return the value of envronment variable
///
/// Implements primitive GETENV. Returns the value of
/// the environment variable which name is the value
/// of \c data. If there is no such variable, then
/// return empty list.
//===================================================

atom_t RUNTIME rt_getenv( atom_t data )
{
  atom_t res;

  if (IS_ERROR( data )) RETURN( USE(data) );

  atom_t word = atom_to_word( data );
  if (IS_ERROR( word )) RETURN( word );

  char* varname = UTF16_to_ASCII( STRING(word) );
  varname[LENGTH(word)] = '\0';
  //printf("varname=|%s| len=%d\n",varname,strlen(varname));
  char* asciivalue = getenv( varname );
  //printf("asciivalue=%s\n",asciivalue);
  if( asciivalue )
  {
    chars_t value = ASCII_to_UTF16( asciivalue );
    res = new_word( value, -1 );
    DEALLOC( value );
  }
  else
  {
    res = empty_list;
  }

  DEUSE( word );
  DEALLOC( varname );
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_getenvs( )
/// \brief	implementation of GETENVS
/// \return a list of all envronment variables
///
/// Implements primitive GETENVS. Returns a list of
/// all environment variables.
//===================================================

atom_t RUNTIME rt_getenvs( )
{
  atom_t res = empty_list;
  char oldv;

#ifdef environ
  char** env = environ;
#else
  char** env = __environ;
#endif

  while( (*env) )
  {
    char* var = *env;
    char* v=var;
    while( (*v!='=') && (*v!='\n') ) v++;
    oldv = *v;
    *v = '\0';
    chars_t uname = ASCII_to_UTF16( var );
    chars_t uvalue = ASCII_to_UTF16( v+1 );
    *v = oldv;

    atom_t pair = new_list( new_word(uvalue,-1), empty_list );
    pair = new_list( new_word(uname,-1), pair );
    res = new_list( pair, res );
    
    DEALLOC( uname );
    DEALLOC( uvalue );
    env++;
  }
  RETURN( res );
}


//===================================================
/// \fn		atom_t rt_eofp( )
/// \brief	implementation of primitive function eof?
/// \return true or false atom
///
/// Implements the primitive function \c eof?.
/// Outputs \c true if there are no more characters to be
/// read, or \c false otherwise.
//===================================================

atom_t RUNTIME rt_eofp( void )
{
  RETURN ( USE(false_true[inner_eof()]) );
}



//===================================================
/// \fn		atom_t rt_currentfolder( )
/// \brief	implementation of primitive function currentfolder
/// \return word atom
///
/// Implements the primitive function \c currentfolder.
/// Outputs a word atom containing the name of the current
/// folder.
//===================================================

atom_t RUNTIME rt_currentfolder( void )
{
  atom_t res;
  char path[PATH_MAX+1];

  char* p = getcwd( path, PATH_MAX );
  if( p )
  {
    chars_t pathwc = UNFILENAME( path );
    res = new_word(pathwc,-1);
    DEALLOC( pathwc);
  }
  else
  {
    res = new_os_error_atom( unbound );
  }
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_makefolder( atom_t name )
/// \brief	implementation of primitive function makefolder
/// \param  name	name of a folder to make
/// \return unbound or error atom
///
/// Implements the primitive function \c makefolder.
/// Create a folder with given name relative to the
/// current folder. Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_makefolder( atom_t name )
{
  atom_t res;

  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );
  int err;
  #ifdef WINDOWS
  err = mkdir( fname );
  #else
  err = mkdir( fname, (S_IRWXU|S_IRWXG|S_IRWXO) );
  #endif

  if( err )
  res = new_os_error_atom( name );
  else
  res = unbound;
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_erasefolder( atom_t name )
/// \brief	implementation of primitive function erasefolder
/// \param  name	name of a folder to erase
/// \return unbound or error atom
///
/// Implements the primitive function \c erasefolder.
/// Erases an empty folder with given name relative to the
/// current folder. Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_erasefolder( atom_t name )
{
  atom_t res;

  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );
  int err = rmdir( fname );

  if( err )
  res = new_os_error_atom( name );
  else
  res = unbound;
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_changefolder( atom_t name )
/// \brief	implementation of primitive function changefolder
/// \param  name	name of a folder to change to
/// \return unbound or error atom
///
/// Implements the primitive function \c changefolder.
/// Changes the current folder to a given folder.
/// Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_changefolder( atom_t name )
{
  atom_t res;

  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );
  int err = chdir( fname );

  if( err )
  res = new_os_error_atom( name );
  else
  res = unbound;
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_folderp( atom_t name )
/// \brief	implementation of primitive function folder?
/// \param  name	name of a folder to test
/// \return boolean or error atom
///
/// Implements the primitive function \c folder?.
/// Returns \c "true if a folder with the given name
/// exists. Otherwise return "false.
//===================================================

atom_t RUNTIME rt_folderp( atom_t name )
{
  atom_t res;

  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );

  struct stat buffer;
  int err = stat( fname, &buffer );

  if( err )
  res = USE(false_true[0]);
  else
  res = USE(false_true[ S_ISDIR(buffer.st_mode)?1:0 ]);
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_renamefolder_or_file( atom_t toname, atom_t fromname, int folders )
/// \brief	implementation of primitive function renamefolder
/// \param  toname	new name of a folder
/// \param  fromname	old name of a folder
/// \param  folders if !=0, rename a folder, otherwise a file
/// \return unbound or error atom
///
/// Renames a folder or a file given its old and new names.
/// Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_renamefolder_or_file( atom_t toname, atom_t fromname, int folders )
{
  atom_t res;

  if (IS_ERROR( fromname )) RETURN( USE(fromname) );
  if (IS_ERROR( toname )) RETURN( USE(toname) );
  
  atom_t fword = atom_to_real_word( fromname );
  atom_t tword = atom_to_real_word( toname );
  
  char* fname = FILENAME( STRING(fword) );
  char* tname = FILENAME( STRING(tword) );

  struct stat buffer;
  int err = stat( fname, &buffer );
  
  int ok;
  if( folders )
    ok = S_ISDIR(buffer.st_mode);
  else
    ok = S_ISREG(buffer.st_mode);
  
  if( ok )
    err = rename( fname, tname );

  if( err )
    res = new_os_error_atom( fromname );
  else
    res = unbound;
  
  DEUSE( fword );
  DEUSE( tword );
  DEALLOC( fname );
  DEALLOC( tname );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_renamefolder( atom_t toname, atom_t fromname )
/// \brief	implementation of primitive function renamefolder
/// \param  toname	new name of a folder
/// \param  fromname	old name of a folder
/// \return unbound or error atom
///
/// Implements the primitive function \c renamefolder.
/// Renames a folder give its old and new names.
/// Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_renamefolder( atom_t toname, atom_t fromname )
{
  atom_t res;

  res = rt_renamefolder_or_file( toname, fromname, 1 );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_renamefile( atom_t toname, atom_t fromname )
/// \brief	implementation of primitive function renamefile
/// \param  toname	new name of a file
/// \param  fromname	old name of a file
/// \return unbound or error atom
///
/// Implements the primitive function \c renamefile.
/// Renames a file given its old and new names.
/// Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_renamefile( atom_t toname, atom_t fromname )
{
  atom_t res;

  res = rt_renamefolder_or_file( toname, fromname, 0 );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_folders_or_files( atom_t name, int folders )
/// \brief	scans a folder
/// \param  name	name of a folder to scan
/// \param  folders if !=0, scans for folders, otherwise for files
/// \return list or error atom
///
/// Returns a list of folders' names (if folders!=0)
/// of files' name (if folders==0) in a given folder.
//===================================================

atom_t RUNTIME rt_folders_or_files( atom_t name, int folders )
{
  // assume the result is empty list
  atom_t res = empty_list;

  // convert the folder name into ASCII
  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  char* fname = FILENAME( STRING(word) );

  // verify that this is really a folder
  struct stat buffer;
  int err = stat( fname, &buffer );
  if( !err && S_ISDIR(buffer.st_mode) )
  {
    // open the folder
    DIR *dp;
    struct dirent *de;
    
    dp = opendir( fname );
    if( dp )
    {
      char long_name[PATH_MAX];
      int len = strlen( fname );
      strcpy( long_name, fname );
      long_name[len] = '/';
      len++;
      
      // scan the files one by one
      atom_t last = empty_list;
      de = readdir( dp );
      while( de )
      {
        struct stat buffer;
        strcpy( long_name+len, de->d_name );
        int err = stat(long_name,&buffer);
        if( err==0 )
        {
          int ok;
          if( folders )
          ok = S_ISDIR(buffer.st_mode);
          else
          ok = S_ISREG(buffer.st_mode);
          if( ok )
          {
            chars_t buf = UNFILENAME(de->d_name);
            atom_t word = new_word( buf, -1 );
            append( word, &res, &last );
            DEALLOC( buf );
          }
        }
        de = readdir( dp );
      }
      closedir( dp );
    }
  }
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_folders( atom_t name )
/// \brief	implementation of primitive function folders
/// \param  name	name of a folder to list
/// \return list or error atom
///
/// Implements the primitive function \c folders.
/// Returns a list of folders' names in a given folder.
//===================================================

atom_t RUNTIME rt_folders( atom_t name )
{
  atom_t res = rt_folders_or_files( name, 1 );
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_files( atom_t name )
/// \brief	implementation of primitive function files
/// \param  name	name of a folder to list
/// \return list or error atom
///
/// Implements the primitive function \c files.
/// Returns a list of files' names in a given folder.
//===================================================

atom_t RUNTIME rt_files( atom_t name )
{
  atom_t res = rt_folders_or_files( name, 0 );
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_erasefile( atom_t name )
/// \brief	implementation of primitive function erasefile
/// \param  name	name of a file to erase
/// \return unbound or error atom
///
/// Implements the primitive function \c erasefile.
/// Erases a file with given name relative to the
/// current folder. Outputs unbound or error atom.
//===================================================

atom_t RUNTIME rt_erasefile( atom_t name )
{
  atom_t res;

  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );
  int err = unlink( fname );

  if( err )
  res = new_os_error_atom( name );
  else
  res = unbound;
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_filep( atom_t name )
/// \brief	implementation of primitive function file?
/// \param  name	name/path of a file to test
/// \return boolean or error atom
///
/// Implements the primitive function \c file?.
/// Returns \c "true if a file with the given name
/// or path exists. Otherwise return "false.
//===================================================

atom_t RUNTIME rt_filep( atom_t name )
{
  atom_t res;

  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );

  struct stat buffer;
  int err = stat( fname, &buffer );

  if( err )
	res = USE(false_true[0]);
  else
	res = USE(false_true[ S_ISREG(buffer.st_mode)?1:0 ]);
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_filesize( atom_t name )
/// \brief	implementation of primitive function filesize
/// \param  name	name of a file
/// \return integer or error atom
///
/// Implements the primitive function \c filesize.
/// Returns size of the file in bytes if the file
/// exists. Otherwise returns -1.
//===================================================

atom_t RUNTIME rt_filesize( atom_t name )
{
  atom_t res;
  
  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );

  struct stat buffer;
  int err = stat( fname, &buffer );

  if( err )
  res = new_integer( -1 );
  else
  res = new_integer( buffer.st_size );
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_filetimes( atom_t name )
/// \brief	implementation of primitive function filetimes
/// \param  name	name of a file
/// \return list or error atom
///
/// Implements the primitive function \c filetimes.
/// Returns a list of three times (each represented
/// as a number): [creation modification access]
/// If the file does not exist or cannot be accessed
/// then returns an empty list.
//===================================================

atom_t RUNTIME rt_filetimes( atom_t name )
{
  atom_t res;
  
  if (IS_ERROR( name )) RETURN( USE(name) );
  atom_t word = atom_to_real_word( name );
  
  char* fname = FILENAME( STRING(word) );

  struct stat buffer;
  int err = stat( fname, &buffer );

  res = empty_list;
  if( !err )
  {
    res = new_list( new_integer( buffer.st_atime ), res );
    res = new_list( new_integer( buffer.st_mtime ), res );
    res = new_list( new_integer( buffer.st_ctime ), res );
  }
  
  DEUSE( word );
  DEALLOC( fname );
  
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_openread( atom_t name, int call_mode )
/// \brief	implementation of primitive function \c openread
/// \param  name	name of a file
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return file handle or error atom
///
/// Implements the primitive function \c openread.
/// Returns an integer atom with the file handle
/// or an error atom.
//===================================================

atom_t RUNTIME rt_openread( atom_t name, int call_mode )
{
  atom_t res = rt_openfile_mode( name, "r", call_mode );
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_openwrite( atom_t name, int call_mode )
/// \brief	implementation of primitive function \c openwrite
/// \param  name	name of a file
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return file handle or error atom
///
/// Implements the primitive function \c openwrite.
/// Returns an integer atom with the file handle
/// or an error atom.
//===================================================

atom_t RUNTIME rt_openwrite( atom_t name, int call_mode )
{
  atom_t res = rt_openfile_mode( name, "w", call_mode );
  RETURN( res );
}





//===================================================
/// \fn		atom_t rt_openappend( atom_t name, int call_mode )
/// \brief	implementation of primitive function \c openappend
/// \param  name	name of a file
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return file handle or error atom
///
/// Implements the primitive function \c openappend.
/// Returns an integer atom with the file handle
/// or an error atom.
//===================================================

atom_t RUNTIME rt_openappend( atom_t name, int call_mode )
{
  atom_t res = rt_openfile_mode( name, "a", call_mode );
  RETURN( res );
}




//===================================================
/// \fn		atom_t rt_openupdate( atom_t name, int call_mode )
/// \brief	implementation of primitive function \c openupdate
/// \param  name	name of a file
/// \param  call_mode	COMPILE_AS_FUNC or COMPILE_AS_PROC
/// \return file handle or error atom
///
/// Implements the primitive function \c o/c/te.
/// Returns an integer atom with the file handle
/// or an error atom.
//===================================================

atom_t RUNTIME rt_openupdate( atom_t name, int call_mode )
{
  atom_t res = rt_openfile_mode( name, "r+", call_mode );
  RETURN( res );
}



//===================================================
/// \fn		atom_t rt_setread( atom_t file )
/// \brief	implementation of SETREAD
/// \param  file file to set as a reader
/// \return unbound or error atom
///
/// Implements primitive command SETREAD. Sets a given
/// file handle as a reader file (i.e. input stream).
/// If \c file is an empty list, then reset the
/// input stream to the default one (usually stdin).
//===================================================
atom_t RUNTIME rt_setread( atom_t file )
{
  // empty list resets the reader
  if( file==empty_list )
  {
    input_stream = NULL;
  }
  else
  {
    int index;
    atom_t res = get_file_index( file, &index );
    if( IS_ERROR(res) ) RETURN(res); // USE is done by get_file_index()
  
    input_stream = file_handles[index];
  }

  RETURN( unbound );
};




//===================================================
/// \fn		atom_t rt_setwrite( atom_t file )
/// \brief	implementation of SETWRITE
/// \param  file file to set as a writer
/// \return unbound or error atom
///
/// Implements primitive command SETWRITE. Sets a given
/// file handle as a writer file (i.e. output stream).
/// If \c file is an empty list, then reset the
/// output stream to the default one (usually stdout).
//===================================================
atom_t RUNTIME rt_setwrite( atom_t file )
{
  // empty list resets the reader
  if( file==empty_list )
  {
    output_stream = NULL;
  }
  else
  {
    int index;
    atom_t res = get_file_index( file, &index );
    if( IS_ERROR(res) ) RETURN(res); // USE is done by get_file_index()
  
    output_stream = file_handles[index];
  }
  
  RETURN( unbound );
};




//===================================================
/// \fn		atom_t rt_reader( )
/// \brief	implementation of READER
/// \return empty list or word atom
///
/// Implements primitive command READER. Returns the
/// name of the current input stream or an empty
/// list if it is stdin.
//===================================================
atom_t RUNTIME rt_reader( )
{
  atom_t res;
  
  if( input_stream==NULL )
  {
    res = empty_list;
  }
  else
  {
    int i = find_file_by_handle( input_stream );
    #ifdef SAFEMODE
      assert( i>-1 );
    #endif
    chars_t name = ASCII_to_UTF16( file_names[i] );
    res = new_word( name, -1 );
    DEALLOC( name );
  }

  RETURN( res );
};



//===================================================
/// \fn		atom_t rt_writer( )
/// \brief	implementation of WRITER
/// \return empty list or word atom
///
/// Implements primitive command WRITER. Returns the
/// name of the current output stream or an empty
/// list if it is stdout.
//===================================================
atom_t RUNTIME rt_writer( )
{
  atom_t res;
  
  if( output_stream==NULL )
  {
    res = empty_list;
  }
  else
  {
    int i = find_file_by_handle( output_stream );
    #ifdef SAFEMODE
      assert( i>-1 );
    #endif
    chars_t name = ASCII_to_UTF16( file_names[i] );
    res = new_word( name, -1 );
    DEALLOC( name );
  }

  RETURN( res );
};



//===================================================
/// \fn		atom_t rt_allopen( )
/// \brief	implementation of ALLOPEN
/// \return empty list atom
///
/// Implements primitive command ALLOPEN. Returns a
/// list of the names of all opened files.
//===================================================
atom_t RUNTIME rt_allopen( )
{
  atom_t res = empty_list;
  atom_t last = empty_list;
  
  int i;
  for( i=0; i<FOPEN_MAX; i++ )
    if( file_handles[i] )
    {
      chars_t name = ASCII_to_UTF16( file_names[i] );
      atom_t word = new_word( name, -1 );
      DEALLOC( name );
     
      append( word, &res, &last );
    }

  RETURN( res );
};



//===================================================
/// \fn		atom_t rt_closeall( )
/// \brief	implementation of CLOSEALL
/// \return unbound atom
///
/// Implements primitive command CLOSEALL. Closes all
/// opened files, returns unbound atom.
//===================================================
atom_t RUNTIME rt_closeall( )
{
  int i;
  for( i=0; i<FOPEN_MAX; i++ )
    if( file_handles[i] )
    {
      DEALLOC( file_names[i] );
      fclose( file_handles[i] );
      file_names[i] = NULL;
      file_handles[i] = NULL;
    }

  RETURN( unbound );
};



//===================================================
/// \fn		atom_t rt_setreadpos( atom_t pos )
/// \brief	implementation of SETREADPOS
/// \param  pos file position for the reader
/// \return unbound or error atom
///
/// Implements primitive command SETREADPOS. Sets the
/// reading position of the reader (i.e. input stream).
/// Is pos>=0 the position is measured from the
/// beginning of the file, otherwise - from the end.
//===================================================
atom_t RUNTIME rt_setreadpos( atom_t pos )
{
  int64_t position;
  GET_INT( pos, position );
  if( position>=0 )
  {
    fseek( input_stream, position, SEEK_SET );
  }
  else
  {
    fseek( input_stream, position, SEEK_END );
  }

  RETURN( unbound );
};




//===================================================
/// \fn		atom_t rt_readpos( )
/// \brief	implementation of READPOS
/// \return integer or error atom
///
/// Implements primitive command READPOS. Returns the
/// reading position of the reader (i.e. input stream).
//===================================================
atom_t RUNTIME rt_readpos( )
{
  int pos;
  
  if( input_stream==NULL )
    pos = 0;
  else
    pos = ftell( input_stream );

  RETURN( new_integer(pos) );
};



//===================================================
/// \fn		atom_t rt_setwritepos( atom_t pos )
/// \brief	implementation of SETWRITEPOS
/// \param  pos file position for the writer
/// \return unbound or error atom
///
/// Implements primitive command SETWRITEPOS. Sets the
/// writing position of the writer (i.e. output stream).
/// Is pos>=0 the position is measured from the
/// beginning of the file, otherwise - from the end.
//===================================================
atom_t RUNTIME rt_setwritepos( atom_t pos )
{
  int64_t position;
  GET_INT( pos, position );
  if( position>=0 )
    fseek( output_stream, position, SEEK_SET );
  else
    fseek( output_stream, position, SEEK_END );

  RETURN( unbound );
};




//===================================================
/// \fn		atom_t rt_writepos( )
/// \brief	implementation of WRITEPOS
/// \return integer or error atom
///
/// Implements primitive command WRITEPOS. Returns the
/// writinging position of the writer (i.e. output stream).
//===================================================
atom_t RUNTIME rt_writepos( )
{
  int pos;

  if( output_stream==NULL )
    pos = 0;
  else
    pos = ftell( output_stream );

  RETURN( new_integer(pos) );
};



//===================================================
/// \fn		atom_t rt_timezone( )
/// \brief	implementation of TIMEZONE
/// \return integer atom
///
/// Implements primitive function TIEMZONE. Returns the
/// timezone difference with GMT in seconds.
//===================================================
atom_t RUNTIME rt_timezone( )
{
  RETURN( new_integer(-timezone) );
};



//===================================================
/// \fn		atom_t rt_nodribble( )
/// \brief	implementation of NODRIBBLE
/// \return unbound atom
///
/// Implements primitive command NODRIBBLE. Closes
/// the dribble file if it is opened.
//===================================================

atom_t RUNTIME rt_nodribble()
{
  if( dribble_handle )
  {
    fclose( dribble_handle );
  }
  dribble_handle = NULL;

  RETURN( unbound );
};



//===================================================
/// \fn		atom_t rt_dribble( atom_t filename )
/// \brief	implements prmitive DRIBBLE
/// \param filename name of dribble file
/// \return unbound or error atom
///
/// Implements the primitive command DRIBBLE. Opens
/// a dribble text file and returns unbound atom if
/// opening is successful. If there is already a
/// dribble file, first close it and then open a new
/// dribble file.
//===================================================
atom_t RUNTIME rt_dribble( atom_t file )
{
  CHECK_PARAM( file );

  atom_t fn = atom_to_real_word( file );
  if( IS_ERROR(fn) ) RETURN(fn);

  if( dribble_handle ) rt_nodribble();
  
  char* file_name = FILENAME(STRING(fn));
  dribble_handle = fopen( file_name, "w" );
  if( errno )
  {
    DEUSE( fn );
    DEALLOC( file_name );
    dribble_handle = NULL;

    last_os_error = errno;
    RETURN( new_error( ERROR_OS_ERROR, file ) );
  }
  
  DEALLOC( file_name );
  DEUSE( fn );
  RETURN( unbound );
};
